@@ -4,7 +4,7 @@
 # can do whatever you want with this stuff. If we meet some day, and you think
 # this stuff is worth it, you can buy me a beer in return.   Anton Berezin
 # ----------------------------------------------------------------------------
-# Copyright (c) 2005-2012 SPARTA, Inc.
+# Copyright (c) 2005-2011 SPARTA, Inc.
 # All rights reserved.
 #  
 # Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,7 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
-# $Id: Fast.pm 6427 2012-01-27 04:50:15Z hardaker $
+# $Id: Fast.pm 5853 2011-09-26 17:15:27Z tewok $
 #
 package Net::DNS::ZoneFile::Fast;
 # documentation at the __END__ of the file
@@ -46,7 +46,7 @@ use Net::DNS;
 use Net::DNS::RR;
 use MIME::Base64;
 
-$VERSION = '1.16';
+$VERSION = '1.15';
 
 my $MAXIMUM_TTL = 0x7fffffff;
 
@@ -202,11 +202,7 @@ sub parse
 	    Net::DNS::RR->new_from_hash(%$z);
 
 	  if ($newrec->{'type'} eq 'DNSKEY') {
-	      if (ref($newrec) ne 'Net::DNS::RR::DNSKEY') {
-		  warn "Failed to define a DNSSEC object (got: " . ref($newrec) . "); you're probably missing either MIME::Base64 or MIME::Base32";
-	      } else {
-		  $newrec->setkeytag;
-	      }
+	      $newrec->setkeytag;
 	  }
 
 	  # no longer an issue with recent Net::DNS
@@ -617,33 +613,6 @@ sub parse_line
 	  } else {
 	      error("bad txtdata in $type");
 	  }
-      } elsif (/\G(type[0-9]+)[ \t]+/igc) {
-          my $type = $1;
-	  if (/\G\\#\s+(\d+)\s+\(\s(.*)$/gc) {
-	      # multi-line
-	      $sshfp = { 
-			Line    => $ln,
-			name    => $domain,
-			type    => uc $type,
-			ttl     => $ttl,
-			class   => "IN",
-                        fptype  => $1,
-			fingerprint => $2,
-		       };
-	      $parse = \&parse_sshfp;
-	  } elsif (/\G\\#\s+(\d+)\s+(.*)$pat_skip$/gc) {
-	      push @zone, {
-			   Line    => $ln,
-			   name    => $domain,
-			   type    => uc $type,
-			   ttl     => $ttl,
-			   class   => "IN",
-                           fptype  => $1,
-			   fingerprint => $2,
-			  };
-	  } else {
-	      error("bad data in in $type");
-	  }
       } elsif (/\G(sshfp)[ \t]+/igc) {
 	  if (/\G(\d+)\s+(\d+)\s+\(\s*$/gc) {
 	      # multi-line
@@ -880,9 +849,6 @@ sub parse_line
 	      $ds->{'digest'} .= $1;
 	      $ds->{'digest'} = lc($ds->{'digest'});
 	      $ds->{'digest'} =~ s/\s//g;
-	      # remove any surrounding single line ()s
-	      $ds->{'digest'} =~ s/^\(//;
-	      $ds->{'digest'} =~ s/\)$//;
 	      $ds->{'digestbin'} = pack("H*", $ds->{'digest'});
 	      push @zone, $ds;
 	      $ds = undef;
@@ -1022,7 +988,7 @@ sub parse_line
       } elsif (/\Gany\s+tsig.*$/igc) {
 	  # XXX ignore tsigs
       } else {
-	  error("unrecognized type for $domain\n$_\n");
+	  error("unrecognized type");
       }
   }
 
@@ -1507,7 +1473,7 @@ Copyright 2003 by Anton Berezin and catpipe Systems ApS
 
   Anton Berezin
 
-Copyright (c) 2004-2012 SPARTA, Inc.
+Copyright (c) 2004-2011 SPARTA, Inc.
   All rights reserved.
    
   Redistribution and use in source and binary forms, with or without
