@@ -1,5 +1,5 @@
 #
-# Copyright 2006-2008 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2006-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -25,44 +25,58 @@ our @EXPORT = qw(
 		   dnssec_tools_defnames
 		);
 
-our $VERSION = "1.0";
+our $VERSION = "1.9";
+our $MODULE_VERSION = "1.9.1";
 
-my $installdir = "/usr/bin";		# DNSSEC-Tools installation directory.
+my $installdir = getprefixdir() . "/bin"; # DNSSEC-Tools installation directory.
 
 my %defaults =
 (
 	"admin-email"	   => "root",		# Admin's email address.
+	"archivedir"	   => getprefixdir() . "/var/key-archive",
 	"algorithm"	   => "rsasha1",	# Encryption algorithm.
-	"enddate"	   => "+2592000",	# Zone life, in seconds.
+	"enddate"	   => "+2764800",	# Zone life, in seconds.
 	"entropy_msg"	   => 1,		# Display entropy message flag.
 	"keygen"	   => getprefixdir() . "/sbin/dnssec-keygen",
 	"keygen-opts"	   => "",		# Options for key generator.
 	"kskcount"	   => 1,		# Number of KSK keys.
 	"ksklength"	   => 2048,		# Length of KSK key.
-	"ksklife"	   => 15768000,		# Lifespan of KSK key.
+	"ksklife"	   => 31536000,		# Lifespan of KSK key.
+	"revperiod"	   => 3888000,		# Revocation period.
 	"lifespan-max"	   => 94608000,		# Max lifespan (two years.)
 	"lifespan-min"	   => 3600,		# Min lifespan (one hour.)
 	"random"	   => "/dev/urandom",	# Random no. generator device.
 	"rndc"		   => getprefixdir() . "/sbin/rndc",
-	"roll_logfile"	   => getlocalstatedir() . "/log.rollerd",
+	"roll_loadzone"	   => 1,		# Zone-reloading flag.
+	"roll_logfile"	   => makelocalstatedir() . "/log.rollerd",
 	"roll_loglevel"	   => "phase",		# Rollerd's logging level.
+	"roll_phasemsg"	   => "long",		# Rollerd's phase logmsg length.
 	"roll_sleeptime"   => 3600,		# Rollerd's sleep time.
+	"log_tz"	   => "gmt",		# Log message timezone.
 	"savekeys"	   => 1,		# Save/delete old keys flag.
+	"mailer-server"    => "localhost",	# Mail server.
+	"mailer-type"      => "smtp",		# Mail type.
 	"tacontact" 	   => "",
+	"tatmpdir" 	   => "/tmp/dnssec-tools/trustman",
 	"tadnsvalconffile" => getconfdir() . "/dnsval.conf",
-	"tanamedconffile"  => "/usr/local/etc/named/named.conf",
+	"tanamedconffile"  => getconfdir() . "/named/named.conf",
 	"tasleeptime"      => 3600,
-	"tasmtpserver"     => "",
+	"tasmtpserver"     => "localhost",	# Trustman's SMTP server.
+	"taresolvconf"	   => "/etc/resolv.conf",	# resolv.conf file.
 	"usegui"	   => 0,		# Use GUI for option entry flag.
-	"viewimage"	   => "/usr/X11R6/bin/viewimage",
+#	"viewimage"	   => "/usr/X11R6/bin/viewimage",
+	"zone_errors"	   => 5,
 	"zonecheck"	   => getprefixdir() . "/sbin/named-checkzone",
 	"zonecheck-opts"   => "",		# Options for zone checker.
 	"zonesign"	   => getprefixdir() . "/sbin/dnssec-signzone",
 	"zonesign-opts"	   => "",		# Options for zone signer.
 	"zskcount"	   => 1,		# Number of Current ZSK keys.
 	"zsklength"	   => 1024,		# Length of ZSK key.
-	"zsklife"	   => 604800,		# Lifespan of ZSK key.
-
+	"zsklife"	   => 7884000,		# Lifespan of ZSK key.
+	"usensec3"         => "no",             # Use NSEC3 by default
+	"nsec3iter"	   => 100,              # default NSEC3 iterations
+	"nsec3salt"	   => "random:64",      # default NSEC3 salt
+	"nsec3optout"	   => "no",             # NSEC3 opt-out default
 	"blinkenlights"	   => "$installdir/blinkenlights",
 	"cleanarch"	   => "$installdir/cleanarch",
 	"cleankrf"	   => "$installdir/cleankrf",
@@ -197,6 +211,10 @@ The following are the defaults defined for DNSSEC-Tools.
 This default holds the default email address for the DNSSEC-Tools
 administrator.
 
+=item B<archivedir>
+
+This default holds the default directory in which keys will be archived.
+
 =item B<algorithm>
 
 This default holds the default encryption algorithm.
@@ -232,6 +250,24 @@ This default holds the default lifespan of a KSK key.  This is only used
 for determining when to rollover the KSK key.  Keys otherwise have no
 concept of a lifespan.  This is measured in seconds.
 
+=item B<mailer-server>
+
+The mail server that will be contacted by I<dt_adminmail()>.  This is
+passed to I<Mail::Send>.
+
+=item B<mailer-type>
+
+The type of mailer that will be contacted by I<dt_adminmail()>.  This is
+passed to I<Mail::Mailer> (by way of I<Mail::Send>.)  Any values recognized
+by I<Mail::Mailer> may be used here.
+
+=item B<revperiod>
+
+This default holds the default revocation period of a KSK key. This is
+the minimum period of time a revoked KSK is required to remain in the
+signing set so that it is properly observed by resolvers.  This is
+measured in seconds.
+
 =item B<lifespan-max>
 
 This default is the maximum lifespan of a key.
@@ -240,6 +276,10 @@ This default is the maximum lifespan of a key.
 
 This default is the minimum lifespan of a key.
 
+=item B<log_tz>
+
+This default is the timezone to be used in log-message timestamps.
+
 =item B<random>
 
 This default holds the default random number generator device.
@@ -248,6 +288,11 @@ This default holds the default random number generator device.
 
 This default is the default path of the BIND B<rndc> program.
 
+=item B<roll_loadzone>
+
+This default is flag indicates if B<rollerd> should have the DNS daemon
+reload its zones.
+
 =item B<roll_logfile>
 
 This default is the path to B<rollerd>'s log file.
@@ -266,36 +311,44 @@ daemon.
 This default indicates whether or not keys should be deleted when they are no
 longer in use.
 
-=item B<tanamedconffile>
-
-This default specifies the name of the B<named> configuration file.
-
-=item B<tadnsvalconffile>
-
-This default specifies the name of the B<dnsval> configuration file.
-
-=item B<tasleeptime>
-
-This default holds the default value for how long the daemon should sleep.
-
 =item B<tacontact>
 
 This is merely a placeholder for the contact information. There is no useful
 default value for this.
 
+=item B<tadnsvalconffile>
+
+This default specifies the path of the B<dnsval> configuration file.
+
+=item B<tanamedconffile>
+
+This default specifies the path of the B<named> configuration file.
+
+=item B<taresolvconf>
+
+This default specifies the path to the DNS B<resolv.conf> file.
+
+=item B<tasleeptime>
+
+This default holds the default value for how long the daemon should sleep.
+
 =item B<tasmtpserver>
 
-This is merely a placeholder for the name of the SMTP server. There is no
-useful default value for this.
+This default specifies the name of the SMTP server.
+
+=item B<tatmpdir>
+
+This default specifies the location of B<trustman>'s temporary directory.
 
 =item B<usegui>
 
 This default indicates whether or not the DNSSEC-Tools GUI should be used for
 option entry.
 
-=item B<viewimage>
+=item B<zone_errors>
 
-This default holds the default image viewer.
+This default holds the maximum number of consecutive errors a particular zone
+may have before it is changed to be a I<skip> zone.
 
 =item B<zonecheck>
 
@@ -448,7 +501,7 @@ This default holds the path to the DNSSEC-Tools B<zonesigner> program.
 
 =head1 COPYRIGHT
 
-Copyright 2006-2008 SPARTA, Inc.  All rights reserved.
+Copyright 2006-2011 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
