@@ -1,5 +1,5 @@
 #
-# Copyright 2005-2008 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -28,6 +28,7 @@
 #			--------	---------
 #			FreeBSD		Unix
 #			Mac OSX		Unix
+#			Solaris		Solaris/Unix
 #
 #		When extending the interface or porting this module to another
 #		O/S, the following entities must be modified as described
@@ -95,7 +96,8 @@ use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::defaults;
 use Net::DNS::SEC::Tools::rolllog;
 
-our $VERSION = "0.91";
+our $VERSION = "1.9";
+our $MODULE_VERSION = "1.9.0";
 
 our @ISA = qw(Exporter);
 
@@ -106,7 +108,9 @@ our @EXPORT = qw(
 		 rollmgr_getid
 		 rollmgr_halt
 		 rollmgr_idfile
+		 rollmgr_set_idfile
 		 rollmgr_loadzone
+		 rollmgr_phasemsg
 		 rollmgr_rmid
 		 rollmgr_running
 		 rollmgr_saveid
@@ -116,8 +120,13 @@ our @EXPORT = qw(
 		 rollmgr_getcmd
 		 rollmgr_getresp
 		 rollmgr_sendcmd
+		 rollmgr_queuecmd
+		 rollmgr_getqueueitem
+		 rollmgr_getallqueuedcmds
 		 rollmgr_sendresp
 		 rollmgr_verifycmd
+		 rollmgr_get_phase
+
 			 ROLLCMD_DISPLAY
 			 ROLLCMD_DSPUB
 			 ROLLCMD_DSPUBALL
@@ -125,31 +134,43 @@ our @EXPORT = qw(
 			 ROLLCMD_LOGFILE
 			 ROLLCMD_LOGLEVEL
 			 ROLLCMD_LOGMSG
+			 ROLLCMD_LOGTZ
+			 ROLLCMD_MERGERRFS
+			 ROLLCMD_PHASEMSG
 			 ROLLCMD_ROLLALL
+			 ROLLCMD_ROLLALLZSKS
 			 ROLLCMD_ROLLKSK
 			 ROLLCMD_ROLLREC
+			 ROLLCMD_ROLLZONE
 			 ROLLCMD_ROLLZSK
 			 ROLLCMD_RUNQUEUE
+			 ROLLCMD_QUEUELIST
+			 ROLLCMD_QUEUESTATUS
 			 ROLLCMD_SHUTDOWN
 			 ROLLCMD_SKIPALL
 			 ROLLCMD_SKIPZONE
 			 ROLLCMD_SLEEPTIME
+			 ROLLCMD_SPLITRRF
 			 ROLLCMD_STATUS
 			 ROLLCMD_ZONELOG
 			 ROLLCMD_ZONESTATUS
+			 ROLLCMD_ZSARGS
 
 			 ROLLCMD_RC_OKAY
 			 ROLLCMD_RC_BADLEVEL
 			 ROLLCMD_RC_BADFILE
 			 ROLLCMD_RC_BADSLEEP
 			 ROLLCMD_RC_BADROLLREC
+			 ROLLCMD_RC_BADTZ
 			 ROLLCMD_RC_DISPLAY
 			 ROLLCMD_RC_RRFOPEN
 			 ROLLCMD_RC_NOZONES
 			 ROLLCMD_RC_BADZONE
 			 ROLLCMD_RC_BADZONEDATA
+			 ROLLCMD_RC_BADEVENT
 			 ROLLCMD_RC_KSKROLL
 			 ROLLCMD_RC_ZSKROLL
+			 ROLLCMD_RC_NOARGS
 
 			 CHANNEL_WAIT
 			 CHANNEL_CLOSE
@@ -177,6 +198,7 @@ my $CHANNEL_WAIT	= 0;
 my $CHANNEL_CLOSE	= 1;
 sub CHANNEL_WAIT		{ return($CHANNEL_WAIT);	};
 sub CHANNEL_CLOSE		{ return($CHANNEL_CLOSE);	};
+my @queuedcmds;
 
 #
 # The ROLLCMD_RC_ entities are return codes sent from rollerd and received
@@ -187,25 +209,31 @@ my $ROLLCMD_RC_BADLEVEL		= 1;
 my $ROLLCMD_RC_BADFILE		= 2;
 my $ROLLCMD_RC_BADSLEEP	 	= 3;
 my $ROLLCMD_RC_BADROLLREC	= 4;
-my $ROLLCMD_RC_RRFOPEN		= 5;
-my $ROLLCMD_RC_NOZONES		= 6;
-my $ROLLCMD_RC_BADZONE		= 7;
-my $ROLLCMD_RC_BADZONEDATA	= 8;
-my $ROLLCMD_RC_DISPLAY		= 9;
-my $ROLLCMD_RC_KSKROLL		= 10;
-my $ROLLCMD_RC_ZSKROLL		= 11;
+my $ROLLCMD_RC_BADTZ		= 5;
+my $ROLLCMD_RC_RRFOPEN		= 6;
+my $ROLLCMD_RC_NOZONES		= 7;
+my $ROLLCMD_RC_BADZONE		= 8;
+my $ROLLCMD_RC_BADZONEDATA	= 9;
+my $ROLLCMD_RC_DISPLAY		= 10;
+my $ROLLCMD_RC_KSKROLL		= 11;
+my $ROLLCMD_RC_ZSKROLL		= 12;
+my $ROLLCMD_RC_NOARGS		= 13;
+my $ROLLCMD_RC_BADEVENT		= 14;
 
 sub ROLLCMD_RC_OKAY		{ return($ROLLCMD_RC_OKAY);		};
-sub ROLLCMD_RC_BADLEVEL		{ return($ROLLCMD_RC_BADLEVEL);		};
+sub ROLLCMD_RC_BADEVENT		{ return($ROLLCMD_RC_BADEVENT);		};
 sub ROLLCMD_RC_BADFILE		{ return($ROLLCMD_RC_BADFILE);		};
-sub ROLLCMD_RC_BADSLEEP		{ return($ROLLCMD_RC_BADSLEEP);		};
+sub ROLLCMD_RC_BADLEVEL		{ return($ROLLCMD_RC_BADLEVEL);		};
 sub ROLLCMD_RC_BADROLLREC	{ return($ROLLCMD_RC_BADROLLREC);	};
-sub ROLLCMD_RC_RRFOPEN		{ return($ROLLCMD_RC_RRFOPEN);		};
-sub ROLLCMD_RC_NOZONES		{ return($ROLLCMD_RC_NOZONES);		};
+sub ROLLCMD_RC_BADSLEEP		{ return($ROLLCMD_RC_BADSLEEP);		};
+sub ROLLCMD_RC_BADTZ		{ return($ROLLCMD_RC_BADTZ);		};
 sub ROLLCMD_RC_BADZONE		{ return($ROLLCMD_RC_BADZONE);		};
 sub ROLLCMD_RC_BADZONEDATA	{ return($ROLLCMD_RC_BADZONEDATA);	};
 sub ROLLCMD_RC_DISPLAY		{ return($ROLLCMD_RC_DISPLAY);		};
 sub ROLLCMD_RC_KSKROLL		{ return($ROLLCMD_RC_KSKROLL);		};
+sub ROLLCMD_RC_NOARGS		{ return($ROLLCMD_RC_NOARGS);		};
+sub ROLLCMD_RC_NOZONES		{ return($ROLLCMD_RC_NOZONES);		};
+sub ROLLCMD_RC_RRFOPEN		{ return($ROLLCMD_RC_RRFOPEN);		};
 sub ROLLCMD_RC_ZSKROLL		{ return($ROLLCMD_RC_ZSKROLL);		};
 
 #
@@ -218,19 +246,28 @@ my $ROLLCMD_DSPUBALL	= "rollcmd_dspuball";
 my $ROLLCMD_GETSTATUS	= "rollcmd_getstatus";
 my $ROLLCMD_LOGFILE	= "rollcmd_logfile";
 my $ROLLCMD_LOGLEVEL	= "rollcmd_loglevel";
-my $ROLLCMD_LOGMSG	= "rollcmd_msg";
-my $ROLLCMD_ROLLALL	= "rollcmd_rollallzsks";
+my $ROLLCMD_LOGMSG	= "rollcmd_logmsg";
+my $ROLLCMD_LOGTZ	= "rollcmd_logtz";
+my $ROLLCMD_MERGERRFS	= "rollcmd_mergerrfs";
+my $ROLLCMD_PHASEMSG	= "rollcmd_phasemsg";
+my $ROLLCMD_ROLLALL	= "rollcmd_rollall";
+my $ROLLCMD_ROLLALLZSKS	= "rollcmd_rollallzsks";
 my $ROLLCMD_ROLLKSK	= "rollcmd_rollksk";
 my $ROLLCMD_ROLLREC	= "rollcmd_rollrec";
+my $ROLLCMD_ROLLZONE	= "rollcmd_rollzone";
 my $ROLLCMD_ROLLZSK	= "rollcmd_rollzsk";
 my $ROLLCMD_RUNQUEUE	= "rollcmd_runqueue";
+my $ROLLCMD_QUEUELIST	= "rollcmd_queuelist";
+my $ROLLCMD_QUEUESTATUS	= "rollcmd_queuestatus";
 my $ROLLCMD_SHUTDOWN	= "rollcmd_shutdown";
 my $ROLLCMD_SKIPALL	= "rollcmd_skipall";
 my $ROLLCMD_SKIPZONE	= "rollcmd_skipzone";
 my $ROLLCMD_SLEEPTIME	= "rollcmd_sleeptime";
+my $ROLLCMD_SPLITRRF	= "rollcmd_splitrrf";
 my $ROLLCMD_STATUS	= "rollcmd_status";
 my $ROLLCMD_ZONELOG	= "rollcmd_zonelog";
 my $ROLLCMD_ZONESTATUS	= "rollcmd_zonestatus";
+my $ROLLCMD_ZSARGS	= "rollcmd_zsargs";
 
 sub ROLLCMD_DISPLAY		{ return($ROLLCMD_DISPLAY);	};
 sub ROLLCMD_DSPUB		{ return($ROLLCMD_DSPUB);	};
@@ -239,18 +276,27 @@ sub ROLLCMD_GETSTATUS		{ return($ROLLCMD_GETSTATUS);	};
 sub ROLLCMD_LOGFILE		{ return($ROLLCMD_LOGFILE);	};
 sub ROLLCMD_LOGLEVEL		{ return($ROLLCMD_LOGLEVEL);	};
 sub ROLLCMD_LOGMSG		{ return($ROLLCMD_LOGMSG);	};
+sub ROLLCMD_LOGTZ		{ return($ROLLCMD_LOGTZ);	};
+sub ROLLCMD_MERGERRFS		{ return($ROLLCMD_MERGERRFS);	};
+sub ROLLCMD_PHASEMSG		{ return($ROLLCMD_PHASEMSG);	};
 sub ROLLCMD_ROLLALL		{ return($ROLLCMD_ROLLALL);	};
+sub ROLLCMD_ROLLALLZSKS		{ return($ROLLCMD_ROLLALLZSKS);	};
 sub ROLLCMD_ROLLKSK		{ return($ROLLCMD_ROLLKSK);	};
 sub ROLLCMD_ROLLREC		{ return($ROLLCMD_ROLLREC);	};
+sub ROLLCMD_ROLLZONE		{ return($ROLLCMD_ROLLZONE);	};
 sub ROLLCMD_ROLLZSK		{ return($ROLLCMD_ROLLZSK);	};
 sub ROLLCMD_RUNQUEUE		{ return($ROLLCMD_RUNQUEUE);	};
+sub ROLLCMD_QUEUELIST		{ return($ROLLCMD_QUEUELIST);	};
+sub ROLLCMD_QUEUESTATUS		{ return($ROLLCMD_QUEUESTATUS);	};
 sub ROLLCMD_SHUTDOWN		{ return($ROLLCMD_SHUTDOWN);	};
 sub ROLLCMD_SKIPALL		{ return($ROLLCMD_SKIPALL);	};
 sub ROLLCMD_SKIPZONE		{ return($ROLLCMD_SKIPZONE);	};
 sub ROLLCMD_SLEEPTIME		{ return($ROLLCMD_SLEEPTIME);	};
+sub ROLLCMD_SPLITRRF		{ return($ROLLCMD_SPLITRRF);	};
 sub ROLLCMD_STATUS		{ return($ROLLCMD_STATUS);	};
 sub ROLLCMD_ZONELOG		{ return($ROLLCMD_ZONELOG);	};
 sub ROLLCMD_ZONESTATUS		{ return($ROLLCMD_ZONESTATUS);	};
+sub ROLLCMD_ZSARGS		{ return($ROLLCMD_ZSARGS);	};
 
 my %roll_commands =
 (
@@ -261,19 +307,28 @@ my %roll_commands =
 	rollcmd_logfile		=> 1,
 	rollcmd_loglevel	=> 1,
 	rollcmd_logmsg		=> 1,
+	rollcmd_logtz		=> 1,
+	rollcmd_mergerrfs	=> 1,
 	rollcmd_nodisplay	=> 1,
+	rollcmd_phasemsg	=> 1,
+	rollcmd_rollall		=> 1,
 	rollcmd_rollallzsks	=> 1,
 	rollcmd_rollksk		=> 1,
 	rollcmd_rollrec		=> 1,
+	rollcmd_rollzone	=> 1,
 	rollcmd_rollzsk		=> 1,
 	rollcmd_runqueue	=> 1,
+	rollcmd_queuelist	=> 1,
+	rollcmd_queuestatus	=> 1,
 	rollcmd_shutdown	=> 1,
 	rollcmd_skipall		=> 1,
 	rollcmd_skipzone	=> 1,
 	rollcmd_sleeptime	=> 1,
+	rollcmd_splitrrf	=> 1,
 	rollcmd_status		=> 1,
 	rollcmd_zonelog		=> 1,
 	rollcmd_zonestatus	=> 1,
+	rollcmd_zsargs		=> 1,
 );
 
 ##############################################################################
@@ -286,6 +341,7 @@ my $GETDIR	= "getdir";
 my $GETID	= "getid";
 my $HALT	= "halt";
 my $IDFILE	= "idfile";
+my $SETIDFILE	= "setidfile";
 my $LOADZONE	= "loadzone";
 my $RMID	= "rmid";
 my $RUNNING	= "running";
@@ -298,44 +354,62 @@ my $SAVEID	= "saveid";
 # 
 my %switch_uninit =
 (
-	$CMDINT	  =>	\&uninit_cmdint,
-	$DROPID	  =>	\&uninit_dropid,
-	$GETDIR	  =>	\&uninit_dir,
-	$GETID	  =>	\&uninit_getid,
-	$HALT	  =>	\&uninit_halt,
-	$IDFILE	  =>	\&uninit_idfile,
-	$LOADZONE =>	\&uninit_loadzone,
-	$RMID	  =>	\&uninit_rmid,
-	$RUNNING  =>	\&uninit_running,
-	$SAVEID	  =>	\&uninit_saveid,
+	$CMDINT	   =>	\&uninit_cmdint,
+	$DROPID	   =>	\&uninit_dropid,
+	$GETDIR	   =>	\&uninit_dir,
+	$GETID	   =>	\&uninit_getid,
+	$HALT	   =>	\&uninit_halt,
+	$IDFILE	   =>	\&uninit_idfile,
+	$SETIDFILE =>	\&uninit_set_idfile,
+	$LOADZONE  =>	\&uninit_loadzone,
+	$RMID	   =>	\&uninit_rmid,
+	$RUNNING   =>	\&uninit_running,
+	$SAVEID	   =>	\&uninit_saveid,
 );
 
 my %switch_unknown =
 (
-	$CMDINT	  =>	\&unknown_cmdint,
-	$DROPID	  =>	\&unknown_dropid,
-	$GETDIR	  =>	\&unknown_dir,
-	$GETID	  =>	\&unknown_getid,
-	$HALT	  =>	\&unknown_halt,
-	$IDFILE	  =>	\&unknown_idfile,
-	$LOADZONE =>	\&unknown_loadzone,
-	$RMID	  =>	\&unknown_rmid,
-	$RUNNING  =>	\&unknown_running,
-	$SAVEID	  =>	\&unknown_saveid,
+	$CMDINT	   =>	\&unknown_cmdint,
+	$DROPID	   =>	\&unknown_dropid,
+	$GETDIR	   =>	\&unknown_dir,
+	$GETID	   =>	\&unknown_getid,
+	$HALT	   =>	\&unknown_halt,
+	$IDFILE	   =>	\&unknown_idfile,
+	$SETIDFILE =>	\&unknown_set_idfile,
+	$LOADZONE  =>	\&unknown_loadzone,
+	$RMID	   =>	\&unknown_rmid,
+	$RUNNING   =>	\&unknown_running,
+	$SAVEID	   =>	\&unknown_saveid,
 );
 
 my %switch_unix =
 (
-	$CMDINT	  =>	\&unix_cmdint,
-	$DROPID	  =>	\&unix_dropid,
-	$GETDIR	  =>	\&unix_dir,
-	$GETID	  =>	\&unix_getid,
-	$HALT	  =>	\&unix_halt,
-	$IDFILE	  =>	\&unix_idfile,
-	$LOADZONE =>	\&unix_loadzone,
-	$RMID	  =>	\&unix_rmid,
-	$RUNNING  =>	\&unix_running,
-	$SAVEID	  =>	\&unix_saveid,
+	$CMDINT	   =>	\&unix_cmdint,
+	$DROPID	   =>	\&unix_dropid,
+	$GETDIR	   =>	\&unix_dir,
+	$GETID	   =>	\&unix_getid,
+	$HALT	   =>	\&unix_halt,
+	$IDFILE	   =>	\&unix_idfile,
+	$SETIDFILE =>	\&unix_set_idfile,
+	$LOADZONE  =>	\&unix_loadzone,
+	$RMID	   =>	\&unix_rmid,
+	$RUNNING   =>	\&unix_running,
+	$SAVEID	   =>	\&unix_saveid,
+);
+
+my %switch_sysv =
+(
+	$CMDINT	   =>	\&unix_cmdint,
+	$DROPID	   =>	\&unix_psef_dropid,
+	$GETDIR	   =>	\&unix_dir,
+	$GETID	   =>	\&unix_getid,
+	$HALT	   =>	\&unix_halt,
+	$IDFILE	   =>	\&unix_idfile,
+	$SETIDFILE =>	\&unix_set_idfile,
+	$LOADZONE  =>	\&unix_loadzone,
+	$RMID	   =>	\&unix_rmid,
+	$RUNNING   =>	\&unix_running,
+	$SAVEID	   =>	\&unix_saveid,
 );
 
 
@@ -350,6 +424,7 @@ my %port_archs =
 	"uninitialized"	=>	\%switch_uninit,
 	"unknown"	=>	\%switch_unknown,
 	"unix"		=>	\%switch_unix,
+	"sysv"	 	=>	\%switch_sysv,
 );
 
 
@@ -358,8 +433,8 @@ my %port_archs =
 # Unix-related constants.
 # 
 
-my $UNIX_ROLLMGR_DIR	    = getconfdir();
-my $UNIX_ROLLMGR_PIDFILE = ($UNIX_ROLLMGR_DIR . "/rollmgr.pid");
+my $UNIX_ROLLMGR_DIR	  = makelocalstatedir("run");
+our $UNIX_ROLLMGR_PIDFILE = ($UNIX_ROLLMGR_DIR . "/rollmgr.pid");
 
 my $PS = "/bin/ps";
 
@@ -371,6 +446,49 @@ my $PS = "/bin/ps";
 my $osclass   = "uninitialized";
 my %switchtab = %{$port_archs{$osclass}};
 
+##############################################################################
+#
+# These are textual descriptions of the rolling phases.
+# 
+my @zsk_roll_phases =
+(
+	'Not Rolling',
+	'Waiting for old zone data to expire from caches',
+	'Signing zone with KSK and Published ZSK',
+	'Waiting for old zone data to expire from caches',
+	'Adjusting keys in keyrec and signing zone with New ZSK',
+);
+
+my @ksk_roll_phases =
+(
+	'Not Rolling',
+	'Waiting for old zone data to expire from caches',
+	'Generating new Published KSK',
+	'Waiting for cache or holddown timer expiration',
+	'Rolling the KSK(s)',
+	'Transfer New KSK keyset to parent',
+	'Waiting for parent to publish new DS record',
+	'Reloading the zone',
+);
+
+my $MAXKSK = 7;					# Maximum KSK phase number.
+my $MAXZSK = 4;					# Maximum ZSK phase number.
+
+my %key_phases = 
+(
+	'KSK' => \@ksk_roll_phases,
+	'ZSK' => \@zsk_roll_phases
+);
+
+#
+# Description-length values for rollmgr_get_phase().
+#	NOTE:  These should be dealt with as configuration values.
+#	       However, logging will be changing RSN, and we'll wait a bit.
+#
+my $PHASELONG	= 1;				# Use long descriptions.
+my $PHASESHORT	= 0;				# Use short descriptions.
+# my $longorshort = $PHASELONG;			# Description length to use.
+my $longorshort = $PHASESHORT;			# Description length to use.
 
 ##############################################################################
 ##############################################################################
@@ -383,7 +501,6 @@ my %switchtab = %{$port_archs{$osclass}};
 #
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_prepdep()
 #
 # Purpose:	This routine prepares for device-dependent calls.  A global
@@ -413,16 +530,23 @@ sub rollmgr_prepdep
 		$osclass = "unix";
 	}
 
+	#
+	# Figure out which operating system class we're running on.
+	#
+	if(($osname eq "solaris"))
+	{
+		$osclass = "sysv";
+	}
+
 	#
 	# Get the appropriate switch table for this O/S class and save
 	# it for later reference.
 	#
-	my $swtab = $port_archs{$osclass};
+	$swtab = $port_archs{$osclass};
 	%switchtab = %$swtab;
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_dir()
 #
 # Purpose:	Front-end to the O/S-specific "get rollerd's
@@ -440,7 +564,6 @@ sub rollmgr_dir
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_dropid()
 #
 # Purpose:	Front-end to the O/S-specific "save rollerd's
@@ -458,7 +581,6 @@ sub rollmgr_dropid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_getid()
 #
 # Purpose:	Front-end to the O/S-specific "get rollerd's
@@ -476,7 +598,6 @@ sub rollmgr_getid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_halt()
 #
 # Purpose:	Front-end to the O/S-specific "halt rollerd"
@@ -494,7 +615,6 @@ sub rollmgr_halt
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_idfile()
 #
 # Purpose:	Front-end to the O/S-specific "get rollerd's
@@ -512,7 +632,23 @@ sub rollmgr_idfile
 }
 
 #--------------------------------------------------------------------------
+# Routine:      rollmgr_set_idfile()
 #
+# Purpose:	Front-end to the O/S-specific "set rollerd's
+#		identity filename" function.
+#
+sub rollmgr_set_idfile
+{
+	my @args = shift;			# Routine arguments.
+	my $func;				# Actual function.
+
+# print "rollmgr_set_idfile\n";
+
+	$func = $switchtab{$SETIDFILE};
+	return(&$func(@args));
+}
+
+#--------------------------------------------------------------------------
 # Routine:      rollmgr_loadzone()
 #
 # Purpose:	Front-end to the O/S-specific "load the zone" function.
@@ -529,7 +665,33 @@ sub rollmgr_loadzone
 }
 
 #--------------------------------------------------------------------------
+# Routine:      rollmgr_phasemsg()
 #
+# Purpose:	Set the phase-message length.  Valid values are "long" 
+#		and "short".
+#		This is generic and not O/S-specific.
+#
+sub rollmgr_phasemsg
+{
+	my $pval = shift;			# The new message length.
+
+# print "rollmgr_phasemsg\n";
+
+	if($pval =~ /long/i)
+	{
+		$longorshort = $PHASELONG; 
+		return(1);
+	}
+	elsif($pval =~ /short/i)
+	{
+		$longorshort = $PHASESHORT; 
+		return(1);
+	}
+
+	return(0);
+}
+
+#--------------------------------------------------------------------------
 # Routine:      rollmgr_cmdint()
 #
 # Purpose:	Front-end to the O/S-specific "rollerd has a
@@ -547,7 +709,6 @@ sub rollmgr_cmdint
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_rmid()
 #
 # Purpose:	Front-end to the O/S-specific "remove rollerd's
@@ -565,7 +726,6 @@ sub rollmgr_rmid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_running()
 #
 # Purpose:	Front-end to the O/S-specific "is rollerd running?" function.
@@ -582,7 +742,6 @@ sub rollmgr_running
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      rollmgr_saveid()
 #
 # Purpose:	Front-end to the O/S-specific "save rollerd's
@@ -614,7 +773,6 @@ sub rollmgr_saveid
 #	call the proper O/S-dependent interface.
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_dir()
 #
 # Purpose:	Switch for uninitialized "get dir" command.
@@ -630,7 +788,6 @@ sub uninit_dir
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_dropid()
 #
 # Purpose:	Switch for uninitialized "drop id" command.
@@ -646,7 +803,6 @@ sub uninit_dropid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_getid()
 #
 # Purpose:	Switch for uninitialized "get id" command.
@@ -662,7 +818,6 @@ sub uninit_getid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_halt()
 #
 # Purpose:	Switch for uninitialized "halt" command.
@@ -678,7 +833,6 @@ sub uninit_halt
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_idfile()
 #
 # Purpose:	Switch for uninitialized "get id file" command.
@@ -694,7 +848,21 @@ sub uninit_idfile
 }
 
 #--------------------------------------------------------------------------
+# Routine:      uninit_idfile()
 #
+# Purpose:	Switch for uninitialized "get id file" command.
+#
+sub uninit_set_idfile
+{
+	my @args = shift;			# Routine arguments.
+
+# print "uninit_set_idfile\n";
+
+	rollmgr_prepdep();
+	return(rollmgr_set_idfile(@args));
+}
+
+#--------------------------------------------------------------------------
 # Routine:      uninit_loadzone()
 #
 # Purpose:	Switch for uninitialized "load the zone" command.
@@ -710,7 +878,6 @@ sub uninit_loadzone
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_cmdint()
 #
 # Purpose:	Switch for uninitialized "force queue" command.
@@ -726,7 +893,6 @@ sub uninit_cmdint
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_rmid()
 #
 # Purpose:	Switch for uninitialized "remove id file" command.
@@ -742,7 +908,6 @@ sub uninit_rmid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_running()
 #
 # Purpose:	Switch for uninitialized "is rollerd running?" command.
@@ -758,7 +923,6 @@ sub uninit_running
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      uninit_saveid()
 #
 # Purpose:	Switch for uninitialized "save id file" command.
@@ -786,7 +950,6 @@ sub uninit_saveid
 #
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_action()
 #
 sub unknown_action
@@ -795,7 +958,6 @@ sub unknown_action
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_dir()
 #
 sub unknown_dir
@@ -804,7 +966,6 @@ sub unknown_dir
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_getid()
 #
 sub unknown_getid
@@ -813,7 +974,6 @@ sub unknown_getid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_halt()
 #
 sub unknown_halt
@@ -822,7 +982,6 @@ sub unknown_halt
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_idfile()
 #
 sub unknown_idfile
@@ -831,7 +990,14 @@ sub unknown_idfile
 }
 
 #--------------------------------------------------------------------------
+# Routine:      unknown_set_idfile()
 #
+sub unknown_set_idfile
+{
+	unknown_action();
+}
+
+#--------------------------------------------------------------------------
 # Routine:      unknown_loadzone()
 #
 sub unknown_loadzone
@@ -840,7 +1006,6 @@ sub unknown_loadzone
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_cmdint()
 #
 sub unknown_cmdint
@@ -849,7 +1014,6 @@ sub unknown_cmdint
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_rmid()
 #
 sub unknown_rmid
@@ -858,7 +1022,6 @@ sub unknown_rmid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_running()
 #
 sub unknown_running
@@ -867,7 +1030,6 @@ sub unknown_running
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unknown_saveid()
 #
 sub unknown_saveid
@@ -887,7 +1049,6 @@ sub unknown_saveid
 #
 
 #--------------------------------------------------------------------------
-#
 # Routine:      unix_dir()
 #
 sub unix_dir
@@ -896,7 +1057,6 @@ sub unix_dir
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_idfile()
 #
 # Purpose:	Return rollerd's id file.
@@ -907,7 +1067,16 @@ sub unix_idfile
 }
 
 #--------------------------------------------------------------------------
+# Routine:	unix_set_idfile()
 #
+# Purpose:	Sets rollerd's id file to a particular value
+#
+sub unix_set_idfile
+{
+	$UNIX_ROLLMGR_PIDFILE = $_[0];
+}
+
+#--------------------------------------------------------------------------
 # Routine:	unix_loadzone()
 #
 # Purpose:	Kick the name server so it'll load the given zone.
@@ -923,7 +1092,7 @@ sub unix_loadzone
 	#
 	# Get the path to the name server control program.
 	#
-	$rndc = dnssec_tools_default("bind_rndc") if($rndc eq "");
+	$rndc = dnssec_tools_default("rndc") if($rndc eq "");
 	return(0) if($rndc eq "");
 
 	#
@@ -936,22 +1105,43 @@ sub unix_loadzone
 }
 
 #--------------------------------------------------------------------------
+# Routine:	unix_psef_dropid()
 #
+# Purpose:      Replaces unix_dropid on ps -ef based systems
+#
+sub unix_psef_dropid
+{
+	return unix_dropid('-ps', '-ef', 1, @_);
+}
+
+#--------------------------------------------------------------------------
 # Routine:	unix_dropid()
 #
 # Purpose:	Ensures that another instance of rollerd is not running and
 #		then creates a pid file for future reference.
 #
+# Options:      [-ps FLAGS PIDPOSITION]
+#
 # Return Values:
 #		 1 - The pidfile was initialized for this process.
 #		 0 - Another process (not this one) is already acting as
 #		     rollerd.
+#		-1 - An error was encountered.
 #
 sub unix_dropid
 {
 	my $ego = $$;					# My identity.
 	my $pfpid;					# Pid from the pidfile.
 	my $pspid = -1;					# Pid from ps execution.
+	my $psflags = "wax";
+	my $pidposition = 0;
+
+	if ($_[0] eq '-ps')
+	{
+		shift @_;
+		$psflags = shift @_;
+		$pidposition = shift @_;
+	}
 
 # print "unix_dropid:  down in\n";
 
@@ -963,12 +1153,19 @@ sub unix_dropid
 	#
 	# Create the file if it doesn't exist.
 	# If it does exist, we'll make sure the listed process isn't running.
+	# If we can't create it, we'll complain and return a failure code.
 	#
 	if($pfpid < 0)
 	{
 # print "unix_dropid:  opening $UNIX_ROLLMGR_PIDFILE\n";
+
 		unlink("$UNIX_ROLLMGR_PIDFILE");
-		open(PIDFILE,"> $UNIX_ROLLMGR_PIDFILE") || warn "DROPID UNABLE TO OPEN \"$UNIX_ROLLMGR_PIDFILE\"\n";
+		if(open(PIDFILE,"> $UNIX_ROLLMGR_PIDFILE") == 0)
+		{
+			warn "DROPID UNABLE TO OPEN \"$UNIX_ROLLMGR_PIDFILE\"\n";
+			return(-1);
+		}
+
 		flock(PIDFILE,LOCK_EX);
 	}
 	else
@@ -991,7 +1188,7 @@ sub unix_dropid
 		#	However, the $pfpid seems to be dropped
 		#	when using that method.
 		#
-		$pscmd = "$PS -wax";
+		$pscmd = "$PS $psflags";
 		$openrc = open(PSOUT,"$pscmd |");
 		$psline = <PSOUT>;
 		while(<PSOUT>)
@@ -1007,8 +1204,8 @@ sub unix_dropid
 			# Get the pid from the line and drop out.
 			#
 			s/^[ ]*//;
-			@psout = split / /;
-			$pspid = $psout[0];
+			@psout = split / +/;
+			$pspid = $psout[$pidposition];
 			last;
 		}
 		close(PSOUT);
@@ -1048,8 +1245,8 @@ sub unix_dropid
 	return(1);
 }
 
+
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_rmid()
 #
 # Purpose:	Delete rollerd's pidfile.  This is done as part of the
@@ -1111,7 +1308,6 @@ sub unix_rmid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_running()
 #
 # Purpose:	Determine if rollerd is running and return a boolean
@@ -1153,13 +1349,12 @@ sub unix_running
 	# Check if the pid's procname is rollerd, returning an indicator.
 	#
 	$ret = `$PS -p $rdpid`;
-	return(0) if($ret !~ /rollerd/);
+	return(0) if($ret !~ /(rollerd|perl)/);
 	return(1);
 
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_getpid()
 #
 # Purpose:	Return rollerd, as recorded in its pidfile.
@@ -1223,7 +1418,6 @@ sub unix_getpid
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_cmdint()
 #
 # Purpose:	Kick rollerd to let it know it should re-read the rollrec
@@ -1251,7 +1445,6 @@ sub unix_cmdint
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	unix_halt()
 #
 # Purpose:	Tell rollerd to shut down.
@@ -1282,7 +1475,6 @@ sub unix_halt
 #############################################################################
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_channel()
 #
 # Purpose:	This routine initializes a socket to use for rollerd
@@ -1348,8 +1540,9 @@ sub rollmgr_channel
 		#
 		# Build the socket name and construct the socket data.
 		#
-		$unixsock = getconfdir() . $UNIXSOCK;
+		$unixsock = makelocalstatedir("/dnssec-tools") . $UNIXSOCK;
 # print STDERR "rollmgr_channel:  unixsock - <$unixsock>\n";
+
 		$sockdata = sockaddr_un($unixsock);
 
 		#
@@ -1375,6 +1568,7 @@ sub rollmgr_channel
 			#
 			socket(CLNTSOCK,PF_UNIX,SOCK_STREAM,0)	|| return(-1);
 			connect(CLNTSOCK,$sockdata)		|| return(0);
+
 		}
 	}
 
@@ -1382,7 +1576,6 @@ sub rollmgr_channel
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_closechan()
 #
 # Purpose:	This routine closes down the communications channel to
@@ -1390,13 +1583,61 @@ sub rollmgr_channel
 #
 sub rollmgr_closechan
 {
-
 # print "rollmgr_closechan:  down in\n";
 	close(CLNTSOCK);
 }
 
 #-----------------------------------------------------------------------------
+# Routine:	rollmgr_queuecmd()
 #
+# Purpose:	This routine can be called internally to queue a command
+#		for later processing via calls to rollmgr_getcmd().
+#		It is useful when doing initial start-up before full
+#		processing is to commence.  Commands queued by this
+#		process take precedence over commands received via the
+#		command interface (ie, via rollmgr_sendcmd()).
+#
+sub rollmgr_queuecmd
+{
+	my ($cmd, $value) = @_;
+
+	return(0) if (rollmgr_verifycmd($cmd) == 0);
+	push @queuedcmds, [$cmd, $value];
+}
+
+#-----------------------------------------------------------------------------
+# Routine:	rollmgr_getqueueitem()
+#
+# Purpose:	This routine can be called pull a command from the queue
+#		This is intended to process the item, so it is removed
+#		from the queue.
+#
+sub rollmgr_getqueueitem
+{
+	if($#queuedcmds > -1)
+	{
+		my $cmd = shift @queuedcmds;
+		return($cmd);
+	}
+}
+
+#-----------------------------------------------------------------------------
+# Routine:	rollmgr_getallqueuedcmds()
+#
+# Purpose:	This routine returns all the queued commands in the stack
+#		The items are left in place unless a truthful argument
+#		(e.g. "1") is passed in.
+#
+sub rollmgr_getallqueuedcmds
+{
+	my $removefromqueue = shift;
+	my @results = @queuedcmds;
+
+	@queuedcmds = () if ($removefromqueue);
+	return(@queuedcmds);
+}
+
+#-----------------------------------------------------------------------------
 # Routine:	rollmgr_getcmd()
 #
 # Purpose:	This routine is called by the server to fetch a command and
@@ -1416,6 +1657,13 @@ sub rollmgr_getcmd
 
 # print "rollmgr_getcmd:  down in\n";
 
+	#
+	# if we have anything queued up, process those first.
+	#
+	my $cmdandvalue = rollmgr_getqueueitem();
+	return(@$cmdandvalue)
+	    if(defined($cmdandvalue) && (ref($cmdandvalue) eq 'ARRAY'));
+
 	#
 	# Set a time limit on how long we'll wait for the connection.
 	# Our alarm handler is a dummy, only intended to keep us from
@@ -1479,7 +1727,6 @@ sub rollmgr_getcmd
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_sendcmd()
 #
 # Purpose:	This routine allows a client to send a message to the server.
@@ -1535,7 +1782,6 @@ sub rollmgr_sendcmd
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_sendresp()
 #
 # Purpose:	This routine allows rollerd to send a message to a client.
@@ -1568,7 +1814,6 @@ sub rollmgr_sendresp
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_getresp()
 #
 # Purpose:	This routine allows a client to wait for a message response
@@ -1613,7 +1858,6 @@ sub rollmgr_getresp
 }
 
 #-----------------------------------------------------------------------------
-#
 # Routine:	rollmgr_verifycmd()
 #
 # Purpose:	This routine returns a boolean indicating if the specified
@@ -1633,6 +1877,47 @@ sub rollmgr_verifycmd
 	return(1);
 }
 
+#-----------------------------------------------------------------------------
+# Routine:	rollmgr_get_phase()
+#
+# Purpose:	This routine translates the numerical phases of rolling
+#		keys into textual strings that are better descriptions
+#		for human consumption.
+#
+sub rollmgr_get_phase
+{
+	my ($keytype, $keyphase) = @_;
+	my $maxphase = -1;				# Maximum phase value.
+
+	#
+	# Don't return anything if the installer doesn't want long descriptions.
+	#	NOTE:  This really should be a config value.
+	#
+	if($longorshort != $PHASELONG)
+	{
+		return('');
+	}
+
+	#
+	# Figure out the maximum phase for this key type.
+	#
+	$maxphase = $MAXKSK if($keytype eq 'KSK');
+	$maxphase = $MAXZSK if($keytype eq 'ZSK');
+
+	#
+	# Ensure the phase is within range and is a single digit.
+	#
+	if(($keyphase < 0) || ($keyphase > $maxphase) || ($keyphase !~ /^\d$/))
+	{
+		return('');
+	}
+
+	#
+	# Return the phase description.
+	#
+	return($key_phases{uc($keytype)}[$keyphase]);
+}
+
 1;
 
 #############################################################################
@@ -1664,6 +1949,8 @@ manager.
 
   rollmgr_halt();
 
+  rollmgr_phasemsg('long');
+
   rollmgr_channel(1);
   ($cmd,$data) = rollmgr_getcmd();
   $ret = rollmgr_verifycmd($cmd);
@@ -1673,6 +1960,8 @@ manager.
   rollmgr_sendcmd(CHANNEL_WAIT,ROLLCMD_ROLLZSK,"example.com");
   ($retcode, $respmsg) = rollmgr_getresp();
 
+  $descr = rollmgr_get_phase('KSK', $phasecnt);
+
 =head1 DESCRIPTION
 
 The B<Net::DNS::SEC::Tools::rollmgr> module provides standard,
@@ -1715,6 +2004,7 @@ Return Values:
 
     1 - the id file was successfully created for this process
     0 - another process is already acting as rollerd
+   -1 - unable to create the id file
 
 =item I<rollmgr_rmid()>
 
@@ -1763,6 +2053,26 @@ returned.
     signaled.
     (This should only ever be 1.)
 
+=item I<rollmgr_phasemsg()>
+
+This routine sets the phase-message length.  of phase-related log messages
+used by B<rollerd>.  The valid levels are "long" and "short", with "long"
+being the default value.
+
+The long message length means that a phase description will be included with
+some log messages.  For example, the long form of a message about ZSK rollover
+phase 3 will look like this:  "ZSK phase 3 (Waiting for old zone data to
+expire from caches)".
+
+The short message length means that a phase description will not be included
+with some log messages.  For example, the short form of a message about ZSK
+rollover phase 3 will look like this:  "ZSK phase 3".
+
+Return Values:
+
+     1 - the phase-message length was set
+     0 - an invalid phase-message length was specified
+
 =back
 
 =head1 ROLLERD COMMUNICATIONS INTERFACES
@@ -1778,11 +2088,20 @@ If I<serverflag> is false, then the client's side of the channel is created.
 Currently, the connection may only be made to the localhost.  This may be
 changed to allow remote connections, if this is found to be needed.
 
+=item I<rollmgr_queuecmd(cmdname, value)>
+
+This interface internally remembers a command and it's optional value
+for later processing.  See the I<rollmgr_getcmd()> next for further
+details.
+
 =item I<rollmgr_getcmd()>
 
-I<rollmgr_getcmd()> retrieves a command sent over B<rollerd>'s communications
-channel by a client program.  The command and the command's data are sent in
-each message.
+I<rollmgr_getcmd()> processes commands that need to be dealt with.  If
+there are any internally stored commands queued via the
+I<rollmgr_queuecmd()> function, they are dealt with first.  After that it
+retrieves a command sent over B<rollerd>'s communications channel by a
+client program.  The command and the command's data are sent in each
+message.
 
 The command and the command's data are returned to the caller.
 
@@ -1803,19 +2122,52 @@ The available commands and their required data are:
 					published
    ROLLCMD_DSPUBALL	none		DS records published for all
 					zones in KSK rollover phase 6
-   ROLLCMD_ROLLALL	none		force all zones to start
+   ROLLCMD_GETSTATUS	none		currently unused
+   ROLLCMD_LOGFILE	log filename	change the log file
+   ROLLCMD_LOGLEVEL	log level	set a new logging level
+   ROLLCMD_LOGMSG	log message	add a message to the log
+   ROLLCMD_LOGTZ	timezone	set timezone for log messages
+   ROLLCMD_MERGERRFS	rollrec files	merge rollrec files with the
+					current rollrec file
+   ROLLCMD_PHASEMSG	long/short	set long or short phase
+					messages
+   ROLLCMD_QUEUELIST	none		returns the names and next
+					event time of zones in the
+					"soon queue
+					(experimental)
+   ROLLCMD_QUEUESTATUS	none		returns information about
+  					the state of soon-queue
+  					processing
+					(experimental)
+   ROLLCMD_ROLLALL	none		resume rollover for all
+					suspended zones
+   ROLLCMD_ROLLALLZSKS	none		force all zones to start
 					ZSK rollover
    ROLLCMD_ROLLKSK	zone-name	force a zone to start
 					KSK rollover
    ROLLCMD_ROLLREC	rollrec-name	change rollerd's rollrec file
+   ROLLCMD_ROLLZONE	zone name	restart rollover for a
+					suspended zone
    ROLLCMD_ROLLZSK	zone-name	force a zone to start
 					ZSK rollover
    ROLLCMD_RUNQUEUE	none		rollerd runs through
 					its queue
    ROLLCMD_SHUTDOWN	none		stop rollerd
+   ROLLCMD_SKIPALL	none		suspend all rollovers
+   ROLLCMD_SKIPZONE	zone name	suspend rollover for a
+					rolling zone
    ROLLCMD_SLEEPTIME	seconds-count	set rollerd's sleep time
-   ROLLCMD_STATUS	none		get rollerd's status
-
+   ROLLCMD_SPLITRRF	rollrec-name,	move a set of zones from the
+			zone names	current rollrec file into a
+					new rollrec file
+   ROLLCMD_STATUS	none		get status of rollerd
+   ROLLCMD_ZONELOG	zone name	set the logging level for
+			logging level	a particular zone
+   ROLLCMD_ZONESTATUS	none		get status of the zones
+   ROLLCMD_ZSARGS	zonesigner args	add a (probably temporary)
+			zone list	set of options to the signing
+					of a set of zones
+	
 The data aren't checked for validity by I<rollmgr_sendcmd()>; validity
 checking is a responsibility of B<rollerd>.
 
@@ -1840,6 +2192,17 @@ specific to the command sent.
 I<rollmgr_verifycmd()> verifies that I<cmd> is a valid command for B<rollerd>.
 1 is returned for a valid command; 0 is returned for an invalid command.
 
+1 is returned for a valid command; 0 is returned for an invalid command.
+
+=item I<rollmgr_get_phase(phasetype, phasenum)>
+
+I<rollmgr_get_phase()> returns a description of a particular phase for a
+particular type of rollover.  I<phasetype> specifies the type of rollover,
+and may be "KSK" or "ZSK".  I<phasenum> specifies the phase number whose
+description is desired.  This must be an integer between 0 and 7 (KSK) or 0
+and 4 (ZSK).  If an invalid phase type or phase number is specified, an empty
+string is returned. 
+
 =back
 
 =head1 WARNINGS
@@ -1852,7 +2215,7 @@ the file prior to opening it, but we can't do so without it being open.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2008 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
