@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright 2005-2008 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details.
 #
 # DNSSEC-Tools
@@ -27,7 +27,8 @@ use strict;
 use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::keyrec;
 
-our $VERSION = "0.9";
+our $VERSION = "1.9";
+our $MODULE_VERSION = "1.9.0";
 
 our @ISA = qw(Exporter);
 
@@ -64,10 +65,12 @@ my @stdopts =
 		["random=s",		"Random number generator device"],
 		["endtime=s",		"End-time for signed zone"],
 		["gends",		"Generate DS records"],
+		["nogends",		"Don't generate DS records"],
 
 	'',
 
 	['GUI:separator',	'Configuration Options:'],
+		["dtconfig=s",		"DNSSEC-Tools configuration file"],
 		["keyrec=s",		"Keyrec name",
 		  helpdesc => 'test'
 		],
@@ -86,6 +89,8 @@ my @stdopts =
 		["kskdirectory=s",	"Directory for KSK keys"],
 		["ksklength=i",		"Length of KSK"],
 		["ksklife=i",		"Life-span of KSK"],
+		["revperiod=i",		"Revocation period"],
+		["dsdir=s",		"Directory for dsset files"],
 		["ksdir=s",		"Directory for keyset files"],
 
 	'',
@@ -118,7 +123,9 @@ my @stdopts =
 	['GUI:separator',	'Roll-over Options:'],
 		["roll_logfile=s",	"Roll-over manager's log filename"],
 		["roll_loglevel=s",	"Roll-over manager's logging level"],
+		["logtz=s",		"Roll-over manager's logging timezone"],
 		["roll_sleeptime=i",	"Sleep-time for Roll-over manager"],
+		["zone_errors=i",	"Maximum consecutive errors for a zone"],
 
 	'',
 
@@ -293,14 +300,9 @@ sub opts_zonekr
 	%optionset = dnssec_tools_alldefaults();
 
 	#
-	# Get the config file and mix the file contents in with the defaults.
-	#
+	# Get the config file contents.
 	#
 	%subopts = parseconfig();
-	foreach my $k (sort(keys(%subopts)))
-	{
-		$optionset{$k} = $subopts{$k};
-	}
 
 	#
 	# Set the GUI-usage flag according to the config file.  This must be
@@ -309,10 +311,27 @@ sub opts_zonekr
 	$gui = $optionset{'usegui'};
 
 	#
-	# Mix in the command-line options with all the others.
+	# Get the command-line options.
 	#
 	%cmdopts = opts_int_cmdline(@csopts);
 
+	#
+	# If the user specified a different config file, we'll parse it.
+	#
+	if(defined($cmdopts{'dtconfig'}))
+	{
+		setconffile($cmdopts{'dtconfig'});
+		%subopts = parseconfig();
+	}
+
+	#
+	# Mix the config file contents in with the defaults.
+	#
+	foreach my $k (sort(keys(%subopts)))
+	{
+		$optionset{$k} = $subopts{$k};
+	}
+
 	#
 	# Get the keyrec file (from command line options) and keyrec name
 	# (from command line args) values if the caller didn't give them.
@@ -322,8 +341,8 @@ sub opts_zonekr
 	$krname = $cmdopts{'zone'}	if(defined($cmdopts{'zone'}));
 
 	#
-	# Initialize and read the keyrec file and the specified zone.
-	# We'll only read the 
+	# Initialize (maybe) and read the keyrec file and the specified zone,
+	# putting the data into a hash table.
 	#
 	$optionset{'krfile'} = $krfile;
 	if($krfile ne "")
@@ -558,6 +577,7 @@ sub opts_int_zonecopy
 	#
 	# Read the keyrec file.
 	#
+	keyrec_close();
 	keyrec_read($krfile);
 
 # print "\nopts_int_zonecopy:  down in\n";
@@ -610,7 +630,11 @@ sub opts_int_zonecopy
 	#
 	# Return failure if there's no zone keyrec.
 	#
-	return(undef) if(!$found);
+	if(!$found)
+	{
+		keyrec_close();
+		return(undef);
+	}
 
 	#
 	# Get the zone's keyrec values.
@@ -901,12 +925,12 @@ A reference to the hash table created in these steps is returned to the caller.
 B<dnssec-tools.conf> has these entries:
 
     ksklength      2048
-    zsklength      512
+    zsklength      1024
 
 B<example.keyrec> has this entry:
 
     key         "Kexample.com.+005+12345"
-            zsklength        "1024"
+            zsklength        "2048"
 
 B<zonesigner> is executed with this command line:
 
@@ -914,7 +938,7 @@ B<zonesigner> is executed with this command line:
 
 I<opts_zonekr("example.keyrec","Kexample.com.+005+12345",("wait=i"))>
 will read each option source in turn, ending up with:
-    I<ksklength>           512
+    I<ksklength>          1024
     I<zsklength>          4096
     I<wait>                600
 
@@ -947,6 +971,9 @@ folded in as well, but the key's I<keyrec_> fields are excluded.  This
 call ensures that the named I<keyrec> is a zone I<keyrec>; if it isn't,
 I<undef> is returned.
 
+The I<keyrec> file is reading with I<keyrec_read()>.  To ensure it is
+properly read, I<keyrec_close()> is called first.
+
 The I<$keyrec_file> argument specifies a I<keyrec> file that will be
 consulted.  The I<keyrec> named by the I<$keyrec_name> argument will be
 loaded.  If a I<keyrec> file is found and I<opts_createkrf()> has been
@@ -960,6 +987,10 @@ taken from I<$ARGV[0]>.
 The I<@specopts> array contains command-specific arguments; the arguments must
 be in the format prescribed by the B<Getopt::Long> Perl module.
 
+If the command line contains the I<-dtconfig> option, then I<opts_zonekr>()
+sets that option to be the configuration file.  It then parses that file and
+uses it as the source for configuration file data.
+
 =item I<opts_setcsopts(@csopts)>
 
 This routine saves a copy of the command-specific options given in I<@csopts>.
@@ -996,7 +1027,7 @@ modified by the calling program itself.
 =item I<opts_gui()>
 
 Set an internal flag so that command arguments may be specified with a GUI.
-GUI usage requires that B<Getopt::Long::GUI> is available.  If it isn't, then 
+GUI usage requires that B<Getopt::GUI::Long> is available.  If it isn't, then
 B<Getopt::Long> will be used.
 
 =item I<opts_nogui()>
@@ -1017,7 +1048,7 @@ The old exit action is returned.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2008 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
