@@ -14,44 +14,19 @@
  * drawvalmap.pl script.  This output is written to stderr.
  */
 #include "validator-config.h"
+#include <validator/validator.h>
+#include <validator/resolver.h>
+
+#include "validator_driver.h"
 
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <errno.h>
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
-#include <sys/types.h>
-
-#include <sys/socket.h>
-#include <arpa/nameser.h>
-#ifdef HAVE_ARPA_NAMESER_COMPAT_H
-#include <arpa/nameser_compat.h>
-#else
-#include "arpa/header.h"
-#endif
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <resolv.h>
-
-#include <validator/resolver.h>
-#include <validator/validator.h>
-#include "validator_driver.h"
 
 #define	NAME	"validate"
 #define	VERS	"version: 1.0"
 #define	DTVERS	 "DNSSEC-Tools Version: 1.8"
 
-
-#define NO_OF_THREADS 0
-
-#if NO_OF_THREADS
-#include <pthread.h>
-#endif
-
 #define BUFLEN 16000
 
 int             MAX_RESPCOUNT = 10;
@@ -74,6 +49,7 @@ static struct option prog_options[] = {
     {"testcase", 1, 0, 'T'},
     {"testcase-conf", 1, 0, 'F'},
     {"label", 1, 0, 'l'},
+    {"multi-thread", 1, 0, 'm'},
     {"output", 1, 0, 'o'},
     {"resolv-conf", 1, 0, 'r'},
     {"dnsval-conf", 1, 0, 'v'},
@@ -99,7 +75,7 @@ sig_shutdown(int a)
 
 int
 check_results(val_context_t * context, const char *desc, char * name,
-              const u_int16_t class, const u_int16_t type,
+              const u_int16_t class_h, const u_int16_t type_h,
               const int *result_ar, struct val_result_chain *results,
               int trusted_only)
 {
@@ -203,30 +179,32 @@ print_val_response(struct val_response *resp)
 //
 int
 sendquery(val_context_t * context, const char *desc, char * name,
-          int class, int type, u_int32_t flags,
+          int class_h, int type_h, u_int32_t flags,
           const int *result_ar, int trusted_only,
           struct val_response *resp)
 {
     int             ret_val;
     struct val_result_chain *results = NULL;
     int             err = 0;
+    struct timeval     now, start, duration;
 
     if ((NULL == desc) || (NULL == name) || (NULL == result_ar) )
         return -1;
 
     fprintf(stderr, "%s: ****START**** \n", desc);
     
+    gettimeofday(&start, NULL);
     ret_val =
-        val_resolve_and_check(context, name, class, type, flags, &results);
+        val_resolve_and_check(context, name, class_h, type_h, flags, &results);
 
     if (ret_val == VAL_NO_ERROR) {
 
         if (resp)
-            ret_val = compose_answer(name, type, class, results, resp);
+            ret_val = compose_answer(name, type_h, class_h, results, resp);
 
         if (result_ar)
             err =
-                check_results(context, desc, name, class, type,
+                check_results(context, desc, name, class_h, type_h,
                               result_ar, results, trusted_only);
 
         val_free_result_chain(results);
@@ -235,9 +213,13 @@ sendquery(val_context_t * context, const char *desc, char * name,
         fprintf(stderr, "FAILED: Error in val_resolve_and_check(): %s\n",
                 p_val_err(ret_val));
     }
+    gettimeofday(&now, NULL);
 
     results = NULL;
-    fprintf(stderr, "%s: ****END**** \n", desc);
+
+    timersub(&now, &start, &duration);
+    fprintf(stderr, "%s: ****END**** %ld.%ld sec\n", desc, duration.tv_sec,
+            duration.tv_usec);
 
     return (err != 0);          /* 0 success, 1 error */
 }
@@ -366,7 +348,7 @@ get_results(val_context_t * context, const char *desc, char *name,
      * Query the validator
      */
     ret_val = val_resolve_and_check(context, name, class_h, type_h, 
-                                VAL_QUERY_NO_AC_DETAIL, &results);
+                                    VAL_QUERY_AC_DETAIL, &results);
 
     if (ret_val == VAL_NO_ERROR) {
 
@@ -421,7 +403,7 @@ process_packet(val_context_t *context)
     HEADER         *query_header, *response_header;
     u_char         *pos;
     int             q_name_len, rc;
-    u_int16_t       q_type, q_class;
+    u_int16_t       type_h, class_h;
 
     struct sockaddr from;
     socklen_t       from_len;
@@ -461,13 +443,13 @@ process_packet(val_context_t *context)
     /*
      * get class and type
      */
-    VAL_GET16(q_type, pos);
-    VAL_GET16(q_class, pos);
+    VAL_GET16(type_h, pos);
+    VAL_GET16(class_h, pos);
 
     response_size = sizeof(response);
     
-    get_results(context, "test", (char *)&query[sizeof(HEADER)], (int)q_class,
-                (int)q_type, response, &response_size, 0);
+    get_results(context, "test", (char *)&query[sizeof(HEADER)], (int)class_h,
+                (int)type_h, response, &response_size, 0);
 
     /*
      * check to see if we need a dummy response
@@ -552,7 +534,7 @@ one_test(val_context_t *context, char *name, int class_h,
         FREE(resp.vr_response);
 }
 
-#if NO_OF_THREADS
+#if defined(HAVE_PTHREAD_H) && !defined(VAL_NO_THREADS)
 struct thread_params_st {
     val_context_t *context;
     int tcs;
@@ -579,9 +561,13 @@ struct thread_params_ot {
 void *firethread_st(void *param) {
     struct thread_params_st *threadparams = (struct thread_params_st *)param;
     /*child process*/
+
+#ifndef WIN32
     fprintf(stderr, "Start of thread %u\n context=%u\n", 
             (unsigned int)pthread_self(), 
             (unsigned int)threadparams->context);
+#endif
+
     do {
         self_test(threadparams->context, threadparams->tcs, threadparams->tce, threadparams->flags, 
                   threadparams->testcase_config, threadparams->suite, threadparams->doprint, threadparams->max_in_flight);
@@ -589,8 +575,10 @@ void *firethread_st(void *param) {
             sleep(threadparams->wait);
     }while (threadparams->wait);
     
+#ifndef WIN32
     fprintf(stderr, "End of thread %u\n", 
             (unsigned int)pthread_self());
+#endif
 
     return NULL;
 }
@@ -598,9 +586,13 @@ void *firethread_st(void *param) {
 void *firethread_ot(void *param) {
     struct thread_params_ot *threadparams = (struct thread_params_ot *)param;
     /*child process*/
+
+#ifndef WIN32
     fprintf(stderr, "Start of thread %u\n context=%u\n", 
             (unsigned int)pthread_self(), 
             (unsigned int)threadparams->context);
+#endif
+
     do {
         one_test(threadparams->context, threadparams->name, threadparams->class_h, 
                   threadparams->type_h, threadparams->flags, 
@@ -609,12 +601,47 @@ void *firethread_ot(void *param) {
             sleep(threadparams->wait);
     }while (threadparams->wait);
 
+#ifndef WIN32
     fprintf(stderr, "End of thread %u\n", 
             (unsigned int)pthread_self());
+#endif
     
     return NULL;
 }
-#endif
+
+void
+do_threads(int num_threads, struct thread_params_st *threadparams)
+{
+#define VALIDATOR_MAX_THREADS 100
+    struct thread_params_st *threadparams_copy;
+    pthread_t tids[VALIDATOR_MAX_THREADS];
+    int j;
+
+    if (num_threads > VALIDATOR_MAX_THREADS) {
+        fprintf(stderr, "limiting threads to %d\n", VALIDATOR_MAX_THREADS);
+        num_threads = VALIDATOR_MAX_THREADS;
+    }
+
+    for (j=0; j < num_threads; j++) {
+        threadparams_copy = malloc(sizeof(*threadparams_copy));
+        if (NULL == threadparams_copy)
+            break;
+        memcpy(threadparams_copy, threadparams, sizeof(*threadparams_copy));
+        pthread_create(&tids[j], NULL, firethread_st,
+                       (void *)threadparams_copy);
+    }
+
+    for (j=0; j < num_threads; j++) {
+        pthread_join(tids[j], NULL);
+    }
+}
+#else
+void
+do_threads(int num_threads, struct thread_params_st *threadparams)
+{
+    fprintf(stderr, "Thread support not available\n");
+}
+#endif /* defined(HAVE_PTHREAD_H) && !defined(VAL_NO_THREADS) */
 
 
 
@@ -631,15 +658,16 @@ main(int argc, char *argv[])
     // Parse the command line for a query and resolve+validate it
     int             c;
     char           *domain_name = NULL;
-    const char     *args = "c:dF:hi:I:l:w:o:pr:S:st:T:v:V";
+    const char     *args = "c:dF:hi:I:l:m:w:o:pr:S:st:T:v:V";
     int            class_h = ns_c_in;
     int            type_h = ns_t_a;
     int             success = 0;
     int             doprint = 0;
     int             selftest = 0;
+    int             num_threads = 0;
     int             max_in_flight = 1;
     int             daemon = 0;
-    u_int32_t       flags = 0;
+    u_int32_t       flags = VAL_QUERY_AC_DETAIL;
     int             retvals[] = { 0 };
     int             tcs = 0, tce = -1;
     int             wait = 0;
@@ -733,7 +761,16 @@ main(int argc, char *argv[])
 
 
         case 'I':
+#ifndef VAL_NO_ASYNC
             max_in_flight = strtol(optarg, &nextarg, 10);
+#else
+            fprintf(stderr, "libval was built without asynchronous support\n");
+            fprintf(stderr, "ignoring -I parameter\n");
+#endif /* ndef VAL_NO_ASYNC */
+            break;
+
+        case 'm':
+            num_threads = atoi(optarg);
             break;
 
         case 'v':
@@ -816,29 +853,21 @@ main(int argc, char *argv[])
             goto done;
         } else {
 
-#if NO_OF_THREADS
-            pthread_t tids[NO_OF_THREADS];
-            struct thread_params_st 
-                threadparams = {context, tcs, tce, flags, testcase_config, suite, doprint, wait};
-            int j;
-                
-            for (j=0; j < NO_OF_THREADS; j++) {
-                pthread_create(&tids[j], NULL, firethread_st, (void *)&threadparams);
-            }
-                
-            for (j=0; j < NO_OF_THREADS; j++) {
-                pthread_join(tids[j], NULL);
-            }
-            fprintf(stderr, "Parent exiting\n");
-#else
-            do { /* endless loop */ 
-                rc = self_test(context, tcs, tce, flags, testcase_config, suite,
-                               doprint, max_in_flight);
-                if (wait)
-                    sleep(wait);
-            } while (wait && !rc);
-#endif
+            if (num_threads > 0) {
+                struct thread_params_st 
+                    threadparams = {context, tcs, tce, flags, testcase_config,
+                                    suite, doprint, wait, max_in_flight};
 
+                do_threads(num_threads, &threadparams);
+                fprintf(stderr, "Parent exiting\n");
+            } else {
+                do { /* endless loop */ 
+                    rc = self_test(context, tcs, tce, flags, testcase_config,
+                                   suite, doprint, max_in_flight);
+                    if (wait)
+                        sleep(wait);
+                } while (wait && !rc);
+            }
         }
 
         goto done;
@@ -846,28 +875,22 @@ main(int argc, char *argv[])
 
     domain_name = argv[optind++];
 
-#if NO_OF_THREADS
-    pthread_t tids[NO_OF_THREADS];
-    struct thread_params_ot 
-                threadparams = {context, domain_name, class_h, type_h, flags, retvals, doprint, wait};
-    int j;
-                
-    for (j=0; j < NO_OF_THREADS; j++) {
-        pthread_create(&tids[j], NULL, firethread_ot, (void *)&threadparams);
-    }
-    
-    for (j=0; j < NO_OF_THREADS; j++) {
-        pthread_join(tids[j], NULL);
-    }
-    fprintf(stderr, "Parent exiting\n");
-#else
-    do { /* endless loop */
-        one_test(context, domain_name, class_h, type_h, flags, retvals, doprint);
+    if (num_threads > 0) {
+        struct thread_params_st 
+            threadparams = {context, tcs, tce, flags, testcase_config,
+                            suite, doprint, wait, max_in_flight};
 
-        if (wait)
-            sleep(wait);
-    } while (wait);
-#endif
+        do_threads(num_threads, &threadparams);
+    } else {
+        do { /* endless loop */
+            one_test(context, domain_name, class_h, type_h, flags, retvals,
+                     doprint);
+
+            if (wait)
+                sleep(wait);
+        } while (wait);
+
+    }
 
 done:
     if (context)
