@@ -5,9 +5,11 @@
 #ifndef VALIDATOR_DRIVER_H
 #define VALIDATOR_DRIVER_H
 
+#define MAX_TEST_RESULTS 10
+
 void print_val_response(struct val_response *resp);
 int sendquery(val_context_t * context, const char *desc, char * name,
-              int class, int type, u_int32_t flags,
+              int class_h, int type_h, u_int32_t flags,
               const int *result_ar, int trusted_only,
               struct val_response *resp);
 
