@@ -12,24 +12,8 @@
 */
 
 #include "validator-config.h"
-
-#include <stdio.h>
-#include <unistd.h>
-#include <string.h>
-#include <strings.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <errno.h>
-#include <ctype.h>
-#include <sys/file.h>
-#include <sys/fcntl.h>
-
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <resolv.h>
-
+#include <validator/validator.h>
 #include <validator/resolver.h>
-#include <validator/val_errors.h>
 #include "validator_driver.h"
 
 typedef struct testcase_st {
@@ -38,7 +22,9 @@ typedef struct testcase_st {
     int                 qc;
     int                 qt;
     int                 qr[MAX_TEST_RESULTS];
+#ifndef VAL_NO_ASYNC
     val_async_status   *as;
+#endif
     struct val_response resp;
     struct testcase_st *next;
 } testcase;
@@ -51,7 +37,6 @@ typedef struct testsuite_st {
     struct testsuite_st *next;
 } testsuite;
 
-static testsuite * testsuite_head = NULL;
 
 extern int             
 val_get_token(char **buf_ptr,
@@ -63,13 +48,13 @@ val_get_token(char **buf_ptr,
               int ignore_space);
 
 static void
-selftest_cleanup(void)
+selftest_cleanup(testsuite *head)
 {
     testsuite *tmp_s;
     testcase  *tmp_c;
 
-    while (NULL != testsuite_head) {
-        tmp_s = testsuite_head;
+    while (NULL != head) {
+        tmp_s = head;
         FREE(tmp_s->name);
         while (NULL != (tmp_c = tmp_s->head)) {
             FREE(tmp_c->desc);
@@ -77,7 +62,7 @@ selftest_cleanup(void)
             tmp_s->head = tmp_c->next;
             FREE(tmp_c);
         }
-        testsuite_head = tmp_s->next;
+        head = tmp_s->next;
         FREE(tmp_s);
     }
 }
@@ -208,16 +193,17 @@ vtc_parse_result(const char *result)
         result += 4;
 
     for (i = 0; rm[i].name; ++ i)
-        if (0 == strcasecmp(result, rm[i].name))
+        if (strlen(result) == strlen(rm[i].name) &&
+            0 == strncasecmp(result, rm[i].name, strlen(rm[i].name)))
             return rm[i].val;
 
     return 0;
 }
 
 static testsuite *
-find_suite(const char *name)
+find_suite(testsuite *head, const char *name)
 {
-    testsuite      *curr_suite = testsuite_head;
+    testsuite      *curr_suite = head;
 
     for(; NULL != curr_suite; curr_suite = curr_suite->next)
         if (0 == strcmp(name, curr_suite->name))
@@ -230,26 +216,35 @@ find_suite(const char *name)
  * Make sense of the validator testcase configuration file
  */
 int
-read_val_testcase_file(const char *filename)
+read_val_testcase_file(const char *filename, testsuite **head)
 {
     int             fd;
+#ifdef HAVE_FLOCK
     struct flock    fl;
+#endif
     int             retval = VAL_NO_ERROR, endst = 0;
     char            token[1025];
     int             line_number = 1;
-    testsuite      *curr_suite = NULL;
+    testsuite      *curr_suite = NULL, *testsuite_head = NULL;
     testcase       *tmp_case = NULL, *tail = NULL;
     char *buf_ptr, *end_ptr;
     char *buf = NULL;
     struct stat sb;
    
+    if (NULL == filename || NULL == head)
+        return VAL_BAD_ARGUMENT;
+
+    *head = NULL;
+
     fd = open(filename, O_RDONLY);
     if (fd == -1) {
         return VAL_CONF_NOT_FOUND;
     }
+#ifdef HAVE_FLOCK
     memset(&fl, 0, sizeof(fl));
     fl.l_type = F_RDLCK;
     fcntl(fd, F_SETLKW, &fl);
+#endif
 
     if (0 != fstat(fd, &sb)) {
         retval = VAL_CONF_NOT_FOUND;
@@ -296,7 +291,7 @@ read_val_testcase_file(const char *filename)
         if ((NULL != pos) && (0 == pos[1])) {
             *pos = 0;
 
-            curr_suite = find_suite(token);
+            curr_suite = find_suite(testsuite_head, token);
             if (NULL == curr_suite) {
                 /** new suite */
                 curr_suite = calloc(1, sizeof(*curr_suite));
@@ -447,8 +442,10 @@ read_val_testcase_file(const char *filename)
 #endif
 
 err:
+#ifdef HAVE_FLOCK
     fl.l_type = F_UNLCK;
     fcntl(fd, F_SETLKW, &fl);
+#endif
     close(fd);
     if (buf)
         free(buf);
@@ -459,6 +456,8 @@ err:
         exit(2);
     }
 
+    *head = testsuite_head;
+
     return retval;
 }
 
@@ -496,23 +495,24 @@ run_suite(val_context_t *context, testcase *curr_test, int tcs, int tce,
     return run;
 }
 
+#ifndef VAL_NO_ASYNC
 int
-suite_async_callback(val_async_status *as)
+suite_async_callback(val_async_status *async_status, int event,
+                     val_context_t *ctx, void *cb_data, val_cb_params_t *cbp)
 {
     testsuite *suite;
-    if (NULL == as || NULL == as->val_as_cb_user_ctx) {
-        val_log(as->val_as_ctx, LOG_ERR, "bad parameter for callback\n");
+    if ((NULL == cb_data) || (NULL == cbp)) {
+        val_log(ctx, LOG_ERR, "bad parameter for callback");
         return VAL_BAD_ARGUMENT;
     }
 
-    suite = (testsuite *)as->val_as_cb_user_ctx;
-
+    suite = (testsuite *)cb_data;
     --suite->in_flight;
     --suite->remaining;
 
-    val_log(as->val_as_ctx, LOG_INFO,
-            "query completed; %d in flight, %d remaining\n",
-            suite->in_flight, suite->remaining);
+    val_log(ctx, LOG_INFO,
+            "as 0x%x %s query completed; %d in flight, %d remaining",
+            async_status, cbp->name, suite->in_flight, suite->remaining);
     return VAL_NO_ERROR;
 }
 
@@ -521,7 +521,7 @@ run_suite_async(val_context_t *context, testsuite *suite, testcase *start_test,
                 int tcs, int tce, u_int32_t flags, int *failed, int doprint,
                 int max_in_flight)
 {
-    int i, j, rc, run = 0, burst = 5, nfds, unsent, ready;
+    int i, j, rc, run = 0, burst = max_in_flight, nfds, unsent, ready;
     fd_set             activefds;
     struct timeval     timeout, now;
     testcase          *curr_test = start_test;
@@ -541,16 +541,17 @@ run_suite_async(val_context_t *context, testsuite *suite, testcase *start_test,
                  j < burst &&
                  curr_test;
              ++i, ++j, curr_test = curr_test->next) {
+            val_log(context, LOG_DEBUG, "starting test %i (max %d) %s", i, tce,
+                    curr_test->desc);
+            memset(&curr_test->resp, 0, sizeof(curr_test->resp));
             rc = val_async_submit(context, curr_test->qn, curr_test->qc,
-                                  curr_test->qt, flags, &curr_test->as);
+                                  curr_test->qt, flags, &suite_async_callback,
+                                  suite, &curr_test->as);
             if ((rc != VAL_NO_ERROR) || (!curr_test->as)) {
-                val_log(context, LOG_ERR, "error sending %i %s\n", i,
+                val_log(context, LOG_ERR, "error sending %i %s", i,
                         curr_test->desc);
                 continue;
             }
-            memset(&curr_test->resp, 0, sizeof(curr_test->resp));
-            curr_test->as->val_as_result_cb = &suite_async_callback;
-            curr_test->as->val_as_cb_user_ctx = suite;
             ++run;
             ++suite->in_flight;
         }
@@ -563,9 +564,20 @@ run_suite_async(val_context_t *context, testsuite *suite, testcase *start_test,
         val_async_select_info(context, &activefds, &nfds, &timeout);
         if (0 == nfds) {
             val_log(context, LOG_DEBUG,
-                    "no file decsriptors set for requests\n");
+                    "no file descriptors set! (%d unsent, %d inflight)",
+                    unsent, suite->in_flight);
+            /* maybe socket got closed, need to send next request */
             rc = val_async_check(context, &activefds, &nfds, 0);
             val_async_select_info(context, &activefds, &nfds, &timeout);
+            if (0 == nfds) {
+                if (suite->in_flight && !unsent) {
+                    val_log(context, LOG_DEBUG,
+                            "**** no file descriptors set! (%d unsent, %d inflight)",
+                            unsent, suite->in_flight);
+                    break;
+                }
+                continue;
+            }
         }
         /** adjust libsres absolute timeout to select relative timeout */
         if (timeout.tv_sec == LONG_MAX)
@@ -581,37 +593,32 @@ run_suite_async(val_context_t *context, testsuite *suite, testcase *start_test,
         /** don't sleep too long if more queries are waiting to be sent */
         if (unsent && suite->in_flight < max_in_flight && timeout.tv_sec > 0) {
             val_log(context, LOG_DEBUG,
-                    "reducing timeout so we can send more requests\n");
+                    "reducing timeout so we can send more requests");
             timeout.tv_sec = 0;
             timeout.tv_usec = 500;
         }
         val_log(context, LOG_INFO,
-                "select @ %d, %d fds, timeout %ld, %d in flight, %d unsent\n",
-               now.tv_sec, nfds, timeout.tv_sec, suite->in_flight, unsent);
-        if ((nfds > 0) && (val_log_debug_level() >= LOG_DEBUG)) {
-            val_log(context, LOG_DEBUG, "activefds: ");
-            i = getdtablesize();
-            if (i > FD_SETSIZE)
-                i = FD_SETSIZE;
-            for (--i; i >= 0; --i)
-                if (FD_ISSET(i, &activefds)) {
-                    val_log(context, LOG_DEBUG, "%d ", i);
-                }
-            val_log(context, LOG_DEBUG, "\n");
-        }
+                "select @ %d, max fd %d, timeout %ld.%ld, %d in flight, %d unsent",
+                now.tv_sec, nfds, timeout.tv_sec, now.tv_usec,
+                suite->in_flight, unsent);
+        if ((nfds > 0) && (val_log_debug_level() >= LOG_DEBUG))
+            res_io_count_ready(&activefds, nfds); // debug
 
         fflush(stdout);
         ready = select(nfds, &activefds, NULL, NULL, &timeout);
         gettimeofday(&now, NULL);
-        val_log(context, LOG_DEBUG, "%d fds @ %d\n", ready, now.tv_sec);
+        val_log(context, LOG_DEBUG, "%d fds ready @ %ld.%ld", ready,
+                now.tv_sec, now.tv_usec);
+        if ((ready > 0) && (val_log_debug_level() >= LOG_DEBUG))
+            res_io_count_ready(&activefds, nfds); // debug
         if (ready < 0 && errno == EINTR)
             continue;
 
         /** no ready fd; check for timeouts/retries */
         if (ready == 0) {
             gettimeofday(&now, NULL);
-            now.tv_usec = 0;
-            val_log(context, LOG_DEBUG, "timeout @ %ld\n", now.tv_sec);
+            val_log(context, LOG_DEBUG, "timeout @ %ld.%ld", now.tv_sec,
+                    now.tv_usec);
         }
 
         rc = val_async_check(context, &activefds, &nfds, 0);
@@ -620,6 +627,7 @@ run_suite_async(val_context_t *context, testsuite *suite, testcase *start_test,
 
     return run;
 }
+#endif /* ndef VAL_NO_ASYNC */
 
 int
 run_test_suite(val_context_t *context, int tcs, int tce, u_int32_t flags,
@@ -658,10 +666,12 @@ run_test_suite(val_context_t *context, int tcs, int tce, u_int32_t flags,
     fprintf(stderr, "Suite '%s': Running %d tests\n", suite->name, tc_count);
 
     gettimeofday(&start, NULL);
+#ifndef VAL_NO_ASYNC
     if (max_in_flight > 1)
         run_cnt = run_suite_async(context, suite, start_test, tcs, tce, flags,
                                   &failed, doprint, max_in_flight);
     else
+#endif /* ndef VAL_NO_ASYNC */
         run_cnt = run_suite(context, start_test, tcs, tce, flags, &failed,
                             doprint);
 
@@ -687,17 +697,14 @@ self_test(val_context_t *context, int tcs, int tce, u_int32_t flags,
           const char *tests_file, const char *suites, int doprint,
           int max_in_flight)
 {
-    testsuite *suite;
+    testsuite *suite, *head;
     int rc;
 
-    if (NULL == testsuite_head) {
-        if (NULL == tests_file)
-            tests_file = VALIDATOR_TESTCASES;
-        read_val_testcase_file(tests_file);
-        atexit(selftest_cleanup);
-    }
+    if (NULL == tests_file)
+        tests_file = VALIDATOR_TESTCASES;
+    read_val_testcase_file(tests_file, &head);
 
-    suite = testsuite_head;
+    suite = head;
 
     if (NULL == suites) {
 
@@ -718,7 +725,7 @@ self_test(val_context_t *context, int tcs, int tce, u_int32_t flags,
                 *next++ = 0;
 
 
-            suite = find_suite(name);
+            suite = find_suite(head, name);
             if (NULL == suite)
                 fprintf(stderr, "unknown suite %s\n", name);
             else {
@@ -732,5 +739,7 @@ self_test(val_context_t *context, int tcs, int tce, u_int32_t flags,
         free(name_save);
     }
 
+    selftest_cleanup(head);
+
     return 0;
 }
