@@ -6,8 +6,9 @@
 #ifndef VAL_ERRORS_H
 #define VAL_ERRORS_H
 
-#include <validator/resolver.h>
-#include <validator/validator.h>
+#ifdef __cplusplus
+extern          "C" {
+#endif
 
 #define VAL_FLAG_CHAIN_COMPLETE 0x80
 #define VAL_MASKED_FLAG_CHAIN_COMPLETE 0x7f
@@ -73,7 +74,7 @@
 #define VAL_AC_PINSECURE            (VAL_AC_DONT_GO_FURTHER+2)
 #define VAL_AC_BARE_RRSIG           (VAL_AC_DONT_GO_FURTHER+3) 
 #define VAL_AC_NO_LINK              (VAL_AC_DONT_GO_FURTHER+4)
-#define VAL_AC_TRUST_ANCHOR         VAL_AC_NO_LINK 
+#define VAL_AC_TRUST_ANCHOR         VAL_AC_NO_LINK  /* backwards compatibility */
 #define VAL_AC_TRUST                (VAL_AC_DONT_GO_FURTHER+5) 
 #define VAL_AC_LAST_STATE           VAL_AC_TRUST
 
@@ -192,5 +193,8 @@
 #define VAL_VALIDATED_ANSWER            ((15) | VAL_FLAG_CHAIN_COMPLETE)
 #define VAL_UNTRUSTED_ANSWER            ((16) | VAL_FLAG_CHAIN_COMPLETE)
 
+#ifdef __cplusplus
+}                               /* extern "C" */
+#endif
 
 #endif                          /* VAL_ERRORS_H */
