@@ -6,264 +6,114 @@
 #ifndef VALIDATOR_H
 #define VALIDATOR_H
 
+#include <stdio.h>
+#include <stdarg.h>
+#include <stddef.h>
+
+/*
+ * XXX fd_set cannot be forward declared
+ */
+#ifndef WIN32
+#include <sys/select.h>
+#endif
+
+#include <validator/val_errors.h>
+
 #ifdef __cplusplus
 extern          "C" {
 #endif
 
-#include <stdio.h>
-#include <syslog.h>
-#include <stdarg.h>
-#include <validator/val_errors.h>
-
-#include <arpa/nameser.h>
-#include <netdb.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-
 /* 
- * XXX FreeBSD no longer defines EAI_NODATA. Need to figure out why 
- * this is so. Following is a temporary fix.
+ * Application is responsible for pointing to the definitions of these types 
+ * if it plans on using them 
  */
-#if !defined(EAI_NODATA) && (EAI_NONAME == 8)
-#define EAI_NODATA 7
-#endif
+struct hostent;
+struct addrinfo;
+struct sockaddr;
+struct timeval;
 
 /*
- * define error codes for val_getaddrinfo and val_getnameinfo which
- * have a DNSSEC validation status.
+ * Section values of an RRset 
  */
-#if defined(EAI_NODATA)
-#define VAL_GETADDRINFO_HAS_STATUS(rc) ( \
-	(rc == 0) || (rc == EAI_NONAME) || (rc == EAI_NODATA))
-#else
-#define VAL_GETADDRINFO_HAS_STATUS(rc) ((rc == 0) || (rc == EAI_NONAME))
-#endif
-
-#define VAL_GETNAMEINFO_HAS_STATUS(rc) VAL_GETADDRINFO_HAS_STATUS(rc)
-
-#if !defined(NS_INT16SZ) && defined(INT16SZ)
-#define NS_INT16SZ INT16SZ
-#define NS_INT32SZ INT32SZ
-#endif
-
-#define VAL_GET16(s, cp) do { \
-        register const u_char *t_cp = (const u_char *)(cp); \
-        (s) = ((u_int16_t)t_cp[0] << 8) \
-            | ((u_int16_t)t_cp[1]) \
-            ; \
-        (cp) += NS_INT16SZ; \
-} while (0)
-
-#define VAL_GET32(l, cp) do { \
-        register const u_char *t_cp = (const u_char *)(cp); \
-        (l) = ((u_int32_t)t_cp[0] << 24) \
-            | ((u_int32_t)t_cp[1] << 16) \
-            | ((u_int32_t)t_cp[2] << 8) \
-            | ((u_int32_t)t_cp[3]) \
-            ; \
-        (cp) += NS_INT32SZ; \
-} while (0)
-
-#if !defined(NS_PUT16) && defined(PUTSHORT)
-#define NS_PUT16 PUTSHORT
-#define NS_PUT32 PUTLONG
-#endif
-
-#ifdef MEMORY_DEBUGGING
-#define MALLOC(s) my_malloc(s, __FILE__, __LINE__)
-#define FREE(p) my_free(p,__FILE__,__LINE__)
-#define STRDUP(p) my_strdup(p,__FILE__,__LINE__)
-#else
-#define MALLOC(s) malloc(s)
-#define FREE(p) free(p)
-#define STRDUP(p) strdup(p)
-#endif
-
-/* various constants */
-#define DNS_PORT                53
-#define VAL_LOG_OPTIONS LOG_PID
-#define VALIDATOR_LOG_PORT 1053
-#define VALIDATOR_LOG_SERVER "127.0.0.1"
-#define VAL_DEFAULT_RESOLV_CONF "/etc/resolv.conf"
-#define VAL_CONTEXT_LABEL "VAL_CONTEXT_LABEL"
-#define VAL_LOG_TARGET "VAL_LOG_TARGET"
-
-    /*
-     * Query states 
-     */
-#define Q_INIT          1
-#define Q_SENT          2
-#define Q_WAIT_FOR_GLUE 3
-#define Q_ANSWERED      4
-#define Q_ERROR_BASE    5
-#define Q_QUERY_ERROR (Q_ERROR_BASE + 0) 
-#define Q_RESPONSE_ERROR (Q_ERROR_BASE + 1) 
-#define Q_WRONG_ANSWER (Q_ERROR_BASE + 2) 
-#define Q_REFERRAL_ERROR (Q_ERROR_BASE + 3) 
-#define Q_MISSING_GLUE (Q_ERROR_BASE + 4) 
-#define Q_CONFLICTING_ANSWERS (Q_ERROR_BASE + 5) 
-
-
-#define QUERY_BAD_CACHE_THRESHOLD 5
-#define QUERY_BAD_CACHE_TTL 60
-
-    /*
-     * Credibility values of an RRset - from DNSIND-Clarify 
-     */
-#define SR_CRED_UNSET            0
-#define SR_CRED_FILE             1      /* From locally trusted file */
-    /*
-     * Data is from an authoritative server 
-     */
-#define SR_CRED_AUTH_ANS         3
-#define SR_CRED_AUTH_AUTH        4
-    /*
-     * Data is from a cache somewhere, or was at best an after thought 
-     */
-#define SR_CRED_NONAUTH_ANS      6
-#define SR_CRED_AUTH_ADD         7
-#define SR_CRED_NONAUTH_AUTH     7
-#define SR_CRED_NONAUTH_ADD      7
-
-    /*
-     * Kinds of answers 
-     */
-#define SR_ANS_UNSET             0
-#define SR_ANS_STRAIGHT          1
-#define SR_ANS_CNAME             2
-#define SR_ANS_DNAME             3
-#define SR_ANS_NACK              4
-#define SR_ANS_BARE_RRSIG        5
-
-
-    /*
-     * Policies associated with Keys 
-     */
-#define CANNOT_BE_USED                  0x00
-#define CAN_SIGN_KEY                    0x01
-#define CAN_SIGN_ZONE                   0x02
-#define CAN_SIGN_ZONE_AND_KEY   CAN_SIGN_KEY|CAN_SIGN_ZONE
-
-#define SIGNBY              18
-#define ENVELOPE            10
-#define RRSIGLABEL           3
-#define TTL                  4
-#define VAL_CTX_IDLEN       20
-#ifndef SHA_DIGEST_LENGTH
-#define SHA_DIGEST_LENGTH 20
-#endif
-#ifndef SHA256_DIGEST_LENGTH
-#define SHA256_DIGEST_LENGTH 32 
-#endif
-#ifndef SHA512_DIGEST_LENGTH
-#define SHA512_DIGEST_LENGTH 64 
-#endif
-#define MAX_DIGEST_LENGTH 64
-
-    /*
-     * Algorithm definitions for DS digest 
-     */
-#define ALG_DS_HASH_SHA1 1
-#define ALG_DS_HASH_SHA256 2
-
-    /*
-     * Algorithm definitions for NSEC3 digest 
-     */
-#ifdef LIBVAL_NSEC3
-#define ALG_NSEC3_HASH_SHA1 1
-#endif
-
-    /*
-     * DNSSEC Algorithm definitions 
-     */
-#define ALG_RSAMD5  1
-#define ALG_DH      2
-#define ALG_DSASHA1 3
-#define ALG_RSASHA1 5
-#ifdef LIBVAL_NSEC3
-#define ALG_NSEC3_DSASHA1 6 
-#define ALG_NSEC3_RSASHA1 7 
-#endif
-#define ALG_RSASHA256 8
-#define ALG_RSASHA512 10 
-    /*
-     * Section values of an RRset 
-     */
 #define VAL_FROM_UNSET            0
 #define VAL_FROM_ANSWER           1
 #define VAL_FROM_AUTHORITY        2
 #define VAL_FROM_ADDITIONAL       3
 #define VAL_FROM_QUERY            4
 
+
+/*
+ * Bitmasks for various sets of query flags 
+ */
 #define VAL_QFLAGS_ANY          0xffffffff
-#define VAL_QFLAGS_CACHE_MASK   0x00ffffff
-#define VAL_QFLAGS_USERMASK     0x0000ffff
+/*
+ * Flags in this bit mask are checked when finding
+ * a matching record in the query cache 
+ */
+#define VAL_QFLAGS_CACHE_MASK       0x0000ffff
+#define VAL_QUERY_AC_DETAIL         0x00000001
+#define VAL_QUERY_DONT_VALIDATE     0x00000002 
+#define VAL_QUERY_NO_DLV            0x00000004 
+#define VAL_QUERY_USING_DLV         0x00000008 
+#define VAL_QUERY_ASYNC             0x00000010
+#define VAL_QUERY_NO_EDNS0_FALLBACK 0x00000020
+#define VAL_QUERY_SKIP_CACHE        0x00000080
+#define VAL_QUERY_SKIP_RESOLVER     0x00000100
 
-#define VAL_QUERY_NO_AC_DETAIL  0x00000001
-#define VAL_QUERY_DONT_VALIDATE 0x00000002 
-#define VAL_QUERY_NO_DLV        0x00000004 
-#define VAL_QUERY_REFRESH_QCACHE 0x00000008
-#define VAL_QUERY_ASYNC         0x00000010
+/*
+ * Flags in this bit mask are not checked when finding
+ * a matching record in the query cache 
+ */
+#define VAL_QFLAGS_NOCACHE_MASK     0xffff0000
+#define VAL_QUERY_EDNS0_FALLBACK    0x00010000 
+#define VAL_QUERY_GLUE_REQUEST      0x00020000
+#define VAL_QUERY_REFRESH_QCACHE    0x00040000
+#define VAL_QUERY_RECURSE           0x00080000
+#define VAL_QUERY_NO_EDNS0          0x00100000
 
-#define VAL_QUERY_NO_EDNS0      0x00010000
-#define VAL_QUERY_USING_DLV     0x00020000 
+#define VAL_QFLAGS_USERMASK (VAL_QUERY_AC_DETAIL |\
+                             VAL_QUERY_DONT_VALIDATE |\
+                             VAL_QUERY_NO_DLV |\
+                             VAL_QUERY_ASYNC |\
+                             VAL_QUERY_NO_EDNS0_FALLBACK |\
+                             VAL_QUERY_SKIP_CACHE |\
+                             VAL_QUERY_SKIP_RESOLVER |\
+                             VAL_QUERY_RECURSE |\
+                             VAL_QUERY_REFRESH_QCACHE)
 
-#define VAL_QUERY_GLUE_REQUEST (0x01000000 | VAL_QUERY_DONT_VALIDATE)
+#ifndef LOG_EMERG
+#define LOG_EMERG 0
+#define LOG_ALERT 1
+#define LOG_CRIT 2
+#define LOG_ERR 3
+#define LOG_WARNING 4
+#define LOG_NOTICE 5
+#define LOG_INFO 6
+#define LOG_DEBUG 7
+#endif
 
-#define MAX_ALIAS_CHAIN_LENGTH 10       /* max length of cname/dname chain */
-#define MAX_GLUE_FETCH_DEPTH 10         /* max length of glue dependency chain */
+/* validator return types */
+typedef unsigned char val_status_t;
+typedef unsigned short val_astatus_t;
 
-    typedef u_int8_t val_status_t;
-    typedef u_int16_t val_astatus_t;
+/* opaque types */
+struct val_context;
+typedef struct val_context val_context_t;
+struct val_policy_handle;
+typedef struct val_policy_handle val_policy_handle_t;
+struct val_log;
+typedef struct val_log val_log_t;
+struct queries_for_query;
 
-    struct val_log;             /* forward declaration */
-    struct val_context;         /* forward declaration */
-    typedef struct val_context val_context_t;
-
-    typedef struct policy_entry {
-        u_char        zone_n[NS_MAXCDNAME];
-        long            exp_ttl;
-        void *          pol;
-        struct policy_entry *next;
-    } policy_entry_t;
-  
-    typedef struct global_opt {
-        int local_is_trusted;
-        long edns0_size;    
-        int env_policy;
-        int app_policy;
-        char *log_target;
-        int closest_ta_only;
-    } global_opt_t;
-
-    typedef struct {
-        policy_entry_t *pe;
-        int index;
-    }  val_policy_handle_t;
-
-    typedef struct {
-        char *keyword;
-        char *zone;
-        char *value;
-        long ttl;
-    } libval_policy_definition_t;
-
-    /*
-     * The above is a generic data type for a policy entry
-     * typecasted to one of the types defined in val_policy.h: 
-     * Policies can be one of the following
-     */
+/*
+ * Validator policies can be one of the following
+ */
 #define POL_TRUST_ANCHOR_STR "trust-anchor"
 #define POL_CLOCK_SKEW_STR "clock-skew"
 #define POL_PROV_INSEC_STR "provably-insecure-status"
 #define POL_ZONE_SE_STR "zone-security-expectation"
-#ifdef LIBVAL_DLV
 #define POL_DLV_TRUST_POINTS_STR  "dlv-trust-points"
-#endif
-#ifdef LIBVAL_NSEC3
 #define POL_NSEC3_MAX_ITER_STR "nsec3-max-iter"
-#endif
-
 #define GOPT_TRUST_OOB_STR "trust-oob-answers"
 #define GOPT_EDNS0_SIZE_STR "edns0-size"
 #define GOPT_YES_STR "yes"
@@ -275,6 +125,7 @@ extern          "C" {
 #define GOPT_OVERRIDE_STR "override"
 #define GOPT_LOGTARGET_STR "log"
 #define GOPT_CLOSEST_TA_ONLY_STR "closest-ta-only"
+#define GOPT_REC_FALLBACK "rec-fallback"
 
 #define VAL_POL_GOPT_DISABLE 0 
 #define VAL_POL_GOPT_ENABLE 1
@@ -293,12 +144,13 @@ extern          "C" {
  */
 #define GOPT_TRUST_LOCAL_STR "trust-local-answers"
 
+
     /*
      * Response structures  
      */
     struct val_rr_rec {
         size_t rr_rdata_length;      /* RDATA length */
-        u_char *rr_rdata;       /* Raw RDATA */
+        unsigned char *rr_rdata;       /* Raw RDATA */
         struct val_rr_rec  *rr_next;
         val_astatus_t   rr_status;
     };
@@ -315,36 +167,8 @@ extern          "C" {
         struct val_rr_rec  *val_rrset_sig;  /* All signatures */
     };
 
-    struct policy_list {
-        int             index;
-        policy_entry_t  *pol;
-        struct policy_list *next;
-    };
-
-    struct dnsval_list {
-        char   *dnsval_conf;
-        time_t v_timestamp;
-        struct dnsval_list *next;
-    };
-        
-    /*
-     * This list is ordered from general to more specific --
-     * so "mozilla" < "sendmail" < "browser:mozilla"
-     */
-    struct policy_overrides {
-        char           *label;
-        int             label_count;
-        struct policy_list *plist;
-        struct policy_overrides *next;
-    };
-
-    struct qname_chain {
-        u_char        qnc_name_n[NS_MAXCDNAME];
-        struct qname_chain *qnc_next;
-    };
-
     struct val_response {
-        u_char  *vr_response;
+        unsigned char  *vr_response;
         size_t  vr_length;
         val_status_t    vr_val_status;
     };
@@ -357,7 +181,7 @@ extern          "C" {
 
     struct rr_rec {
         size_t rr_length;
-        u_char *rr_data;
+        unsigned char *rr_data;
         struct rr_rec *rr_next;
     };
 
@@ -382,130 +206,58 @@ extern          "C" {
     };
 
     typedef struct val_dnskey_rdata {
-        u_int16_t       flags;
-        u_int8_t        protocol;
-        u_int8_t        algorithm;
-        u_int32_t       public_key_len; /* in bytes */
-        u_char         *public_key;
-        u_int16_t       key_tag;
+        unsigned short flags;
+        unsigned char  protocol;
+        unsigned char  algorithm;
+        unsigned int   public_key_len; /* in bytes */
+        unsigned char  *public_key;
+        unsigned short key_tag;
         struct val_dnskey_rdata *next;
     } val_dnskey_rdata_t;
 
     typedef struct val_rrsig_rdata {
-        u_int16_t       type_covered;
-        u_int8_t        algorithm;
-        u_int8_t        labels;
-        u_int32_t       orig_ttl;
-        u_int32_t       sig_expr;
-        u_int32_t       sig_incp;
-        u_int16_t       key_tag;
-        u_char          signer_name[256];       /* null terminated */
-        u_int32_t       signature_len;  /* in bytes */
-        u_char         *signature;
+        unsigned short type_covered;
+        unsigned char  algorithm;
+        unsigned char  labels;
+        unsigned int   orig_ttl;
+        unsigned int   sig_expr;
+        unsigned int   sig_incp;
+        unsigned short key_tag;
+        unsigned char  signer_name[256];       /* null terminated */
+        unsigned int   signature_len;  /* in bytes */
+        unsigned char  *signature;
         struct val_rrsig_rdata *next;
     } val_rrsig_rdata_t;
 
     typedef struct val_ds_rdata {
-        u_int16_t       d_keytag;
-        u_int8_t        d_algo;
-        u_int8_t        d_type;
-        u_char         *d_hash;
-        u_int32_t       d_hash_len;
+        unsigned short d_keytag;
+        unsigned char  d_algo;
+        unsigned char  d_type;
+        unsigned char  *d_hash;
+        unsigned int   d_hash_len;
     } val_ds_rdata_t;
 
-#ifdef LIBVAL_NSEC3
     typedef struct val_nsec3_rdata {
-        u_int8_t        alg;
-        u_int8_t        flags;
-        u_int16_t       iterations;
-        u_int8_t        saltlen;
-        u_char         *salt;
-        u_int8_t        nexthashlen;
-        u_char         *nexthash;
-        u_int16_t       bit_field;
+        unsigned char  alg;
+        unsigned char  flags;
+        unsigned short iterations;
+        unsigned char  saltlen;
+        unsigned char  *salt;
+        unsigned char  nexthashlen;
+        unsigned char  *nexthash;
+        unsigned short bit_field;
     } val_nsec3_rdata_t;
 
-#define NSEC3_FLAG_OPTOUT 0x01
-
-#endif
-
-#ifndef VAL_NO_ASYNC
-    /*
-     * asynchronous status
-     */
-#define VAL_AS_CTX_USER_SUPPLIED     0x00000001 /* i.e. don't delete it! */
-#define VAL_AS_IGNORE_CACHE          0x00000002
-#define VAL_AS_NO_NEW_QUERIES        0x00000004
-#define VAL_AS_DONE                  0x00000008 /* have results/answers */
-#define VAL_AS_CB_COMPLETED          0x00000010 /* called user callbacks */
-#define VAL_AS_NO_ANSWERS            0x00000020 /* don't care about answers */
-#define VAL_AS_NO_CALLBACKS          0x00000040 /* don't call callbacks */
-
-    typedef struct val_async_status_s val_async_status;
-    typedef int (*val_cb_results)(val_async_status *as);
-
-    struct val_async_status_s {
-        val_context_t                 *val_as_ctx;
-        u_int32_t                      val_as_flags;
-
-        u_char                         val_as_inflight;
-        struct queries_for_query      *val_as_top_q;
-        struct queries_for_query      *val_as_queries;
-
-        u_char                         val_as_domain_name_n[NS_MAXCDNAME];
-        int                            val_as_class;
-        int                            val_as_type;
-
-        struct val_result_chain       *val_as_results;
-        struct val_answer_chain       *val_as_answers;
-
-        val_cb_results                 val_as_result_cb;
-        void                          *val_as_cb_user_ctx;
-
-        struct val_async_status_s     *val_as_next;
-    };
-#endif /* VAL_NO_ASYNC */
-
-    /*
-     * Logging-related definitions 
-     */
     typedef void    (*val_log_logger_t) (struct val_log * logp,
                                          const val_context_t * ctx,
                                          int level,
                                          const char *format, va_list ap);
+
     typedef void    (*val_log_cb_t) (struct val_log *logp, int level,
-                                     const char *buf);
+                                                     const char *buf);
 
-    typedef struct val_log {
-        val_log_logger_t logf;  /* log function ptr */
-        u_char   level;  /* 0 - 9, corresponds w/sylog severities */
-        u_char   lflags; /* generic log flags */
 
-        void           *a_void; /* logger dependent */
-
-        union {
-            struct {
-                int             sock;
-                struct sockaddr_in server;
-            } udp;
-            struct {
-                char           *name;
-                FILE           *fp;
-            } file;
-            struct {
-                int             facility;
-            } syslog;
-            struct {
-                val_log_cb_t    func;
-            } cb;
-            struct {
-                void           *my_ptr;
-            } user;
-        } opt;
-        struct val_log *next;
-    } val_log_t;
-
-    char           *get_hex_string(const u_char *data, size_t datalen,
+    char           *get_hex_string(const unsigned char *data, size_t datalen,
                                    char *buf, size_t buflen);
     void            val_log_rrsig_rdata(const val_context_t * ctx,
                                         int level, const char *prefix,
@@ -522,6 +274,8 @@ extern          "C" {
                                                  *results);
     void            val_log(const val_context_t * ctx, int level,
                             const char *format, ...);
+    void            val_log_ap(const val_context_t * ctx, int level,
+                               const char *log_template, va_list ap);
 
     val_log_t      *val_log_add_cb(val_log_t **log_head, int level, val_log_cb_t func);
     val_log_t      *val_log_add_filep(val_log_t **log_head, int level, FILE * p);
@@ -543,6 +297,79 @@ extern          "C" {
     const char     *p_val_status(val_status_t err);
     const char     *p_val_err(int err);
 
+#ifndef VAL_NO_ASYNC
+    /*
+     * asynchronous status
+     */
+#define VAL_AS_CTX_USER_SUPPLIED     0x00000001 /* i.e. don't delete it! */
+#define VAL_AS_IGNORE_CACHE          0x00000002
+#define VAL_AS_NO_NEW_QUERIES        0x00000004
+#define VAL_AS_NO_ANSWERS            0x00000008 /* don't care about answers */
+#define VAL_AS_NO_CALLBACKS          0x00000010 /* don't call callbacks */
+#define VAL_AS_NO_CANCEL_CALLBACKS   0x00000020 /* no cb if req cancelled */
+
+#define VAL_AS_DONE                  0x01000000 /* have results/answers */
+#define VAL_AS_CALLBACK_CALLED       0x02000000 /* called user callbacks */
+
+    /*
+     * asynchronous events
+     */
+#define VAL_AS_EVENT_COMPLETED             0x01
+#define VAL_AS_EVENT_CANCELED              0x02
+
+    /** opaque status object for async request */
+    typedef struct val_async_status_s val_async_status;
+
+    typedef struct val_cb_params_s {
+        val_status_t             val_status;
+        char                    *name;
+        int                      class_h;
+        int                      type_h;
+        struct val_result_chain *results;
+        struct val_answer_chain *answers;
+    } val_cb_params_t;
+
+    typedef int (*val_async_event_cb)(val_async_status *async_status,
+                                      int event, val_context_t *ctx,
+                                      void *cb_data, val_cb_params_t *cbp);
+
+    int             val_async_submit(val_context_t * ctx,
+                                     const char * domain_name, int class_h,
+                                     int type_h, unsigned int flags,
+                                     val_async_event_cb callback, void *cb_data,
+                                     val_async_status **async_status);
+    int             val_async_check_wait(val_context_t *context,
+                                         fd_set *pending_desc, int *nfds,
+                                         struct timeval *tv, unsigned int flags);
+    int             val_async_select(val_context_t *context, fd_set *pending,
+                                     int *nfds, struct timeval *timeout,
+                                     u_int32_t flags);
+    int             val_async_select_info(val_context_t *context,
+                                    fd_set *fds,
+                                    int *num_fds,
+                                    struct timeval *timeout);
+
+    /*
+     * cancellation flags
+     */
+#define VAL_AS_CANCEL_NO_CALLBACKS     0x00000001 /* no cb if req completed */
+#define VAL_AS_CANCEL_RESERVED_MASK    0xFF000000 /* one byte internal use */
+
+    int             val_async_cancel(val_context_t *context,
+                                     val_async_status *as,
+                                     u_int flags);
+    int             val_async_cancel_all(val_context_t *context, u_int flags);
+
+    /*
+     * backwards compatibility
+     */
+    /** val_async_check: use val_async_check_wait instead */
+    int             val_async_check(val_context_t *context,
+                                    fd_set *pending_desc, int *nfds,
+                                    unsigned int flags);
+
+#endif /* VAL_NO_ASYNC */
+
     /*
      *******************************************
      * Other functions exported by the validator
@@ -558,9 +385,9 @@ extern          "C" {
                                           *results);
     int             val_resolve_and_check(val_context_t * context,
                                           const char * domain_name,
-                                          int qclass,
-                                          int qtype,
-                                          u_int32_t flags,
+                                          int class_h,
+                                          int type_h,
+                                          unsigned int flags,
                                           struct val_result_chain
                                           **results);
 
@@ -592,19 +419,14 @@ extern          "C" {
                                       val_policy_handle_t **pol);
     int             val_remove_valpolicy(val_context_t *context, 
                                       val_policy_handle_t *pol);
-    /*
-     * from val_support.h 
-     */
-    size_t       wire_name_length(const u_char * field);
-
     /*
      * from val_x_query.c 
      */
     int             val_res_query(val_context_t * ctx, const char *dname,
-                                  int q_class, int type, u_char * answer,
+                                  int class_h, int type_h, unsigned char * answer,
                                   int anslen, val_status_t * val_status);
     int             val_res_search(val_context_t * ctx, const char *dname,
-                                   int class_h, int type, u_char * answer,
+                                   int class_h, int type, unsigned char * answer,
                                    int anslen, val_status_t * val_status);
     int             compose_answer(const char * name,
                                    int type_h,
@@ -614,9 +436,6 @@ extern          "C" {
     /*
      * from val_gethostbyname.c 
      */
-#ifndef h_errno                 /* on linux, netdb.h defines this as a macro */
-    extern int      h_errno;
-#endif
     struct hostent *val_gethostbyname(val_context_t * ctx,
                                       const char *name,
                                       val_status_t * val_status);
@@ -647,25 +466,42 @@ extern          "C" {
     /*
      * from val_getaddrinfo.c 
      */
+
     int             val_getaddrinfo(val_context_t * ctx,
                                     const char *nodename,
                                     const char *servname,
                                     const struct addrinfo *hints,
                                     struct addrinfo **res,
                                     val_status_t * val_status);
-#ifndef HAVE_FREEADDRINFO
-    void            freeaddrinfo(struct addrinfo *ainfo);
-#endif
 
     int             val_getnameinfo(val_context_t * ctx,
                                     const struct sockaddr *sa,
-                                    socklen_t salen,
+                                    size_t salen,
                                     char *host,
                                     size_t hostlen,
                                     char *serv,
                                     size_t servlen,
                                     int flags, val_status_t * val_status);
 
+    int             val_getaddrinfo_has_status(int rc);
+
+#define VAL_GETADDRINFO_HAS_STATUS val_getaddrinfo_has_status 
+#define VAL_GETNAMEINFO_HAS_STATUS val_getaddrinfo_has_status 
+
+    typedef struct val_gai_status_s val_gai_status;
+    typedef int (*val_gai_callback)(void *callback_data, int eai_retval,
+                                    struct addrinfo *res,
+                                    val_status_t val_status);
+
+    int             val_getaddrinfo_submit(val_context_t * context,
+                                           const char *nodename,
+                                           const char *servname,
+                                           const struct addrinfo *hints_in,
+                                           val_gai_callback callback,
+                                           void *callback_data,
+                                           u_int vgai_flags,
+                                           val_gai_status **status);
+
     /*
      * A thread-safe, re-entrant version of val_gethostbyaddr 
      */
@@ -695,21 +531,22 @@ extern          "C" {
      */
     int val_get_rrset(val_context_t *ctx,
                       const char *name,
-                      int classid,
-                      int type,
-                      u_int32_t flags,
+                      int class_h,
+                      int type_h,
+                      unsigned int flags,
                       struct val_answer_chain **answers);
 
     void val_free_answer_chain(struct val_answer_chain *answers);
 
 
     /*
-     * async functions from val_resquery.c
+     * utility functions. mostly used internal to libval.
      */
-    int val_async_select_info(val_context_t *context,
-                              fd_set *fds,
-                              int *num_fds,
-                              struct timeval *timeout);
+    int val_get_answer_from_result(val_context_t *context, const char *name,
+                                   int class_h, int type_h,
+                                   struct val_result_chain **results,
+                                   struct val_answer_chain **answers,
+                                   u_int32_t vgafr_flags);
 
 
     /*
