@@ -15,18 +15,12 @@
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 #include "validator-config.h"
+#include "validator-internal.h"
 
-#include <sys/types.h>
-
-#include <errno.h>
-#include <string.h>
-#include <arpa/nameser.h>
-#include <netinet/in.h>
-#include <resolv.h>
-
-#include "validator/resolver.h"
 #include "res_support.h"
+#include "res_comp.h"
 
+#if HAVE_ARPA_NAMESER_H && !defined(__OpenBSD__)
 #ifndef STRUCT___NS_MSG_HAS__MSG_PTR
 #   ifdef STRUCT___NS_MSG_HAS__PTR
 #      define _msg_ptr _ptr
@@ -34,6 +28,7 @@
 #      error "unknown msg ptr member in struct __ns_msg"
 #   endif
 #endif
+#endif
 
 /*
  * Forward. 
@@ -78,7 +73,7 @@ struct _ns_flagdata _ns_flagdata_flags[16] = {
  * both use defines of the function to map to other function
  * names.
  *
- * We completely rename the function for internal usage just to independent
+ * We rename this function for internal usage
  */
 int
 libsres_msg_getflag(ns_msg han, int flag)
