@@ -93,23 +93,11 @@
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 #include "validator-config.h"
+#include "validator-internal.h"
 
-
-#include <sys/types.h>
-#include <sys/param.h>
-
-#include <ctype.h>
-#include <errno.h>
-#include <math.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
-#include <arpa/nameser.h>
-#include <netinet/in.h>
-#include <resolv.h>
-#include "validator/resolver.h"
 #include "res_support.h"
+#include "res_comp.h"
+#include "res_debug.h"
 
 const char     *_libsres_opcodes[] = {
     "QUERY",
@@ -131,16 +119,6 @@ const char     *_libsres_opcodes[] = {
 };
 
 
-/*
- * some system prototypes don't match up
- */
-#ifdef sun
-#define P_OPTION_ARG_TYPE uint_t
-#define P_SECSTODATE_ARG_TYPE  uint_t
-#else
-#define P_OPTION_ARG_TYPE u_long
-#define P_SECSTODATE_ARG_TYPE  u_long
-#endif
 
 /*
  * some system headers are outdated
@@ -173,6 +151,21 @@ extern const char *_res_sectioncodes[];
 
 #define ERRBUFLEN 80
 
+static void
+_print_or_log(void *file, int level, const char *template, ...)
+{
+    va_list         ap;
+
+    va_start(ap, template);
+
+    if (NULL == file)
+        res_log((val_context_t*)NULL, level, template, ap);
+    else
+        vfprintf(file, template, ap);
+
+    va_end(ap);
+}
+
 static void
 do_section(ns_msg * handle, ns_sect section, int pflag, FILE * file)
 {
@@ -187,7 +180,7 @@ do_section(ns_msg * handle, ns_sect section, int pflag, FILE * file)
 
     buf = MALLOC(buflen);
     if (buf == NULL) {
-        fprintf(file, ";; memory allocation failure\n");
+        res_log(NULL,LOG_DEBUG, ";; memory allocation failure\n");
         return;
     }
 
@@ -198,22 +191,22 @@ do_section(ns_msg * handle, ns_sect section, int pflag, FILE * file)
             if (errno != ENODEV) {
 #ifdef HAVE_STRERROR_R
                 if (!strerror_r(errno, err_buf, ERRBUFLEN))
-                    fprintf(file, ";; ns_parserr: %s\n", err_buf);
+                    _print_or_log(file, LOG_DEBUG, ";; ns_parserr: %s\n", err_buf);
                 else
-                    fprintf(file, ";; ns_parserr: Error\n");
+                    _print_or_log(file, LOG_DEBUG, ";; ns_parserr: Error\n");
 #else
-                fprintf(file, ";; ns_parserr: %s\n", strerror(errno));
+                _print_or_log(file, LOG_DEBUG, ";; ns_parserr: %s\n", strerror(errno));
 #endif
             }
             goto cleanup;
         }
         if (section == ns_s_qd)
-            fprintf(file, ";;\t%s, type = %s, class = %s\n",
+            _print_or_log(file, LOG_DEBUG, ";;\t%s, type = %s, class = %s\n",
                     ns_rr_name(rr),
                     p_type(ns_rr_type(rr)), p_class(ns_rr_class(rr)));
         else if (section == ns_s_ar && ns_rr_type(rr) == ns_t_opt) {
             u_int32_t       ttl = ns_rr_ttl(rr);
-            fprintf(file,
+            _print_or_log(file, LOG_DEBUG,
                     "; EDNS: version: %u, udp=%u, flags=%04x\n",
                     (ttl >> 16) & 0xff, ns_rr_class(rr), ttl & 0xffff);
         } else {
@@ -227,23 +220,22 @@ do_section(ns_msg * handle, ns_sect section, int pflag, FILE * file)
                         buf = MALLOC(buflen += 1024);
 #endif
                     if (buf == NULL) {
-                        fprintf(file, ";; memory allocation failure\n");
+                        _print_or_log(file, LOG_DEBUG, ";; memory allocation failure\n");
                         return;
                     }
                     continue;
                 }
 #ifdef HAVE_STRERROR_R
                 if (!strerror_r(errno, err_buf, ERRBUFLEN))
-                    fprintf(file, ";; ns_sprintrr: %s\n", err_buf);
+                    _print_or_log(file, LOG_DEBUG, ";; ns_sprintrr: %s\n", err_buf);
                 else
-                    fprintf(file, ";; ns_sprintrr: Error\n");
+                    _print_or_log(file, LOG_DEBUG, ";; ns_sprintrr: Error\n");
 #else
-                fprintf(file, ";; ns_sprintrr: %s\n", strerror(errno));
+                _print_or_log(file, LOG_DEBUG, ";; ns_sprintrr: %s\n", strerror(errno));
 #endif
                 goto cleanup;
             }
-            fputs(buf, file);
-            fputc('\n', file);
+            _print_or_log(file, LOG_DEBUG, "%s\n", buf);
         }
         rrnum++;
     }
@@ -252,82 +244,6 @@ do_section(ns_msg * handle, ns_sect section, int pflag, FILE * file)
         FREE(buf);
 }
 
-/*
- * Print the contents of a query.
- * This is intended to be primarily a debugging routine.
- */
-void
-libsres_pquery(const u_char * msg, size_t len, FILE * file)
-{
-    ns_msg          handle;
-    int             qdcount, ancount, nscount, arcount;
-    u_int           opcode, rcode, id;
-#ifdef HAVE_STRERROR_R
-    char            err_buf[ERRBUFLEN + 1];
-#endif
-
-    if (ns_initparse(msg, len, &handle) < 0) {
-#ifdef HAVE_STRERROR_R
-        if (!strerror_r(errno, err_buf, ERRBUFLEN))
-            fprintf(file, ";; ns_initparse: %s\n", err_buf);
-        else
-            fprintf(file, ";; ns_initparse: Error\n");
-#else
-        fprintf(file, ";; ns_initparse: %s\n", strerror(errno));
-#endif
-
-        return;
-    }
-    opcode = libsres_msg_getflag(handle, ns_f_opcode);
-    rcode = libsres_msg_getflag(handle, ns_f_rcode);
-    id = ns_msg_id(handle);
-    qdcount = ns_msg_count(handle, ns_s_qd);
-    ancount = ns_msg_count(handle, ns_s_an);
-    nscount = ns_msg_count(handle, ns_s_ns);
-    arcount = ns_msg_count(handle, ns_s_ar);
-
-    /*
-     * Print header fields.
-     */
-    fprintf(file,
-            ";; ->>HEADER<<- opcode: %s, status: %s, id: %d\n",
-            _libsres_opcodes[opcode], p_rcode(rcode), id);
-    putc(';', file);
-    fprintf(file, "; flags:");
-    if (libsres_msg_getflag(handle, ns_f_qr))
-        fprintf(file, " qr");
-    if (libsres_msg_getflag(handle, ns_f_aa))
-        fprintf(file, " aa");
-    if (libsres_msg_getflag(handle, ns_f_tc))
-        fprintf(file, " tc");
-    if (libsres_msg_getflag(handle, ns_f_rd))
-        fprintf(file, " rd");
-    if (libsres_msg_getflag(handle, ns_f_ra))
-        fprintf(file, " ra");
-    if (libsres_msg_getflag(handle, ns_f_z))
-        fprintf(file, " ??");
-    if (libsres_msg_getflag(handle, ns_f_ad))
-        fprintf(file, " ad");
-    if (libsres_msg_getflag(handle, ns_f_cd))
-        fprintf(file, " cd");
-
-    fprintf(file, "; %s: %d", p_section(ns_s_qd, opcode), qdcount);
-    fprintf(file, ", %s: %d", p_section(ns_s_an, opcode), ancount);
-    fprintf(file, ", %s: %d", p_section(ns_s_ns, opcode), nscount);
-    fprintf(file, ", %s: %d", p_section(ns_s_ar, opcode), arcount);
-
-    putc('\n', file);
-
-    /*
-     * Print the various sections.
-     */
-    do_section(&handle, ns_s_qd, RES_PRF_QUES, file);
-    do_section(&handle, ns_s_an, RES_PRF_ANS, file);
-    do_section(&handle, ns_s_ns, RES_PRF_AUTH, file);
-    do_section(&handle, ns_s_ar, RES_PRF_ADD, file);
-    if (qdcount == 0 && ancount == 0 && nscount == 0 && arcount == 0)
-        putc('\n', file);
-}
 
 
 const u_char   *
@@ -339,9 +255,9 @@ p_cdnname(const u_char * cp, const u_char * msg, int len, FILE * file)
     if ((n = dn_expand(msg, msg + len, cp, name, sizeof(name))) < 0)
         return (NULL);
     if (name[0] == '\0')
-        putc('.', file);
+        _print_or_log(file, LOG_DEBUG, "%c", '.');
     else
-        fputs(name, file);
+        _print_or_log(file, LOG_DEBUG, "%s", name);
     return (cp + n);
 }
 
@@ -357,11 +273,7 @@ p_cdname(const u_char * cp, const u_char * msg, FILE * file)
  */
 
 const u_char   *
-p_fqnname(cp, msg, msglen, name, namelen)
-     const u_char   *cp, *msg;
-     int             msglen;
-     char           *name;
-     int             namelen;
+p_fqnname(const u_char *cp, const u_char *msg, int msglen, char *name, int namelen) 
 {
     int             n, newlen;
 
@@ -390,7 +302,7 @@ p_fqname(const u_char * cp, const u_char * msg, FILE * file)
     n = p_fqnname(cp, msg, NS_MAXCDNAME, name, sizeof(name));
     if (n == NULL)
         return (NULL);
-    fputs(name, file);
+    _print_or_log(file, LOG_DEBUG, "%s", name);
     return (n);
 }
 
@@ -398,23 +310,24 @@ p_fqname(const u_char * cp, const u_char * msg, FILE * file)
  * the original res_sym struct used regular, non-const, char* pointers.
  * this causes a slew of warning about initialization discarding qualifiers,
  * so this is the same structure but with const char* pointers.
+ * On BSD, the definition from resolv.h cannot be included safely with the current
+ * configure script, so we use this internal type.
  *
  * Unfortuantely, on OS X, __p_class_sym is a macro, and exported with the
- * original res_sym type. And on FreeBSD/NetBSD, the extern declares in the header
- * conflict with the redefinition. So in these cases, we don't use this hack.
+ * original res_sym type. So in this case, we don't use this hack.
  *
  * And, for once, solaris has a better header than the rest, and has const
  * char ptrs in res_sym.
  */
-#if (defined(__p_class_syms) || defined(sun) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__CYGWIN__))
+#if (defined(__APPLE__) || defined(sun) || defined(__CYGWIN__) || defined(linux))
 #define RES_SYM_TYPE res_sym
 #else
-#define RES_SYM_TYPE res_sym_const
 struct res_sym_const {
     int             number;     /* Identifying number, like T_MX */
     const char     *name;       /* Its symbolic name, like "MX" */
     const char     *humanname;  /* Its fun name, like "mail exchanger" */
 };
+#define RES_SYM_TYPE res_sym_const
 #endif
 
 /*
@@ -422,7 +335,7 @@ struct res_sym_const {
  * that ns_c_any is a qclass but not a class.  (You can ask for records of class
  * ns_c_any, but you can't have any records of that class in the database.)
  */
-const struct RES_SYM_TYPE __p_class_syms[] = {
+const struct RES_SYM_TYPE __p_res_class_syms[] = {
     {ns_c_in, "IN", (char *) 0},
     {ns_c_chaos, "CH", (char *) 0},
     {ns_c_chaos, "CHAOS", (char *) 0},
@@ -562,10 +475,15 @@ const struct RES_SYM_TYPE __p_rcode_syms[] = {
 };
 
 int
-sym_ston(const struct res_sym *syms, const char *name, int *success)
+sym_ston(const struct RES_SYM_TYPE *syms, const char *name, int *success)
 {
+    size_t namelen = 0;
+    if (name)
+        namelen = strlen(name);
+
     for ((void) NULL; syms->name != 0; syms++) {
-        if (strcasecmp(name, syms->name) == 0) {
+        if ((namelen == strlen(syms->name)) &&
+            strncasecmp(name, syms->name, namelen) == 0) {
             if (success)
                 *success = 1;
             return (syms->number);
@@ -577,7 +495,7 @@ sym_ston(const struct res_sym *syms, const char *name, int *success)
 }
 
 const char     *
-sym_ntos(const struct res_sym *syms, int number, int *success)
+sym_ntos(const struct RES_SYM_TYPE *syms, int number, int *success)
 {
     static char     unname[20];
     
@@ -598,7 +516,7 @@ sym_ntos(const struct res_sym *syms, int number, int *success)
 }
 
 const char     *
-sym_ntop(const struct res_sym *syms, int number, int *success)
+sym_ntop(const struct RES_SYM_TYPE *syms, int number, int *success)
 {
     static char     unname[20];
 
@@ -626,7 +544,7 @@ p_sres_type(int type)
     static char     typebuf[20];
 
     result =
-        sym_ntos((const struct res_sym *) __p_type_sres_syms, type, &success);
+        sym_ntos((const struct RES_SYM_TYPE *) __p_type_sres_syms, type, &success);
 
     if (success)
         return (result);
@@ -642,14 +560,14 @@ p_sres_type(int type)
 const char     *
 p_section(int section, int opcode)
 {
-    const struct res_sym *symbols;
+    const struct RES_SYM_TYPE *symbols;
 
     switch (opcode) {
     case ns_o_update:
-        symbols = (const struct res_sym *) __p_update_section_syms;
+        symbols = (const struct RES_SYM_TYPE *) __p_update_section_syms;
         break;
     default:
-        symbols = (const struct res_sym *) __p_default_section_syms;
+        symbols = (const struct RES_SYM_TYPE *) __p_default_section_syms;
         break;
     }
     return (sym_ntos(symbols, section, (int *) 0));
@@ -659,19 +577,19 @@ p_section(int section, int opcode)
  * Return a mnemonic for class.
  */
 const char     *
-p_class(int class)
+p_class(int class_h)
 {
     int             success;
     const char     *result;
     static char     classbuf[20];
 
     result =
-        sym_ntos((const struct res_sym *) __p_class_syms, class, &success);
+        sym_ntos((const struct RES_SYM_TYPE *) __p_res_class_syms, class_h, &success);
     if (success)
         return (result);
-    if (class < 0 || class > 0xfff)
+    if (class_h < 0 || class_h > 0xfff)
         return ("BADCLASS");
-    snprintf(classbuf, sizeof(classbuf), "CLASS%d", class);
+    snprintf(classbuf, sizeof(classbuf), "CLASS%d", class_h);
     return (classbuf);
 }
 
@@ -761,7 +679,7 @@ const char     *
 p_rcode(int rcode)
 {
     return (sym_ntos
-            ((const struct res_sym *) __p_rcode_syms, rcode, (int *) 0));
+            ((const struct RES_SYM_TYPE *) __p_rcode_syms, rcode, (int *) 0));
 }
 
 /*
@@ -808,7 +726,7 @@ static const unsigned int poweroften[10] =
  * takes an XeY precision/size value, returns a string representation. 
  */
 static const char *
-precsize_ntoa(u_int8_t prec)
+precsize_ntoa(u_char prec)
 {
     static char     retbuf[sizeof("90000000.00")];      /* XXX nonreentrant */
     unsigned long   val;
@@ -827,11 +745,11 @@ precsize_ntoa(u_int8_t prec)
 /*
  * converts ascii size/precision X * 10**Y(cm) to 0xXY.  moves pointer. 
  */
-static          u_int8_t
+static          u_char
 precsize_aton(const char **strptr)
 {
     unsigned int    mval = 0, cmval = 0;
-    u_int8_t        retval = 0;
+    u_char        retval = 0;
     const char     *cp;
     int             exponent;
     int             mantissa;
@@ -978,9 +896,7 @@ latlon2ul(const char **latlonstrptr, int *which)
  * * representation. 
  */
 int
-loc_aton(ascii, binary)
-     const char     *ascii;
-     u_char         *binary;
+loc_aton(const char *ascii, u_char *binary)
 {
     const char     *cp, *maxcp;
     u_char         *bcp;
@@ -988,9 +904,9 @@ loc_aton(ascii, binary)
     u_int32_t       latit = 0, longit = 0, alt = 0;
     u_int32_t       lltemp1 = 0, lltemp2 = 0;
     int             altmeters = 0, altfrac = 0, altsign = 1;
-    u_int8_t        hp = 0x16;  /* default = 1e6 cm = 10000.00m = 10km */
-    u_int8_t        vp = 0x13;  /* default = 1e3 cm = 10.00m */
-    u_int8_t        siz = 0x12; /* default = 1e2 cm = 1.00m */
+    u_char          hp = 0x16;  /* default = 1e6 cm = 10000.00m = 10km */
+    u_char          vp = 0x13;  /* default = 1e3 cm = 10.00m */
+    u_char          siz = 0x12; /* default = 1e2 cm = 1.00m */
     int             which1 = 0, which2 = 0;
 
     cp = ascii;
@@ -1078,7 +994,7 @@ loc_aton(ascii, binary)
   defaults:
 
     bcp = binary;
-    *bcp++ = (u_int8_t) 0;      /* version byte */
+    *bcp++ = (u_char) 0;      /* version byte */
     *bcp++ = siz;
     *bcp++ = hp;
     *bcp++ = vp;
@@ -1093,9 +1009,7 @@ loc_aton(ascii, binary)
  * takes an on-the-wire LOC RR and formats it in a human readable format. 
  */
 const char     *
-loc_ntoa(binary, ascii)
-     const u_char   *binary;
-     char           *ascii;
+loc_ntoa(const u_char *binary, char *ascii)
 {
     static const char *error = "?";
     static char     tmpbuf[sizeof
@@ -1110,9 +1024,9 @@ loc_ntoa(binary, ascii)
 
     const u_int32_t referencealt = 100000 * 100;
 
-    int32_t         latval, longval, altval;
+    u_int32_t       latval, longval, altval;
     u_int32_t       templ;
-    u_int8_t        sizeval, hpval, vpval, versionval;
+    u_char          sizeval, hpval, vpval, versionval;
 
     char           *sizestr, *hpstr, *vpstr;
 
@@ -1122,7 +1036,7 @@ loc_ntoa(binary, ascii)
         ascii = tmpbuf;
 
     if (versionval) {
-        (void) sprintf(ascii, "; error: unknown LOC RR version");
+        (void) SPRINTF((ascii, "; error: unknown LOC RR version"));
         return (ascii);
     }
 
@@ -1181,14 +1095,14 @@ loc_ntoa(binary, ascii)
     hpstr = strdup(precsize_ntoa(hpval));
     vpstr = strdup(precsize_ntoa(vpval));
 
-    sprintf(ascii,
+    SPRINTF((ascii,
             "%d %.2d %.2d.%.3d %c %d %.2d %.2d.%.3d %c %s%d.%.2dm %sm %sm %sm",
             latdeg, latmin, latsec, latsecfrac, northsouth,
             longdeg, longmin, longsec, longsecfrac, eastwest,
             altsign, altmeters, altfrac,
             (sizestr != NULL) ? sizestr : error,
             (hpstr != NULL) ? hpstr : error,
-            (vpstr != NULL) ? vpstr : error);
+            (vpstr != NULL) ? vpstr : error));
 
     if (sizestr != NULL)
         FREE(sizestr);
@@ -1248,9 +1162,13 @@ p_secstodate(P_SECSTODATE_ARG_TYPE secs)
     static char     output[15]; /* YYYYMMDDHHMMSS and null */
     time_t          clock = secs;
     struct tm      *time;
-    struct tm       res;
 
+#ifdef HAVE_GMTIME_R
+    struct tm       res;
     time = gmtime_r(&clock, &res);
+#else
+    time = gmtime(&clock);
+#endif
     time->tm_year += 1900;
     time->tm_mon += 1;
     snprintf(output, sizeof(output), "%04d%02d%02d%02d%02d%02d",
@@ -1267,7 +1185,7 @@ res_nametoclass(const char *buf, int *successp)
     int             success;
 
     result =
-        sym_ston((const struct res_sym *) __p_class_syms, buf, &success);
+        sym_ston((const struct RES_SYM_TYPE *) __p_res_class_syms, buf, &success);
     if (success)
         goto done;
 
@@ -1292,7 +1210,7 @@ res_nametotype(const char *buf, int *successp)
     int             success;
 
     result =
-        sym_ston((const struct res_sym *) __p_type_sres_syms, buf, &success);
+        sym_ston((const struct RES_SYM_TYPE *) __p_type_sres_syms, buf, &success);
     if (success)
         goto done;
 
@@ -1308,3 +1226,78 @@ res_nametotype(const char *buf, int *successp)
         *successp = success;
     return (result);
 }
+
+/*
+ * Print the contents of a query.
+ * This is intended to be primarily a debugging routine.
+ */
+void
+libsres_pquery(const u_char * msg, size_t len, FILE * file)
+{
+    ns_msg          handle;
+    int             qdcount, ancount, nscount, arcount;
+    u_int           opcode, rcode, id;
+#ifdef HAVE_STRERROR_R
+    char            err_buf[ERRBUFLEN + 1];
+#endif
+    char            buf[128];
+
+    if (ns_initparse(msg, len, &handle) < 0) {
+#ifdef HAVE_STRERROR_R
+        if (!strerror_r(errno, err_buf, ERRBUFLEN))
+            _print_or_log(file, LOG_DEBUG, ";; ns_initparse: %s\n", err_buf);
+        else
+            _print_or_log(file, LOG_DEBUG, ";; ns_initparse: Error\n");
+#else
+        _print_or_log(file, LOG_DEBUG, ";; ns_initparse: %s\n", strerror(errno));
+#endif
+
+        return;
+    }
+    opcode = libsres_msg_getflag(handle, ns_f_opcode);
+    rcode = libsres_msg_getflag(handle, ns_f_rcode);
+    id = ns_msg_id(handle);
+    qdcount = ns_msg_count(handle, ns_s_qd);
+    ancount = ns_msg_count(handle, ns_s_an);
+    nscount = ns_msg_count(handle, ns_s_ns);
+    arcount = ns_msg_count(handle, ns_s_ar);
+
+    /*
+     * Print header fields.
+     */
+    _print_or_log(file, LOG_DEBUG,
+            ";; ->>HEADER<<- opcode: %s, status: %s, id: %d\n",
+            _libsres_opcodes[opcode], p_rcode(rcode), id);
+
+    strcpy(buf, ";; flags:");
+    if (libsres_msg_getflag(handle, ns_f_qr))
+        strcat(buf, " qr");
+    if (libsres_msg_getflag(handle, ns_f_aa))
+        strcat(buf, " aa");
+    if (libsres_msg_getflag(handle, ns_f_tc))
+        strcat(buf, " tc");
+    if (libsres_msg_getflag(handle, ns_f_rd))
+        strcat(buf, " rd");
+    if (libsres_msg_getflag(handle, ns_f_ra))
+        strcat(buf, " ra");
+    if (libsres_msg_getflag(handle, ns_f_z))
+        strcat(buf, " ??");
+    if (libsres_msg_getflag(handle, ns_f_ad))
+        strcat(buf, " ad");
+    if (libsres_msg_getflag(handle, ns_f_cd))
+        strcat(buf, " cd");
+    _print_or_log(file, LOG_DEBUG, "%s", buf);
+
+    _print_or_log(file, LOG_DEBUG, "; %s: %d", p_section(ns_s_qd, opcode),
+                  qdcount);
+    _print_or_log(file, LOG_DEBUG, ", %s: %d", p_section(ns_s_an, opcode),
+                  ancount);
+    _print_or_log(file, LOG_DEBUG, ", %s: %d", p_section(ns_s_ns, opcode),
+                  nscount);
+    _print_or_log(file, LOG_DEBUG, ", %s: %d\n", p_section(ns_s_ar, opcode),
+                  arcount);
+    do_section(&handle, ns_s_qd, RES_PRF_QUES, file);
+    do_section(&handle, ns_s_an, RES_PRF_ANS, file);
+    do_section(&handle, ns_s_ns, RES_PRF_AUTH, file);
+    do_section(&handle, ns_s_ar, RES_PRF_ADD, file);
+}
