@@ -20,20 +20,8 @@
  * See the COPYING file distributed with this software for details.
  */
 #include "validator-config.h"
+#include "validator-internal.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
-
-#include <arpa/nameser.h>
-#ifdef HAVE_ARPA_NAMESER_COMPAT_H
-#include <arpa/nameser_compat.h>
-#elif ! defined( HAVE_ARPA_NAMESER_H )
-#include "arpa/header.h"
-#endif
-
-#include "validator/resolver.h"
 #include "res_mkquery.h"
 #include "res_support.h"
 #include "res_tsig.h"
@@ -46,7 +34,7 @@
 #define ENVELOPE   10
 #define EMSG_MAX   2048
 
-static          size_t
+size_t
 wire_name_length(const u_char * field)
 {
     /*
@@ -175,14 +163,18 @@ theres_something_wrong_with_header(u_char * response,
     /*
      * Check to see if this is supposed to be a real response 
      */
-    if (header->qr == 1 && header->opcode != ns_o_query)
+    if (header->qr == 1 && header->opcode != ns_o_query) {
+        res_log(NULL, LOG_DEBUG, "libsres: ""header: not a query!");
         return SR_HEADER_ERROR;
+    }
 
     /*
      * Check the length and count of the records 
      */
-    if (right_sized(response, response_length) == FALSE)
+    if (right_sized(response, response_length) == FALSE) {
+        res_log(NULL, LOG_DEBUG, "libsres: ""header: not right sized!");
         return SR_HEADER_ERROR;
+    }
 
     /*
      * Check the RCODE value.
@@ -234,23 +226,29 @@ theres_something_wrong_with_header(u_char * response,
             }
         }
 
+        res_log(NULL, LOG_DEBUG, "libsres: ""header: nxdomain!");
         return SR_NXDOMAIN;
     }
 
     switch (header->rcode) {
     case ns_r_formerr:
+        res_log(NULL, LOG_DEBUG, "libsres: ""header: formerr!");
         return SR_FORMERR;
 
     case ns_r_servfail:
+        res_log(NULL, LOG_DEBUG, "libsres: ""header: servfail!");
         return SR_SERVFAIL;
 
     case ns_r_notimpl:
+        res_log(NULL, LOG_DEBUG, "libsres: ""header: notimpl!");
         return SR_NOTIMPL;
 
     case ns_r_refused:
+        res_log(NULL, LOG_DEBUG, "libsres: ""header: refused!");
         return SR_REFUSED;
 
     default:
+        res_log(NULL, LOG_DEBUG, "libsres: ""header: genericerr!");
         return SR_DNS_GENERIC_ERROR;
     }
 
@@ -370,6 +368,32 @@ query_send(const char *name,
            int *trans_id)
 {
     int             ret_val;
+    struct timeval      dummy;
+
+    ret_val = query_queue(name, type_h, class_h, pref_ns, trans_id);
+    if (SR_UNSET != ret_val)
+        return ret_val;
+
+    timerclear(&dummy);
+    res_io_check_one_tid(*trans_id, &dummy, NULL);
+
+    return SR_UNSET;
+}
+
+/*
+ * create payloads and queue them in the transaction array, but do not
+ * start sending them.
+ *
+ * this is usefull if you want to change something about the transaction
+ * first (e.g. set it to default to tcp instead of udp). To start
+ * transaction processing, call res_io_check_one_tid(), and then
+ * response_recv() in a loop.
+ */
+int
+query_queue(const char *name, const u_int16_t type_h, const u_int16_t class_h,
+            struct name_server *pref_ns, int *trans_id)
+{
+    int             ret_val;
 
     u_char         *signed_query = NULL;
     size_t          signed_length = 0;
@@ -404,8 +428,8 @@ query_send(const char *name,
                         &signed_query, &signed_length)) < 0) {
             continue;
         }
-        if ((ret_val = res_io_deliver(trans_id, signed_query,
-                                      signed_length, ns, delay)) < 0) {
+        if ((ret_val = res_io_queue(trans_id, signed_query,
+                                    signed_length, ns, delay)) < 0) {
             continue;
         }
 
@@ -424,6 +448,12 @@ response_recv(int *trans_id,
 {
     int             ret_val;
 
+    if ((NULL == trans_id) || (NULL == pending_desc) || (NULL == closest_event) ||
+        (NULL == respondent) || (NULL == answer) || (NULL == answer_length))
+        return SR_INTERNAL_ERROR;
+
+    res_log(NULL, LOG_DEBUG, "libsres: ""response_recv tid %d", *trans_id);
+
     /*
      * Prepare the default response 
      */
@@ -441,34 +471,29 @@ response_recv(int *trans_id,
         return SR_NO_ANSWER;
 
     if (ret_val == SR_IO_GOT_ANSWER) {
+        log_response(*answer, *answer_length);
         if ((*respondent != NULL) && 
             (res_tsig_verifies(*respondent, 
-                *answer, *answer_length) == SR_TS_OK) && 
+                               *answer, *answer_length) == SR_TS_OK) && 
             (theres_something_wrong_with_header(*answer, 
-                *answer_length) == SR_UNSET)) {
-#if 0
-            printf("The Response: ");
-            printf(":\n");
-            print_response(*answer, *answer_length);
-#endif
+                                                *answer_length) == SR_UNSET)) {
             return SR_UNSET;
 
         } else {
+            res_log(NULL,LOG_DEBUG,"libsres: ""error in response; dropping");
             FREE(*answer);
             *answer = NULL;
             *answer_length = 0;
-            if (*respondent != NULL)  {
+            if (*respondent != NULL)
                 free_name_server(respondent);
-            }
             *respondent = NULL;
             return SR_NO_ANSWER; 
         }
     } 
-        
+
     return SR_NO_ANSWER_YET;
 }
 
-
 int
 get(const char *name,
     const u_int16_t type_h,
@@ -482,11 +507,10 @@ get(const char *name,
     struct timeval closest_event;
     fd_set pending_desc;
     if (SR_UNSET == (ret_val = query_send(name, type_h, class_h, nslist, &trans_id))) {
-
+        res_log(NULL,LOG_DEBUG,"libsres: ""get %s", name);
         do {
             FD_ZERO(&pending_desc);
-            closest_event.tv_sec = 0;
-            closest_event.tv_usec = 0;
+            timerclear(&closest_event);
 
             ret_val = response_recv(&trans_id, &pending_desc, &closest_event, server, response,
                                     response_length);
@@ -501,3 +525,39 @@ get(const char *name,
 
     return ret_val;
 }
+
+int
+get_tcp(const char *name, u_int16_t type_h, u_int16_t class_h,
+        struct name_server *nslist,
+        struct name_server **server,
+        u_char ** response, size_t * response_length)
+{
+    int             ret_val;
+    int             trans_id;
+    struct timeval closest_event;
+    fd_set pending_desc;
+
+    ret_val = query_queue(name, type_h, class_h, nslist, &trans_id);
+    if (SR_UNSET != ret_val)
+        return ret_val;
+
+    res_switch_all_to_tcp_tid(trans_id);
+
+    res_log(NULL,LOG_DEBUG,"libsres: ""get_tcp %s", name);
+    do {
+        FD_ZERO(&pending_desc);
+        timerclear(&closest_event);
+
+        ret_val = response_recv(&trans_id, &pending_desc, &closest_event,
+                                server, response, response_length);
+
+        if (ret_val == SR_NO_ANSWER_YET) {
+            /* wait for some data to become available */
+            wait_for_res_data(&pending_desc, &closest_event);
+        }
+    } while (ret_val == SR_NO_ANSWER_YET);
+
+    res_cancel(&trans_id); /* cleanup transaction */
+
+    return ret_val;
+}
