@@ -20,20 +20,11 @@
  * See the COPYING file distributed with this software for details.
  */
 #include "validator-config.h"
+#include "validator-internal.h"
 
-#include <stdio.h>
-#include <ctype.h>
-#include <string.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <sys/types.h>
 #include <openssl/rand.h>
 
-#include <resolv.h>
-#include <arpa/nameser.h>
-#include <arpa/inet.h>
-
-#include "validator/resolver.h"
+#include "res_support.h"
 
 extern void     libsres_pquery(const u_char * msg, size_t len, FILE * file);
 
@@ -42,6 +33,29 @@ static int      seq_number = 0;
 FILE           *logfile = NULL;
 #define MEM_LOGFILE "memory_logfile"
 
+#ifndef HAVE_GETTIMEOFDAY
+/*
+   Implementation as per:
+   The Open Group Base Specifications, Issue 6
+   IEEE Std 1003.1, 2004 Edition
+
+   The timezone pointer arg is ignored.  Errors are ignored.
+*/
+
+int gettimeofday(struct timeval* p, void* tz /* IGNORED */)
+{
+    union {
+        long long ns100; /*time since 1 Jan 1601 in 100ns units */
+        FILETIME ft;
+    } now;
+
+    GetSystemTimeAsFileTime( &(now.ft) );
+    p->tv_usec=(long)((now.ns100 / 10LL) % 1000000LL );
+    p->tv_sec= (long)((now.ns100-(116444736000000000LL))/10000000LL);
+    return 0;
+}
+#endif
+
 void
 my_free(void *p, char *filename, int lineno)
 {
@@ -57,11 +71,18 @@ my_free(void *p, char *filename, int lineno)
 void           *
 my_malloc(size_t t, char *filename, int lineno)
 {
-    void           *p = malloc(t);
+    void           *p;
 
     if (logfile == NULL)
         logfile = fopen(MEM_LOGFILE, "w");
 
+    if (t == 0) {
+        res_log(NULL,LOG_DEBUG, "0 byte alloc from %-20s %5d", filename, lineno);
+        p = NULL;
+    }
+    else
+        p = malloc(t);
+
     fprintf(logfile, "0x%08lx %5d aMALL %-20s %5d size=%6d\n", (u_long) p,
             seq_number++, filename, lineno, (u_int) t);
     fflush(logfile);
@@ -86,16 +107,17 @@ my_strdup(const char *str, char *filename, int lineno)
 void
 print_response(u_char * ans, size_t resplen)
 {
-    /*
-     * fp_nquery is a resolver debug routine (I think), the rest
-     * would dump the response in byte form, formatted to match
-     * the query's structure 
-     */
-    //fp_nquery(ans, resplen, stdout);
     if (ans && (resplen > 0))
         libsres_pquery(ans, resplen, stdout);
 }
 
+void
+log_response(u_char * ans, size_t resplen)
+{
+    if (ans && (resplen > 0))
+        libsres_pquery(ans, resplen, NULL);
+}
+
 void
 print_hex_field(u_char field[], size_t length, size_t width, char *pref)
 {
@@ -148,23 +170,6 @@ print_hex(u_char field[], size_t length)
     } while (start < length);
 }
 
-int
-complete_read(int sock, void *field, size_t length)
-{
-    size_t             bytes;
-    size_t             bytes_read = 0;
-    memset(field, '\0', length);
-
-    do {
-        bytes = read(sock, field + bytes_read, length - bytes_read);
-        if (bytes == -1)
-            return -1;
-        if (bytes == 0)
-            return -1;
-        bytes_read += bytes;
-    } while (bytes_read < length);
-    return length;
-}
 
 struct sockaddr_storage **
 create_nsaddr_array(int num_addrs)
@@ -201,13 +206,13 @@ parse_name_server(const char *cp, const char *name_n)
     struct sockaddr_in *sin = (struct sockaddr_in *)&serv_addr;
 #ifdef VAL_IPV6
     struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)&serv_addr;
+#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+    size_t addrlen6 = sizeof(struct sockaddr_in6);
 #endif
-    union {
-        struct in_addr   v4;
-#ifdef VAL_IPV6
-        struct in6_addr  v6;
 #endif
-    } address;
+#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+    size_t addrlen4 = sizeof(struct sockaddr_in);
+#endif
 
     if (cp ==  NULL)
         return NULL;
@@ -233,7 +238,7 @@ parse_name_server(const char *cp, const char *name_n)
 
     ns->ns_retrans = RES_TIMEOUT;
     ns->ns_retry = RES_RETRY;
-    ns->ns_options = RES_DEFAULT | RES_RECURSE | RES_DEBUG;
+    ns->ns_options = RES_DEFAULT | RES_DEBUG;
     ns->ns_edns0_size = RES_EDNS0_DEFAULT;
 
     ns->ns_next = NULL;
@@ -247,7 +252,7 @@ parse_name_server(const char *cp, const char *name_n)
     if ( (*cpt == '[') && (cpt = strchr(cpt,']')) ) {
         if ( sizeof(addr) < (cpt - cp) )
             goto err;
-        bzero(addr, sizeof(addr));
+        memset(addr, 0, sizeof(addr));
         strncpy(addr, (cp + 1), (cpt - cp - 1));
         cp = addr;
         if ( (*(++cpt) == ':') && (0 == (port_num = atoi(++cpt))) )
@@ -257,19 +262,17 @@ parse_name_server(const char *cp, const char *name_n)
     /*
      * convert address string
      */
-    bzero(&serv_addr, sizeof(serv_addr));
-    if (inet_pton(AF_INET, cp, &address.v4) > 0) {
+    memset(&serv_addr, 0, sizeof(serv_addr));
+    if (INET_PTON(AF_INET, cp, ((struct sockaddr *)sin), &addrlen4) > 0) {
         sin->sin_family = AF_INET;     // host byte order
-        sin->sin_addr = address.v4;
         sin->sin_port = htons(port_num);       // short, network byte order
     }
     else {
 #ifdef VAL_IPV6
-        if (inet_pton(AF_INET6, cp, &address.v6) != 1)
+        if (INET_PTON(AF_INET6, cp, ((struct sockaddr *)sin6), &addrlen6) != 1)
             goto err;
 
         sin6->sin6_family = AF_INET6;     // host byte order
-        memcpy(&sin6->sin6_addr, &address.v6, sizeof(address.v6));
         sin6->sin6_port = htons(port_num);       // short, network byte order
 #else
         goto err;
@@ -337,3 +340,59 @@ u_int16_t libsres_random(void)
 
     return rnd;
 }
+
+/*
+ * using val_log for logging introduces a circular dependency. Default to
+ * using stderr for logging unless USE_LIBVAL_LOGGING is defined.
+ */
+#ifndef USE_LIBVAL_LOGGING
+
+static int sres_level = LOG_WARNING;
+
+void
+res_log(void *dont_care, int level, const char *template, ...)
+{
+    char            buf[1028];
+    struct timeval  tv;
+    struct tm       *tm;
+    va_list         ap;
+
+    if (NULL == template)
+        return;
+
+    /** check individual level */
+    if (level > sres_level)
+        return;
+
+    gettimeofday(&tv, NULL);
+    tm = localtime(&tv.tv_sec);
+
+    snprintf(buf, sizeof(buf) - 2, "%04d%02d%02d::%02d:%02d:%02d ", 
+            tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
+            tm->tm_hour, tm->tm_min, tm->tm_sec);
+    va_start(ap, template);
+    vsnprintf(&buf[19], sizeof(buf) - 21, template, ap);
+    va_end(ap);
+
+    fprintf(stderr, buf);
+    fprintf(stderr, "\n");
+    fflush(stderr);
+}
+
+#else /* ifdef USE_LIBVAL_LOGGING */
+
+/** pass messages on to val_log... */
+void
+res_log(void *dont_care, int level, const char *template, ...)
+{
+    va_list         ap;
+
+    if (NULL == template)
+        return;
+
+    va_start(ap, template);
+    val_log_ap((val_context_t*)dont_care, level, template, ap);
+    va_end(ap);
+}
+
+#endif /* ifdef USE_LIBVAL_LOGGING */
