@@ -15,12 +15,8 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 #include "validator-config.h"
+#include "validator-internal.h"
 
-#include <stdlib.h>
-#include <string.h>
-#include <sys/types.h>
-#include <arpa/nameser.h>
-#include "validator/resolver.h"
 #include "res_tsig.h"
 #include "res_support.h"
 
@@ -59,6 +55,8 @@ res_tsig_verifies(struct name_server *respondent,
 {
     if (!(respondent->ns_security_options & ZONE_USE_TSIG))
         return SR_TS_OK;
-    else
+    else {
+        res_log(NULL, LOG_ERR, "libsres: ""tsig failure!");
         return SR_TS_FAIL;
+    }
 }
