@@ -6,6 +6,30 @@
 #define VAL_ASSERTION_H
 
 
+#ifndef VAL_NO_THREADS
+ 
+#define LOCK_QC_SH(qc) \
+    pthread_rwlock_rdlock(&qc->qc_rwlock)
+
+#define TRY_LOCK_QC_EX(qc) \
+   (0 == pthread_rwlock_trywrlock(&qc->qc_rwlock)) 
+
+#define LOCK_QC_EX(qc) \
+    pthread_rwlock_wrlock(&qc->qc_rwlock)
+
+#define UNLOCK_QC(qc) \
+    pthread_rwlock_unlock(&qc->qc_rwlock)
+
+#else
+
+#define LOCK_QC_SH(qc) 
+#define TRY_LOCK_QC_EX(qc) (1 == 1)
+#define LOCK_QC_EX(qc) 
+#define UNLOCK_QC(qc) 
+
+#endif /*VAL_NO_THREADS*/
+
+
 struct nsecprooflist {
     struct rrset_rec *the_set;
     struct val_internal_result *res;
@@ -28,11 +52,10 @@ int             add_to_qfq_chain(val_context_t *context,
                                  const u_int16_t class_h, 
                                  const u_int32_t flags,
                                  struct queries_for_query **added_qfq);
+int             free_qfq_chain(val_context_t *context, struct queries_for_query *queries);
 void            free_authentication_chain(struct val_digested_auth_chain
                                           *assertions);
 void            free_query_chain_structure(struct val_query_chain *queries);
-void            requery_with_edns0(val_context_t *context,
-                                    struct val_query_chain *matched_q);
 int             get_zse(val_context_t * ctx, u_char * name_n, 
                         u_int32_t flags, u_int16_t *status, u_char ** match_ptr, u_int32_t *ttl_x);
 int             find_trust_point(val_context_t * ctx, u_char * zone_n, 
@@ -46,11 +69,15 @@ int             check_anc_proof(val_context_t *context,
 #endif
 int             try_chase_query(val_context_t * context,
                                 u_char * domain_name_n,
-                                const u_int16_t q_class,
-                                const u_int16_t type,
+                                const u_int16_t class_h,
+                                const u_int16_t type_h,
                                 const u_int32_t flags,
                                 struct queries_for_query **queries,
                                 struct val_result_chain **results,
                                 int *done);
 
+#ifndef VAL_NO_ASYNC
+int             val_async_status_free(val_async_status *as);
+#endif
+
 #endif
