@@ -8,15 +8,10 @@
  * Contains implementation of val_get_rrset() 
  */
 #include "validator-config.h"
+#include "validator-internal.h"
 
-#include <stdlib.h>
-#include <string.h>
-
-#include <arpa/nameser.h>
-#include <validator/resolver.h>
-#include <validator/validator.h>
-#include <validator/validator-internal.h>
 #include "val_support.h"
+#include "val_context.h"
 
 void
 val_free_answer_chain(struct val_answer_chain *answers)
@@ -40,14 +35,11 @@ val_free_answer_chain(struct val_answer_chain *answers)
 }
 
 int
-val_get_rrset(val_context_t *context,
-              const char *name,
-              int class,
-              int type,
-              u_int32_t flags,
-              struct val_answer_chain **answers) 
+val_get_answer_from_result(val_context_t *context, const char *name, int class_h,
+                           int type_h, struct val_result_chain **results,
+                           struct val_answer_chain **answers,
+                           u_int32_t vgafr_flags)
 {
-    struct val_result_chain *results = NULL;
     struct val_result_chain *res = NULL;
     struct val_answer_chain *ans = NULL;
     struct val_answer_chain *last_ans = NULL;
@@ -55,35 +47,16 @@ val_get_rrset(val_context_t *context,
     const char *n = NULL;
     int len;
     char *name_alias = NULL;
-    val_context_t *ctx = NULL;
     int trusted, validated;
     
-    if (context == NULL) {
-        if (VAL_NO_ERROR != (retval = val_create_context(NULL, &ctx))) {
-            return retval;
-        } 
-    } else {
-        ctx = context;
-    }
-    
-    if (name == NULL || answers == NULL) {
+    if (name == NULL || answers == NULL || results == NULL || answers == NULL) {
         return VAL_BAD_ARGUMENT;
     }
 
     *answers = NULL;
     last_ans = NULL;
    
-
-    if ((retval = val_resolve_and_check(ctx, name, class, type, 
-                                       flags | VAL_QUERY_NO_AC_DETAIL,
-                                       &results)) != VAL_NO_ERROR) {
-        val_log(ctx, LOG_INFO,
-                "get_addrinfo_from_dns(): val_resolve_and_check failed - %s",
-                p_val_err(retval));
-    }
-
-    if (results == NULL) {
-        val_log(ctx, LOG_INFO, "val_get_rrset(): returned NULL result");
+    if (*results == NULL) {
         /* Construct a single val_answer_chain with the untrusted status */
         ans = (struct val_answer_chain *) MALLOC (sizeof(struct val_answer_chain));
         if (ans == NULL) {
@@ -96,8 +69,8 @@ val_get_rrset(val_context_t *context,
             return VAL_OUT_OF_MEMORY;
         } 
         ans->val_ans_status = VAL_UNTRUSTED_ANSWER;
-        ans->val_ans_class = class;
-        ans->val_ans_type = type;
+        ans->val_ans_class = class_h;
+        ans->val_ans_type = type_h;
         ans->val_ans = NULL;
         ans->val_ans_next = NULL;
         *answers = ans;
@@ -108,7 +81,7 @@ val_get_rrset(val_context_t *context,
     validated = 1;
 
     /* Construct the val_answer_chain linked list for returned results */
-    for (res = results; res; res=res->val_rc_next) {
+    for (res = *results; res; res=res->val_rc_next) {
 
         if (!validated || !val_isvalidated(res->val_rc_status))
             validated = 0;
@@ -152,8 +125,8 @@ val_get_rrset(val_context_t *context,
             } else {
                 n = name; /* the name being queried for */
             }
-            ans->val_ans_class = class; 
-            ans->val_ans_type = type; 
+            ans->val_ans_class = class_h; 
+            ans->val_ans_type = type_h; 
             ans->val_ans = NULL;
         } 
 
@@ -199,12 +172,50 @@ val_get_rrset(val_context_t *context,
         trusted = 1;
     } 
 
-    val_free_result_chain(results);
+    val_free_result_chain(*results);
+    *results = NULL;
     return VAL_NO_ERROR;
 
 err:
     val_free_answer_chain(*answers);
     *answers = NULL;
-    val_free_result_chain(results);
+    val_free_result_chain(*results);
+    *results = NULL;
+    return retval;
+} 
+
+int
+val_get_rrset(val_context_t *context,
+              const char *name,
+              int class_h,
+              int type_h,
+              u_int32_t flags,
+              struct val_answer_chain **answers) 
+{
+    struct val_result_chain *results = NULL;
+    int retval = VAL_NO_ERROR;
+    val_context_t *ctx = NULL;
+    
+    if (name == NULL || answers == NULL) {
+        return VAL_BAD_ARGUMENT;
+    }
+
+    ctx = val_create_or_refresh_context(context);/* does CTX_LOCK_POL_SH */
+    if (ctx == NULL)
+        return VAL_INTERNAL_ERROR;
+
+    if ((retval = val_resolve_and_check(ctx, name, class_h, type_h, 
+                                       flags,
+                                       &results)) != VAL_NO_ERROR) {
+        val_log(ctx, LOG_INFO,
+                "get_addrinfo_from_dns(): val_resolve_and_check failed - %s",
+                p_val_err(retval));
+        CTX_UNLOCK_POL(ctx); 
+        return retval;
+    }
+    CTX_UNLOCK_POL(ctx); 
+
+    retval = val_get_answer_from_result(ctx, name, class_h, type_h, &results,
+                                        answers, 0);
     return retval;
 } 
