@@ -13,23 +13,13 @@
  * Contains implementation of val_getaddrinfo() and friends
  */
 #include "validator-config.h"
+#include "validator-internal.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <string.h>
-#include <strings.h>
-#include <resolv.h>
-
-#include <validator/validator.h>
-#include <validator/resolver.h>
 #include "val_policy.h"
-#include <errno.h>
+#include "val_parse.h"
+#include "val_context.h"
 
-#ifndef HAVE_FREEADDRINFO
+#if !HAVE_FREEADDRINFO
 
 /* 
  * define this freeaddrinfo if we don't have it already available
@@ -189,9 +179,12 @@ val_setport(struct sockaddr *saddr, const char *serv, const char *proto)
      */
     if (PF_INET == saddr->sa_family) {
         ((struct sockaddr_in *) saddr)->sin_port = portnum;
-    } else if (PF_INET6 == saddr->sa_family) {
+    } 
+#ifdef VAL_IPV6
+    else if (PF_INET6 == saddr->sa_family) {
         ((struct sockaddr_in6 *) saddr)->sin6_port = portnum;
     }
+#endif
 }                               /* val_setport */
 
 
@@ -388,8 +381,16 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
 
     while (hs) {
         int             alias_index = 0;
-        struct in_addr  ip4_addr;
-        struct in6_addr ip6_addr;
+        struct sockaddr_in  sa;
+#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+        size_t addrlen4 = sizeof(struct sockaddr_in);
+#endif
+#ifdef VAL_IPV6
+        struct sockaddr_in6 sa6;
+#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+        size_t addrlen6 = sizeof(struct sockaddr_in6);
+#endif
+#endif
         struct hosts   *h_prev = hs;
         struct addrinfo *ainfo;
 
@@ -415,13 +416,11 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
         //val_log(ctx, LOG_DEBUG, "}");
 
         memset(ainfo, 0, sizeof(struct addrinfo));
-        memset(&ip4_addr, 0, sizeof(struct in_addr));
-        memset(&ip6_addr, 0, sizeof(struct in6_addr));
 
         /*
          * Check if the address is an IPv4 address 
          */
-        if (inet_pton(AF_INET, hs->address, &ip4_addr) > 0) {
+        if (INET_PTON(AF_INET, hs->address, ((struct sockaddr *)&sa), &addrlen4) > 0) {
             struct sockaddr_in *saddr4 =
                 (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
             if (saddr4 == NULL) {
@@ -434,14 +433,15 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
             ainfo->ai_family = AF_INET;
             saddr4->sin_family = AF_INET;
             ainfo->ai_addrlen = sizeof(struct sockaddr_in);
-            memcpy(&(saddr4->sin_addr), &ip4_addr, sizeof(struct in_addr));
+            memcpy(&(saddr4->sin_addr), &sa.sin_addr, sizeof(struct in_addr));
             ainfo->ai_addr = (struct sockaddr *) saddr4;
             ainfo->ai_canonname = NULL;
         }
+#ifdef VAL_IPV6
         /*
          * Check if the address is an IPv6 address 
          */
-        else if (inet_pton(AF_INET6, hs->address, &ip6_addr) > 0) {
+        else if (INET_PTON(AF_INET6, hs->address, ((struct sockaddr *)&sa6), &addrlen6) > 0) {
             struct sockaddr_in6 *saddr6 = (struct sockaddr_in6 *)
                 malloc(sizeof(struct sockaddr_in6));
             if (saddr6 == NULL) {
@@ -454,13 +454,15 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
             ainfo->ai_family = AF_INET6;
             saddr6->sin6_family = AF_INET6;
             ainfo->ai_addrlen = sizeof(struct sockaddr_in6);
-            memcpy(&(saddr6->sin6_addr), &ip6_addr,
+            memcpy(&(saddr6->sin6_addr), &sa6.sin6_addr,
                    sizeof(struct in6_addr));
             ainfo->ai_addr = (struct sockaddr *) saddr6;
             ainfo->ai_canonname = NULL;
-        } else {
+        } 
+#endif
+        else {
             val_log(ctx, LOG_WARNING, 
-                    "get_addrinfo_from_etc_hosts(): Host is nether an A nor an AAAA address");
+                    "get_addrinfo_from_etc_hosts(): Unkown address type");
             freeaddrinfo(ainfo);
             continue;
         }
@@ -603,6 +605,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
                 /*
                  * Check if the record-type is AAAA 
                  */
+#ifdef VAL_IPV6
                 else if (result->val_ans_type == ns_t_aaaa) {
                     struct sockaddr_in6 *saddr6 = (struct sockaddr_in6 *)
                         malloc(sizeof(struct sockaddr_in6));
@@ -617,7 +620,9 @@ get_addrinfo_from_result(const val_context_t * ctx,
                     memcpy(&(saddr6->sin6_addr.s6_addr), rr->rr_data,
                            rr->rr_length);
                     ainfo->ai_addr = (struct sockaddr *) saddr6;
-                } else {
+                } 
+#endif
+                else {
                     freeaddrinfo(ainfo);
                     rr = rr->rr_next;
                     continue;
@@ -707,21 +712,36 @@ static int
 get_addrinfo_from_dns(val_context_t * ctx,
                       const char *nodename,
                       const char *servname,
-                      const struct addrinfo *hints,
+                      const struct addrinfo *hints_param,
                       struct addrinfo **res,
                       val_status_t *val_status)
 {
     struct val_answer_chain *results = NULL;
     struct addrinfo *ainfo = NULL;
+    const struct addrinfo *hints;
+    struct addrinfo default_hints;
     int    ret = EAI_FAIL;
 
     val_log(ctx, LOG_DEBUG, "get_addrinfo_from_dns() called");
 
     *val_status = VAL_VALIDATED_ANSWER;
 
+    if (NULL == nodename && NULL == servname) {
+        return EAI_NONAME;
+    }
+
+    /*
+     * use a default hints structure if one is not available.
+     */
+    if (hints_param == NULL) {
+        memset(&default_hints, 0, sizeof(default_hints));
+        hints = &default_hints;
+    } else {
+        hints = hints_param;
+    }
+
     if (res == NULL ||
-        (hints != NULL && 
-         hints->ai_family != AF_UNSPEC &&
+        (hints->ai_family != AF_UNSPEC &&
          hints->ai_family != AF_INET &&
          hints->ai_family != AF_INET6)) {
 
@@ -732,8 +752,7 @@ get_addrinfo_from_dns(val_context_t * ctx,
     /*
      * Check if we need to return IPv4 addresses based on the hints 
      */
-    if (hints == NULL || hints->ai_family == AF_UNSPEC
-        || hints->ai_family == AF_INET) {
+    if (hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET) {
 
         val_log(ctx, LOG_DEBUG, "get_addrinfo_from_dns(): checking for A records");
 
@@ -757,8 +776,7 @@ get_addrinfo_from_dns(val_context_t * ctx,
     /*
      * Check if we need to return IPv6 addresses based on the hints 
      */
-    if (hints == NULL || hints->ai_family == AF_UNSPEC
-        || hints->ai_family == AF_INET6) {
+    if (hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET6) {
 
         val_log(ctx, LOG_DEBUG, "get_addrinfo_from_dns(): checking for AAAA records");
         
@@ -784,6 +802,232 @@ get_addrinfo_from_dns(val_context_t * ctx,
 
 }                               /* get_addrinfo_from_dns() */
 
+/**
+ * _getaddrinfo_local: helper function for val_getaddrinfo and
+ *    val_getaddrinfo async. Checks local resources.
+ *
+ * Return value: This function returns 0 if it succeeds, or one of the
+ *               non-zero error codes if it fails.  The error code
+ *               EAI_AGAIN indicates that the caller should try DNS next.
+ */
+static int
+_getaddrinfo_local(val_context_t * ctx, const char *nodename,
+                   const char *servname, const struct addrinfo *hints,
+                   struct addrinfo **res, val_status_t *val_status)
+{
+    struct sockaddr_in  sa;
+    struct addrinfo *ainfo4 = NULL;
+#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+    size_t addrlen4 = sizeof(struct sockaddr_in);
+#endif
+#ifdef VAL_IPV6
+    struct sockaddr_in6 sa6;
+    struct addrinfo *ainfo6 = NULL;
+#if defined( WIN32 ) && !defined( LIBVAL_USE_WOCK )
+    size_t addrlen6 = sizeof(struct sockaddr_in6);
+#endif
+#endif
+    int             retval = 0;
+    const char     *localhost4 = "127.0.0.1";
+    const char     *localhost6 = "::1";
+    const char     *nname = nodename;
+    const struct addrinfo *cur_hints;
+    struct addrinfo default_hints;
+    val_status_t local_ans_status = VAL_OOB_ANSWER;
+    int trusted = 0;
+    
+    val_log(ctx, LOG_DEBUG,
+            "val_getaddrinfo called with nodename = %s, servname = %s",
+            nodename == NULL ? "(null)" : nodename,
+            servname == NULL ? "(null)" : servname);
+
+
+    if (VAL_NO_ERROR == val_is_local_trusted(ctx, &trusted)) {
+        if (trusted) {
+            local_ans_status = VAL_TRUSTED_ANSWER;
+        }
+    }
+    if (res == NULL || val_status == NULL) {
+        retval = 0;
+        goto done;
+    }
+
+    *res = NULL;
+    *val_status = VAL_UNTRUSTED_ANSWER;
+
+    /*
+     * use a default hints structure if one is not available.
+     */
+    if (hints == NULL) {
+        memset(&default_hints, 0, sizeof(default_hints));
+        cur_hints = &default_hints;
+    } else {
+        cur_hints = hints;
+    }
+
+    /*
+     * Check that at least one of nodename or servname is non-NULL
+     */
+    if (NULL == nodename && NULL == servname) {
+        retval = EAI_NONAME;
+        goto done;
+    }
+
+    /*
+     * if nodename is blank and hints includes ipv4 or unspecified,
+     * use IPv4 localhost 
+     */
+    if (NULL == nodename &&
+        (AF_INET == cur_hints->ai_family ||
+         AF_UNSPEC == cur_hints->ai_family)
+        ) {
+        nname = localhost4;
+    }
+
+    /*
+     * check for IPv4 addresses 
+     */
+    if (NULL != nname && INET_PTON(AF_INET, nname, ((struct sockaddr *)&sa), &addrlen4) > 0) {
+
+        struct sockaddr_in *saddr4 =
+            (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
+        if (saddr4 == NULL) {
+            retval = EAI_MEMORY;
+            goto done;
+        }
+        ainfo4 =
+            (struct addrinfo *) malloc(sizeof(struct addrinfo));
+        if (ainfo4 == NULL) {
+            free(saddr4);
+            retval = EAI_MEMORY;
+            goto done;
+        }
+
+        memset(ainfo4, 0, sizeof(struct addrinfo));
+        memset(saddr4, 0, sizeof(struct sockaddr_in));
+
+        saddr4->sin_family = AF_INET;
+        ainfo4->ai_family = AF_INET;
+        memcpy(&(saddr4->sin_addr), &sa.sin_addr, sizeof(struct in_addr));
+        ainfo4->ai_addr = (struct sockaddr *) saddr4;
+        saddr4 = NULL;
+        ainfo4->ai_addrlen = sizeof(struct sockaddr_in);
+        ainfo4->ai_canonname = NULL;
+
+        if ((retval = process_service_and_hints(servname, cur_hints, &ainfo4))
+            != 0) {
+            freeaddrinfo(ainfo4);
+            val_log(ctx, LOG_INFO, 
+                    "val_getaddrinfo(): Failed in process_service_and_hints()");
+            goto done;
+        }
+
+        *res = ainfo4;
+        retval = 0;
+    }
+
+    /*
+     * if nodename is blank and hints includes ipv6 or unspecified,
+     * use IPv6 localhost 
+     */
+    if (NULL == nodename &&
+        (AF_INET6 == cur_hints->ai_family ||
+         AF_UNSPEC == cur_hints->ai_family)
+        ) {
+        nname = localhost6;
+    }
+
+#ifdef VAL_IPV6
+    /*
+     * Check for IPv6 address 
+     */
+    if (nname != NULL && INET_PTON(AF_INET6, nname, ((struct sockaddr *)&sa6), &addrlen6) > 0) {
+
+        struct sockaddr_in6 *saddr6 =
+            (struct sockaddr_in6 *) malloc(sizeof(struct sockaddr_in6));
+        if (saddr6 == NULL) {
+            retval = EAI_MEMORY;
+            goto done;
+        }
+        ainfo6 =
+            (struct addrinfo *) malloc(sizeof(struct addrinfo));
+        if (ainfo6 == NULL) {
+            free(saddr6);
+            retval = EAI_MEMORY;
+            goto done;
+        }
+
+        memset(ainfo6, 0, sizeof(struct addrinfo));
+        memset(saddr6, 0, sizeof(struct sockaddr_in6));
+
+        saddr6->sin6_family = AF_INET6;
+        ainfo6->ai_family = AF_INET6;
+        memcpy(&(saddr6->sin6_addr), &sa6.sin6_addr, sizeof(struct in6_addr));
+        ainfo6->ai_addr = (struct sockaddr *) saddr6;
+        saddr6 = NULL;
+        ainfo6->ai_addrlen = sizeof(struct sockaddr_in6);
+        ainfo6->ai_canonname = NULL;
+
+        if ((retval = process_service_and_hints(servname, cur_hints, &ainfo6))
+            != 0) {
+            freeaddrinfo(ainfo6);
+            val_log(ctx, LOG_INFO, 
+                    "val_getaddrinfo(): Failed in process_service_and_hints()");
+            goto done;
+        }
+
+        if (NULL != *res) {
+            *res = append_addrinfo(*res, ainfo6);
+        } else {
+            *res = ainfo6;
+        }
+        retval = 0;
+    }
+#endif
+
+    if (*res) {
+        *val_status = VAL_TRUSTED_ANSWER;
+        goto done;
+    } 
+    if (nodename) {
+
+        /*
+         * If nodename was specified and was not an IPv4 or IPv6
+         * address, get its information from local store or from dns
+         * or return error if AI_NUMERICHOST specified
+         */
+        if (cur_hints->ai_flags & AI_NUMERICHOST) {
+            retval = EAI_NONAME;
+            goto done;
+        }
+        /*
+         * First check ETC_HOSTS file
+         * * XXX: TODO check the order in the ETC_HOST_CONF file
+         */
+        if (get_addrinfo_from_etc_hosts(ctx, nodename, servname,
+                                        cur_hints, res) == EAI_SERVICE) {
+            retval = EAI_SERVICE;
+        } else if (*res != NULL) {
+            retval = 0;
+            *val_status = local_ans_status;
+        }
+
+        /*
+         * Try DNS
+         */
+        else {
+            /*
+             * should never get EAI_AGAIN for local resources, so use it
+             * to singal the user to try DNS.
+             */
+            retval = EAI_AGAIN;
+        }
+    }
+
+done:
+    return retval;
+}                               /* _getaddrinfo_local */
+
 /**
  * val_getaddrinfo: A validating getaddrinfo function.
  *                  Based on getaddrinfo() as defined in RFC3493.
@@ -820,213 +1064,25 @@ val_getaddrinfo(val_context_t * context,
                 const struct addrinfo *hints, struct addrinfo **res,
                 val_status_t *val_status)
 {
-    struct in_addr  ip4_addr;
-    struct in6_addr ip6_addr;
-    struct addrinfo *ainfo4 = NULL;
-    struct addrinfo *ainfo6 = NULL;
-    int             is_ip4 = 0;
-    int             is_ip6 = 0;
-    int             retval = 0;
-    const char     *localhost4 = "127.0.0.1";
-    const char     *localhost6 = "::1";
-    const char     *nname = nodename;
-    struct addrinfo default_hints;
-    const struct addrinfo *cur_hints;
-    val_status_t local_ans_status = VAL_OOB_ANSWER;
-    int trusted = 0;
     val_context_t *ctx = NULL;
-    
-    if (context == NULL) {
-        if (VAL_NO_ERROR != (retval = val_create_context(NULL, &ctx))) {
-            val_log(ctx, LOG_DEBUG, "val_getaddrinfo: context could not be created");
-            return EAI_FAIL; 
-        } 
-    } else {
-        ctx = context;
-    }
-    val_log(ctx, LOG_DEBUG,
-            "val_getaddrinfo called with nodename = %s, servname = %s",
-            nodename == NULL ? "(null)" : nodename,
-            servname == NULL ? "(null)" : servname);
+    int            retval;
 
+    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
+    if (ctx == NULL) 
+        return EAI_FAIL;
 
-    if (VAL_NO_ERROR == val_is_local_trusted(ctx, &trusted)) {
-        if (trusted) {
-            local_ans_status = VAL_TRUSTED_ANSWER;
-        }
-    }
-    if (res == NULL || val_status == NULL)
-        return 0;
+    /** try local sources first */
+    retval = _getaddrinfo_local(context, nodename, servname, hints, res,
+                                val_status);
 
-    *res = NULL;
-    *val_status = VAL_UNTRUSTED_ANSWER;
+    if (EAI_AGAIN == retval) { /* EAI_AGAIN from local means try DNS */
 
-    /*
-     * use a default hints structure if one is not available.
-     */
-    if (hints == NULL) {
-        memset(&default_hints, 0, sizeof(default_hints));
-        cur_hints = &default_hints;
-    } else {
-        cur_hints = hints;
-    }
-
-    /*
-     * Check that at least one of nodename or servname is non-NULL
-     */
-    if (NULL == nodename && NULL == servname) {
-        return EAI_NONAME;
-    }
-
-    memset(&ip4_addr, 0, sizeof(struct in_addr));
-    memset(&ip6_addr, 0, sizeof(struct in6_addr));
-
-    /*
-     * if nodename is blank or hints includes ipv4 or unspecified,
-     * use IPv4 localhost 
-     */
-    if (NULL == nodename &&
-        (AF_INET == cur_hints->ai_family ||
-         AF_UNSPEC == cur_hints->ai_family)
-        ) {
-        nname = localhost4;
-    }
-
-    /*
-     * check for IPv4 addresses 
-     */
-    if (NULL != nname && inet_pton(AF_INET, nname, &ip4_addr) > 0) {
-
-        struct sockaddr_in *saddr4 =
-            (struct sockaddr_in *) malloc(sizeof(struct sockaddr_in));
-        if (saddr4 == NULL) {
-            return EAI_MEMORY;
-        }
-        ainfo4 =
-            (struct addrinfo *) malloc(sizeof(struct addrinfo));
-        if (ainfo4 == NULL) {
-            free(saddr4);
-            return EAI_MEMORY;
-        }
-
-        is_ip4 = 1;
-
-        memset(ainfo4, 0, sizeof(struct addrinfo));
-        memset(saddr4, 0, sizeof(struct sockaddr_in));
-
-        saddr4->sin_family = AF_INET;
-        ainfo4->ai_family = AF_INET;
-        memcpy(&(saddr4->sin_addr), &ip4_addr, sizeof(struct in_addr));
-        ainfo4->ai_addr = (struct sockaddr *) saddr4;
-        saddr4 = NULL;
-        ainfo4->ai_addrlen = sizeof(struct sockaddr_in);
-        ainfo4->ai_canonname = NULL;
-
-        if ((retval = process_service_and_hints(servname, cur_hints, &ainfo4))
-            != 0) {
-            freeaddrinfo(ainfo4);
-            val_log(ctx, LOG_INFO, 
-                    "val_getaddrinfo(): Failed in process_service_and_hints()");
-            return retval;
-        }
-
-        *res = ainfo4;
-        retval = 0;
-    }
-
-    /*
-     * if nodename is blank and hints includes ipv6 or unspecified,
-     * use IPv6 localhost 
-     */
-    if (NULL == nodename &&
-        (AF_INET6 == cur_hints->ai_family ||
-         AF_UNSPEC == cur_hints->ai_family)
-        ) {
-        nname = localhost6;
-    }
-
-    /*
-     * Check for IPv6 address 
-     */
-    if (nname != NULL && inet_pton(AF_INET6, nname, &ip6_addr) > 0) {
-
-        struct sockaddr_in6 *saddr6 =
-            (struct sockaddr_in6 *) malloc(sizeof(struct sockaddr_in6));
-        if (saddr6 == NULL) {
-            return EAI_MEMORY;
-        }
-        ainfo6 =
-            (struct addrinfo *) malloc(sizeof(struct addrinfo));
-        if (ainfo6 == NULL) {
-            free(saddr6);
-            return EAI_MEMORY;
-        }
-
-        is_ip6 = 1;
-
-        memset(ainfo6, 0, sizeof(struct addrinfo));
-        memset(saddr6, 0, sizeof(struct sockaddr_in6));
-
-        saddr6->sin6_family = AF_INET6;
-        ainfo6->ai_family = AF_INET6;
-        memcpy(&(saddr6->sin6_addr), &ip6_addr, sizeof(struct in6_addr));
-        ainfo6->ai_addr = (struct sockaddr *) saddr6;
-        saddr6 = NULL;
-        ainfo6->ai_addrlen = sizeof(struct sockaddr_in6);
-        ainfo6->ai_canonname = NULL;
-
-        if ((retval = process_service_and_hints(servname, cur_hints, &ainfo6))
-            != 0) {
-            freeaddrinfo(ainfo6);
-            val_log(ctx, LOG_INFO, 
-                    "val_getaddrinfo(): Failed in process_service_and_hints()");
-            return retval;
-        }
-
-        if (NULL != *res) {
-            *res = append_addrinfo(*res, ainfo6);
-        } else {
-            *res = ainfo6;
-        }
-        retval = 0;
-    }
-
-    if (*res) {
-        *val_status = VAL_TRUSTED_ANSWER;
-        return retval;
-    } 
-    if (nodename) {
-
-        /*
-         * If nodename was specified and was not an IPv4 or IPv6
-         * address, get its information from local store or from dns
-         * or return error if AI_NUMERICHOST specified
-         */
-        if (cur_hints->ai_flags & AI_NUMERICHOST)
-            return EAI_NONAME;
-        /*
-         * First check ETC_HOSTS file
-         * * XXX: TODO check the order in the ETC_HOST_CONF file
-         */
-        if (get_addrinfo_from_etc_hosts(ctx, nodename, servname,
-                                        cur_hints, res) == EAI_SERVICE) {
-            retval = EAI_SERVICE;
-        } else if (*res != NULL) {
-            retval = 0;
-            *val_status = local_ans_status;
-        }
-
-        /*
-         * Try DNS
-         */
-        else {
-            retval = get_addrinfo_from_dns(ctx, nodename, servname,
-                                           cur_hints, res, val_status);
-        }
+        retval = get_addrinfo_from_dns(ctx, nodename, servname,
+                                       hints, res, val_status);
     }
 
+    CTX_UNLOCK_POL(ctx); /* when was it locked? */
     return retval;
-
 }                               /* val_getaddrinfo() */
 
 
@@ -1159,7 +1215,7 @@ address_to_string(const u_char *saddr, int family, char *nadd, int nlen)
 int
 val_getnameinfo(val_context_t * context,
                 const struct sockaddr *sa,
-                socklen_t salen,
+                size_t salen,
                 char *host,
                 size_t hostlen,
                 char *serv,
@@ -1169,20 +1225,16 @@ val_getnameinfo(val_context_t * context,
     char            domain_string[NS_MAXDNAME], number_string[NS_MAXDNAME];
     const u_char   *theAddress = NULL;
     int             theAddressFamily;
-    int             val_rnc_status = 0, ret_status = 0;
+    int             retval = 0, ret_status = 0;
 
     struct val_answer_chain *res;
     struct val_answer_chain *val_res = NULL;
-    int retval;
     val_context_t *ctx = NULL;
     
-    if (context == NULL) {
-        if (VAL_NO_ERROR != (retval = val_create_context(NULL, &ctx))) {
-            return EAI_FAIL; 
-        } 
-    } else {
-        ctx = context;
-    }
+    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
+    if (ctx == NULL)
+        return EAI_FAIL;;
+
     val_log(ctx, LOG_DEBUG,
             "val_getnameinfo called with host = %s, serv = %s",
             host == NULL ? "(null)" : host,
@@ -1195,11 +1247,15 @@ val_getnameinfo(val_context_t * context,
      * check misc parameters, there should be at least one of host or
      * server, check if flags indicate host is required 
      */
-    if (!val_status || !sa)
-      return EAI_FAIL;
+    if (!val_status || !sa) {
+      retval = EAI_FAIL;
+      goto done;
+    }
 
-    if (!host && !serv) 
-      return EAI_NONAME;
+    if (!host && !serv) {
+      retval = EAI_NONAME;
+      goto done;
+    }
     
     /*
      * should the services be looked up 
@@ -1209,11 +1265,16 @@ val_getnameinfo(val_context_t * context,
         u_int16_t port;
         if (sa->sa_family == AF_INET) {
             port = ((const struct sockaddr_in*)sa)->sin_port;
-        } else if (sa->sa_family == AF_INET6) {
+        } 
+#ifdef VAL_IPV6
+        else if (sa->sa_family == AF_INET6) {
             port = ((const struct sockaddr_in6*)sa)->sin6_port;
-        } else {
+        } 
+#endif
+        else {
             val_log(ctx, LOG_DEBUG, "val_getnameinfo(): Address family %d not known.", sa->sa_family);
-            return EAI_FAMILY;
+            retval = EAI_FAMILY;
+            goto done;
         }
 
         val_log(ctx, LOG_DEBUG, 
@@ -1242,7 +1303,8 @@ val_getnameinfo(val_context_t * context,
      */
     if (!host || hostlen == 0) {
         *val_status = VAL_TRUSTED_ANSWER;
-        return 0;
+        retval = 0;
+        goto done;
     }
 
     /*
@@ -1252,7 +1314,9 @@ val_getnameinfo(val_context_t * context,
         theAddress =
             (const u_char *) &((const struct sockaddr_in *) sa)->sin_addr;
         theAddressFamily = AF_INET;
-    } else if (AF_INET6 == sa->sa_family
+    } 
+#ifdef VAL_IPV6
+    else if (AF_INET6 == sa->sa_family
                && salen >= sizeof(struct sockaddr_in6)) {
         static const u_char _ipv6_wrapped_ipv4[] = {
             0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -1269,9 +1333,12 @@ val_getnameinfo(val_context_t * context,
         else
             theAddressFamily = AF_INET6;
             
-    } else {
+    } 
+#endif
+    else {
         val_log(ctx, LOG_DEBUG, "val_getnameinfo(): Address family %d not known or length %d too small.", sa->sa_family, salen);
-        return (EAI_FAMILY);
+        retval = EAI_FAMILY;
+        goto done;
     }
 
     /*
@@ -1288,7 +1355,8 @@ val_getnameinfo(val_context_t * context,
         (0 != (ret_status =
              address_to_reverse_domain(theAddress, theAddressFamily,
                       domain_string, sizeof(domain_string))))) {
-            return ret_status;
+            retval = ret_status;
+            goto done;
     }
 
     /*
@@ -1301,7 +1369,8 @@ val_getnameinfo(val_context_t * context,
     if ((flags & NI_NUMERICHOST) && !(flags & NI_NAMEREQD)) {
         *val_status = VAL_TRUSTED_ANSWER;
         val_log(ctx, LOG_DEBUG, "val_getnameinfo(): returning host (%s)", host);
-        return 0;
+        retval = 0;
+        goto done;
     }
 
     val_log(ctx, LOG_DEBUG, "val_getnameinfo(): val_get_rrset host flags(%d)", flags);
@@ -1316,16 +1385,18 @@ val_getnameinfo(val_context_t * context,
                 "val_getnameinfo(): val_get_rrset failed - %s", 
                 p_val_err(retval));
         *val_status = VAL_UNTRUSTED_ANSWER;
-        return EAI_FAIL;
+        retval = EAI_FAIL;
+        goto done;
     }
 
     if (!val_res) {
         val_log(ctx, LOG_ERR, "val_getnameinfo(): EAI_MEMORY");
         *val_status = VAL_UNTRUSTED_ANSWER;
-        return EAI_MEMORY;
+        retval = EAI_MEMORY;
+        goto done;
     }
 
-    val_rnc_status = 0;
+    retval = 0;
 
     for (res = val_res; res; res=res->val_ans_next) {
         /* set the value of merged trusted and validated status values */
@@ -1346,9 +1417,9 @@ val_getnameinfo(val_context_t * context,
         } else if (val_does_not_exist(res->val_ans_status)) {
             if ((res->val_ans_status == VAL_NONEXISTENT_TYPE) ||
                     (res->val_ans_status == VAL_NONEXISTENT_TYPE_NOCHAIN)) {
-                val_rnc_status = EAI_NODATA;
+                retval = EAI_NODATA;
             } else { 
-                val_rnc_status = EAI_NONAME;
+                retval = EAI_NONAME;
             }
             break;
         } 
@@ -1358,10 +1429,328 @@ val_getnameinfo(val_context_t * context,
     val_log(ctx, LOG_DEBUG,
           "val_getnameinfo(): val_get_rrset for host %s, returned %s with lookup status %d and validator status %d : %s",
           domain_string, host,
-          val_rnc_status, 
+          retval, 
           *val_status, p_val_status(*val_status));
 
-
-    return val_rnc_status;
+done:
+    CTX_UNLOCK_POL(ctx);
+    return retval;
 
 }                               // val_getnameinfo
+
+/*
+ * define error codes for val_getaddrinfo and val_getnameinfo which
+ * have a DNSSEC validation status.
+ */
+int val_getaddrinfo_has_status(int rc) {
+#if defined(EAI_NODATA) && defined(EAI_NONAME)
+    return ((rc == 0) || (rc == EAI_NONAME) || (rc == EAI_NODATA));
+#elif defined(EAI_NONAME)
+    return ((rc == 0) || (rc == EAI_NONAME));
+#elif defined(WSAHOST_NOT_FOUND) && defined(WSANO_DATA) 
+	return ((rc == 0) || (rc == WSAHOST_NOT_FOUND) || (rc == WSANO_DATA));
+#else
+    return (rc == 0);
+#endif
+}
+
+/** ====================================================================== **/
+/** ====================================================================== **/
+/** ====================================================================== **/
+/** ====================================================================== **/
+/** ====================================================================== **/
+
+#ifndef VAL_NO_ASYNC
+
+#define VAL_GAI_DONE         0x0001
+#define VAL_GAI_FREE_CONTEXT 0x0002
+
+struct val_gai_status_s {
+    char             *nodename;
+    char             *servname;
+    struct addrinfo  *hints;
+
+    val_context_t    *context;
+    struct addrinfo  *res;
+    val_status_t      val_status;
+    val_async_status *inet_status;
+    val_async_status *inet6_status;
+
+    u_int             flags;
+    val_gai_callback  callback;
+    void             *callback_data;
+};
+
+static void
+_free_vgai( val_gai_status *vgai )
+{
+    if (NULL == vgai)
+        return;
+
+    if (NULL != vgai->inet_status) {
+        val_async_cancel(vgai->context, vgai->inet_status, 
+                         VAL_AS_CANCEL_NO_CALLBACKS);
+        vgai->inet_status = NULL;
+    }
+
+    if (NULL != vgai->inet6_status) {
+        val_async_cancel(vgai->context, vgai->inet6_status,
+                         VAL_AS_CANCEL_NO_CALLBACKS);
+        vgai->inet6_status = NULL;
+    }
+
+    if (NULL != vgai->nodename) {
+        free(vgai->nodename);
+        vgai->nodename = NULL;
+    }
+
+    if (NULL != vgai->servname) {
+        free(vgai->servname);
+        vgai->servname = NULL;
+    }
+
+    if (NULL != vgai->res) {
+        freeaddrinfo(vgai->res);
+        vgai->res = NULL;
+    }
+
+    if (vgai->flags & VAL_GAI_FREE_CONTEXT) {
+        val_free_context(vgai->context);
+        vgai->context = NULL;
+    }
+
+    free(vgai);
+}
+
+/*
+ * our internal callback for request completion, which in turns calls
+ * original caller vgai_callback.
+ */
+static int
+_vgai_async_callback(val_async_status *as, int event,
+                     val_context_t *ctx, void *cb_data, val_cb_params_t *cbp)
+{
+    int                rc;
+    val_gai_status    *vgai;
+
+    val_log(ctx, LOG_DEBUG, "val_getaddrinfo async callback for %p", as);
+
+    vgai = (val_gai_status *)cb_data;
+    if (NULL == vgai) {
+        val_log(ctx, LOG_DEBUG, "val_getaddrinfo no callback data!");
+        return VAL_NO_ERROR;
+    }
+
+    if (0 == vgai->val_status) /* recently created */
+        vgai->val_status = VAL_VALIDATED_ANSWER;
+
+    /*
+     * get answer_chain from result_chain
+     */
+    rc = val_get_answer_from_result(ctx, vgai->nodename, cbp->class_h,
+                                    cbp->type_h, &cbp->results, &cbp->answers,
+                                    0);
+    if (VAL_NO_ERROR != rc) {
+        val_log(ctx, LOG_DEBUG,
+                "val_gai_callback: val_get_answer_from_result() returned=%d", rc);
+    }
+    else {
+        /*
+         * get addrinfo from results
+         */
+        rc = get_addrinfo_from_result(ctx, cbp->answers, vgai->servname,
+                                      vgai->hints, &vgai->res,
+                                      &vgai->val_status);
+        if (VAL_NO_ERROR == rc) {
+            val_log(ctx, LOG_DEBUG,
+                    "val_gai_callback get_addrinfo_from_result() returned=%d with val_status=%d",
+                    rc, vgai->val_status);
+        }
+    }
+
+    /*
+     * clear async_status ptr, as it will be freed after this callback 
+     * completes.
+     */
+    if (ns_t_a == cbp->type_h) {
+        vgai->inet_status = NULL;
+        if (rc != VAL_NO_ERROR) {
+            if (vgai->inet6_status) {
+                val_async_cancel(vgai->context, vgai->inet6_status,
+                                 VAL_AS_CANCEL_NO_CALLBACKS);
+                vgai->inet6_status = NULL;
+            }
+        }
+    }
+    else if (ns_t_aaaa == cbp->type_h) {
+        vgai->inet6_status = NULL;
+        if (rc != VAL_NO_ERROR) {
+            if (vgai->inet_status) {
+                val_async_cancel(vgai->context, vgai->inet_status,
+                                 VAL_AS_CANCEL_NO_CALLBACKS);
+                vgai->inet_status = NULL;
+            }
+        }
+    }
+
+    /*
+     * if both async_status ptr are NULL, we're done and should call
+     * our callback.
+     */
+    if (NULL != vgai->inet6_status || NULL != vgai->inet_status)
+        return VAL_NO_ERROR;
+
+    if (NULL == vgai->callback) {
+        val_log(ctx, LOG_DEBUG, "val_getaddrinfo async NULL callback!");
+    } else {
+        (*vgai->callback)(vgai->callback_data, rc, vgai->res, vgai->val_status);
+        /** let caller keep res structure */
+        vgai->res = NULL;
+    }
+
+    _free_vgai(vgai);
+
+    return VAL_NO_ERROR;
+}
+
+/*
+ * Function: val_getaddrinfo_submit
+ *
+ * Purpose: submit asynchronous request for getaddrinfo style resoloution
+ *
+ * Parameters: context -- context for request. May be NULL, in which case
+ *                        the default context will be used.
+ *             nodename, servname, hints_in -- see getaddrinfo man page
+ *             callback -- val_gai_callback function which will be called when
+ *                         the request completes
+ *             callback_data -- optional void pointer which will also be
+ *                              passed to the callback. Not used internally,
+ *                              strictly for caller to pass context to callback.
+ *             val_gai_flags -- flags affecting function operation
+ *             status -- opaque val_gai_status object which caller can use to
+ *                       cancel request
+ *
+ * Returns: VAL_NO_ERROR -- successful submission
+ *          VAL_BAD_ARGUMENT -- need callback and nodename or servname
+ *
+ *          May return other validator errors from internal function calls
+ *
+ * Note: caller assumes responsibility for the memory for the addrinfo
+ *       linked list passed to the callback function.
+ */
+int
+val_getaddrinfo_submit(val_context_t * context, const char *nodename,
+                       const char *servname, const struct addrinfo *hints_in,
+                       val_gai_callback callback, void *callback_data,
+                       u_int val_gai_flags, val_gai_status **status)
+{
+    val_gai_status        *vgai = NULL;
+    int                    vretval = VAL_NO_ERROR, rc;
+    struct addrinfo       *res = NULL;
+    const struct addrinfo *hints;
+    struct addrinfo        default_hints;
+    val_context_t         *ctx;
+    val_status_t           val_status;
+
+    if (((NULL == nodename) && (NULL == servname)) || (NULL == callback))
+        return VAL_BAD_ARGUMENT;
+
+    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
+    if (ctx == NULL)
+        return VAL_INTERNAL_ERROR;
+    else if (ctx != context) /* we allocated our own context */
+        val_gai_flags |= VAL_GAI_FREE_CONTEXT;
+
+    /*
+     * use a default hints structure if one is not available.
+     */
+    if (hints_in == NULL) {
+        memset(&default_hints, 0, sizeof(default_hints));
+        hints = &default_hints;
+    } else {
+        hints = hints_in;
+    }
+
+    /** check we have a supported family */
+    if (hints->ai_family != AF_UNSPEC &&
+        hints->ai_family != AF_INET && hints->ai_family != AF_INET6) {
+
+        (*callback)(callback_data, EAI_NONAME, NULL, VAL_UNTRUSTED_ANSWER);
+        goto done;
+    }
+
+    /** try local sources first */
+    rc = _getaddrinfo_local(ctx, nodename, servname, hints, &res, &val_status);
+    if (EAI_AGAIN != rc) {
+        /** _getaddrinfo_local returns eai rc, not libval */
+        (*callback)(callback_data, rc, res, val_status);
+        goto done;
+    }
+
+    vgai = (val_gai_status *)calloc(1, sizeof(val_gai_status));
+    if (NULL == vgai) {
+        vretval = VAL_ENOMEM;
+        goto done;
+    }
+
+    vgai->context = ctx;
+    vgai->flags = val_gai_flags;
+    vgai->callback = callback;
+    vgai->callback_data = callback_data;
+    if (servname)
+        vgai->servname = (char *)strdup(servname);
+    if (nodename)
+        vgai->nodename = (char *)strdup(nodename);
+
+    /*
+     * Check if we need to return IPv4 addresses based on the hints
+     */
+    if (hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET) {
+
+        val_log(ctx, LOG_DEBUG,
+                "val_getaddrinfo_submit(): checking for A records");
+
+        rc = val_async_submit(ctx, nodename, ns_c_in, ns_t_a, 0,
+                              &_vgai_async_callback, vgai, &vgai->inet_status);
+        if (VAL_NO_ERROR != rc) {
+            vgai->flags |= VAL_GAI_DONE;
+            vretval = rc;
+            goto done;
+        }
+    }
+
+#ifdef VAL_IPV6
+    /*
+     * Check if we need to return IPv6 addresses based on the hints
+     */
+    if (hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET6) {
+
+        val_log(ctx, LOG_DEBUG,
+                "val_getaddrinfo_submit(): checking for AAAA records");
+
+        rc = val_async_submit(ctx, nodename, ns_c_in, ns_t_aaaa, 0,
+                              &_vgai_async_callback, vgai, &vgai->inet6_status);
+        if (VAL_NO_ERROR != rc) {
+            vgai->flags |= VAL_GAI_DONE;
+            vretval = rc;
+            val_async_cancel(ctx, vgai->inet_status,
+                             VAL_AS_CANCEL_NO_CALLBACKS);
+            goto done;
+        }
+        vgai->inet6_status->val_as_result_cb = &_vgai_async_callback;
+        vgai->inet6_status->val_as_cb_user_ctx = vgai;
+    }
+#endif
+
+done:
+    CTX_UNLOCK_POL(ctx);
+
+    if (VAL_NO_ERROR == vretval)
+        *status = vgai;
+    else
+        _free_vgai( vgai );
+
+    return vretval;
+}                               /* val_getaddrinfo_submit() */
+
+#endif /* VAL_NO_ASYNC */
