@@ -7,38 +7,18 @@
  * Read the contents of the resolver, validator and root.hints configuration 
  * files and update the validator context.
  */
+
 #include "validator-config.h"
+#include "validator-internal.h"
 
-#include <stdio.h>
-#include <stdarg.h>
-#include <sys/types.h>
-#include <regex.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <string.h>
-#include <strings.h>
-#include <ctype.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <fcntl.h>
-#include <sys/file.h>
-#include <resolv.h>
-#include <errno.h>
-#ifndef VAL_NO_THREADS
-#include <pthread.h>
-#endif
-
-#include <validator/resolver.h>
-#include <validator/validator.h>
-#include <validator/validator-internal.h>
 #include "val_policy.h"
 #include "val_support.h"
 #include "val_cache.h"
 #include "val_resquery.h"
 #include "val_context.h"
 #include "val_assertion.h"
+#include "val_parse.h"
+
 
 #define READ_POL_FOR_ZONE(buf_ptr, end_ptr, line_number, endst, retval, err, token)  do {\
     /*\
@@ -957,6 +937,39 @@ parse_closest_ta_target_gopt(char **buf_ptr, char *end_ptr, int *line_number,
     return VAL_NO_ERROR;
 }
 
+static int
+parse_rec_fallback(char **buf_ptr, char *end_ptr, int *line_number,
+                   int *endst, global_opt_t *g_opt)
+{
+    char            token[TOKEN_MAX];
+    int retval;
+
+    if ((buf_ptr == NULL) || (*buf_ptr == NULL) || (end_ptr == NULL) || 
+        (g_opt == NULL) || (endst == NULL) || (line_number == NULL))
+        return VAL_BAD_ARGUMENT;
+
+    /* read the next token */
+    if (VAL_NO_ERROR != (retval = 
+        val_get_token(buf_ptr, end_ptr, line_number, 
+                      token, sizeof(token), endst,
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
+        return retval;
+    }
+    if ((endst && (strlen(token) == 0)) ||
+        (*buf_ptr >= end_ptr)) { 
+        return VAL_CONF_PARSE_ERROR;
+    }
+
+    if (!strncmp(token, GOPT_YES_STR, strlen(GOPT_YES_STR))) {
+        g_opt->rec_fallback = 1;
+    } else if (!strncmp(token, GOPT_NO_STR, strlen(GOPT_NO_STR))) {
+        g_opt->rec_fallback = 0;
+    } else {
+        return VAL_CONF_PARSE_ERROR;
+    }
+    return VAL_NO_ERROR;
+}
+
 static int
 get_global_options(char **buf_ptr, char *end_ptr, 
                    int *line_number, global_opt_t **g_opt) 
@@ -978,6 +991,7 @@ get_global_options(char **buf_ptr, char *end_ptr,
     (*g_opt)->app_policy = VAL_POL_GOPT_DISABLE;
     (*g_opt)->log_target = NULL;
     (*g_opt)->closest_ta_only = 0;
+    (*g_opt)->rec_fallback = 1;
     
     while (!endst) {
         /*
@@ -1036,6 +1050,13 @@ get_global_options(char **buf_ptr, char *end_ptr,
                 goto err;
             }
 
+        } else if (!strcmp(token, GOPT_REC_FALLBACK)) {
+            if (VAL_NO_ERROR != 
+                    (retval = parse_rec_fallback(buf_ptr, end_ptr, 
+                                                 line_number, &endst, *g_opt))) {
+                goto err;
+            }
+
         } else {
             retval = VAL_CONF_PARSE_ERROR;
             goto err;
@@ -1392,6 +1413,12 @@ destroy_valpol(val_context_t * ctx)
 #ifdef solaris2
 #define getprogname() getexecname()
 #endif
+#ifdef WIN32
+#define getprogname() NULL
+#endif
+#ifdef __OpenBSD__
+#define getprogname() NULL
+#endif
 static int
 read_next_val_config_file(val_context_t *ctx, 
                           char **label, 
@@ -1402,7 +1429,9 @@ read_next_val_config_file(val_context_t *ctx,
                           global_opt_t **g_opt)
 {
     int    fd = -1;
+#ifdef HAVE_FLOCK
     struct flock    fl;
+#endif
     struct stat sb;
     char token[TOKEN_MAX];
     int endst = 0;
@@ -1443,9 +1472,11 @@ read_next_val_config_file(val_context_t *ctx,
             return VAL_NO_ERROR;
         }
     }
+#ifdef HAVE_FLOCK
     memset(&fl, 0, sizeof(fl));
     fl.l_type = F_RDLCK;
     fcntl(fd, F_SETLKW, &fl);
+#endif
 
     if (0 != fstat(fd, &sb)) {
         val_log(ctx, LOG_ERR, 
@@ -1698,8 +1729,10 @@ read_next_val_config_file(val_context_t *ctx,
     *label = next_label;
     FREE(buf);
     buf = NULL;
+#ifdef HAVE_FLOCK
     fl.l_type = F_UNLCK;
     fcntl(fd, F_SETLKW, &fl);
+#endif
     close(fd);
     fd = -1;
     return VAL_NO_ERROR;
@@ -1715,8 +1748,10 @@ err:
         FREE(buf);
     }
     if (fd != -1) {
+#ifdef HAVE_FLOCK
         fl.l_type = F_UNLCK;
         fcntl(fd, F_SETLKW, &fl);
+#endif
         close(fd);
     }
     return retval;
@@ -1743,10 +1778,8 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
     if (ctx == NULL || is_override == NULL)
         return VAL_BAD_ARGUMENT;
     
-    CTX_LOCK_VALPOL_SH(ctx);
     if (ctx->dnsval_l && ctx->dnsval_l->dnsval_conf)
         base_dnsval_conf = strdup(ctx->dnsval_l->dnsval_conf);
-    CTX_UNLOCK_VALPOL(ctx);
     
     if (base_dnsval_conf == NULL) 
         return VAL_OUT_OF_MEMORY;
@@ -1810,20 +1843,20 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
             destroy_valpolovr(&overrides->next);
     } else { 
         /* clone the label */
-        newctxlab = (char *) MALLOC (strlen(label) + 1);
+        newctxlab = strdup(label); 
         if (newctxlab == NULL) {
             retval = VAL_OUT_OF_MEMORY;
             goto err;
         }
-        strcpy(newctxlab, label);
     }
 
     if (label != scope) {
         /* check if the pointer has changed */
         *is_override = 1;
     }
-        
-    CTX_LOCK_VALPOL_EX(ctx);
+
+    CTX_UNLOCK_POL(ctx);
+    CTX_LOCK_POL_EX(ctx);
 
     if (ctx->label)
         FREE(ctx->label);
@@ -1868,12 +1901,13 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
      * Re-initialize caches 
      */
     for(q = ctx->q_list; q; q=q->qc_next) {
-        clear_query_chain_structure(q);
+        q->qc_flags |= VAL_QUERY_REFRESH_QCACHE;
     }
 
     ctx->dnsval_l = dlist;
 
-    CTX_UNLOCK_VALPOL(ctx);
+    CTX_UNLOCK_POL(ctx);
+    CTX_LOCK_POL_SH(ctx);
 
     val_log(ctx, LOG_DEBUG, "read_val_config_file(): Done reading validator configuration");
 
@@ -1907,7 +1941,9 @@ read_res_config_file(val_context_t * ctx)
 {
     char           *resolv_config;
     int             fd;
+#ifdef HAVE_FLOCK
     struct flock    fl;
+#endif
     char            token[TOKEN_MAX];
     int             line_number = 0;
     int             endst = 0;
@@ -1950,11 +1986,13 @@ read_res_config_file(val_context_t * ctx)
         if (fd == -1) {
             val_log(ctx, LOG_ERR, "read_res_config_file(): Could not open default resolver conf file for reading: %s",
                     resolv_conf);
+            return VAL_CONF_NOT_FOUND;
         }
-        return VAL_CONF_NOT_FOUND;
     }
+#ifdef HAVE_FLOCK
     fl.l_type = F_RDLCK;
     fcntl(fd, F_SETLKW, &fl);
+#endif
 
     if (0 != fstat(fd, &sb)) {
         retval = VAL_CONF_NOT_FOUND;
@@ -2012,8 +2050,9 @@ read_res_config_file(val_context_t * ctx)
                 val_log(ctx, LOG_WARNING,
 			"read_res_config_file(): error parsing nameserver token!");
                 goto err;
-	    }
+	        }
             if (ns != NULL) {
+                ns->ns_options |= RES_RECURSE;
                 if (ns_tail == NULL) {
                     ns_head = ns;
                     ns_tail = ns;
@@ -2071,8 +2110,10 @@ read_res_config_file(val_context_t * ctx)
 
     FREE(buf);
   empty:
+#ifdef HAVE_FLOCK
     fl.l_type = F_UNLCK;
     fcntl(fd, F_SETLKW, &fl);
+#endif
     close(fd);
 
     /*
@@ -2086,11 +2127,13 @@ read_res_config_file(val_context_t * ctx)
         }
     } 
 
-    CTX_LOCK_RESPOL_EX(ctx);
+    CTX_UNLOCK_POL(ctx);
+    CTX_LOCK_POL_EX(ctx);
     destroy_respol(ctx);
     ctx->nslist = ns_head;
     ctx->r_timestamp = sb.st_mtime;
-    CTX_UNLOCK_RESPOL(ctx);
+    CTX_UNLOCK_POL(ctx);
+    CTX_LOCK_POL_SH(ctx);
 
     val_log(ctx, LOG_DEBUG, 
             "read_res_config_file(): Done reading resolver configuration");
@@ -2105,8 +2148,10 @@ read_res_config_file(val_context_t * ctx)
         FREE(buf);
 
     if (fd != -1) {
+#ifdef HAVE_FLOCK
         fl.l_type = F_UNLCK;
         fcntl(fd, F_SETLKW, &fl);
+#endif
         close(fd);
     }
 
@@ -2121,7 +2166,9 @@ read_root_hints_file(val_context_t * ctx)
 {
     struct rrset_rec *root_info = NULL;
     int             fd;
+#ifdef HAVE_FLOCK
     struct flock    fl;
+#endif
     char            token[TOKEN_MAX];
     char           *root_hints;
     u_char          zone_n[NS_MAXCDNAME];
@@ -2164,8 +2211,10 @@ read_root_hints_file(val_context_t * ctx)
                 root_hints);
         return VAL_NO_ERROR;
     }
+#ifdef HAVE_FLOCK
     fl.l_type = F_RDLCK;
     fcntl(fd, F_SETLKW, &fl);
+#endif
 
     if (0 != fstat(fd, &sb)) { 
         retval = VAL_CONF_NOT_FOUND;
@@ -2288,13 +2337,14 @@ read_root_hints_file(val_context_t * ctx)
             goto err;
         }
         if (type_h == ns_t_a) {
-            struct in_addr  address;
-            if (inet_pton(AF_INET, token, &address) != 1) {
+            struct sockaddr_in sa;
+            size_t addrlen4 = sizeof(struct sockaddr_in);
+            if (INET_PTON(AF_INET, token, ((struct sockaddr *)&sa), &addrlen4) != 1) {
                 retval = VAL_CONF_PARSE_ERROR;
                 goto err;
             }
             rdata_len_h = sizeof(struct in_addr);
-            memcpy(rdata_n, &address, rdata_len_h);
+            memcpy(rdata_n, &sa.sin_addr, rdata_len_h);
         } else if (type_h == ns_t_ns) {
             if (ns_name_pton(token, rdata_n, sizeof(rdata_n)) == -1) {
                 retval = VAL_CONF_PARSE_ERROR;
@@ -2310,7 +2360,7 @@ read_root_hints_file(val_context_t * ctx)
         //                        zone_n);
         rr_set = find_rr_set(NULL, &root_info, zone_n, type_h, type_h,
                              ns_c_in, (u_int32_t)ttl_h, NULL, rdata_n, VAL_FROM_UNSET,
-                             0, zone_n);
+                             0, 0, zone_n);
         if (rr_set == NULL) {
             retval = VAL_OUT_OF_MEMORY;
             goto err;
@@ -2354,19 +2404,23 @@ read_root_hints_file(val_context_t * ctx)
     }
 #endif
 
-    CTX_LOCK_RESPOL_EX(ctx);
+    CTX_UNLOCK_POL(ctx);
+    CTX_LOCK_POL_EX(ctx);
     if (ctx->root_ns)
         free_name_servers(&ctx->root_ns);
     ctx->root_ns = ns_list;
     ctx->h_timestamp = sb.st_mtime;
-    CTX_UNLOCK_RESPOL(ctx);
+    CTX_UNLOCK_POL(ctx);
+    CTX_LOCK_POL_SH(ctx);
 
     res_sq_free_rrset_recs(&root_info);
 
     val_log(ctx, LOG_DEBUG, "read_root_hints_file(): Done reading root hints");
     FREE(buf);
+#ifdef HAVE_FLOCK
     fl.l_type = F_UNLCK;
     fcntl(fd, F_SETLKW, &fl);
+#endif
     close(fd);
 
     return VAL_NO_ERROR;
@@ -2375,8 +2429,10 @@ read_root_hints_file(val_context_t * ctx)
 
     if (buf)
         FREE(buf);
+#ifdef HAVE_FLOCK
     fl.l_type = F_UNLCK;
     fcntl(fd, F_SETLKW, &fl);
+#endif
     close(fd);
     res_sq_free_rrset_recs(&root_info);
     val_log(ctx, LOG_ERR, "read_root_hints_file(): Error encountered while reading file %s - %s", 
@@ -2406,7 +2462,9 @@ parse_etc_hosts(const char *name)
     }
 
     while (fgets(line, MAX_LINE_SIZE, fp) != NULL) {
+#ifdef HAVE_STRTOK_R
         char           *buf = NULL;
+#endif
         char           *cp = NULL;
         char            addr_buf[INET6_ADDRSTRLEN];
         char           *domain_name = NULL;
@@ -2422,7 +2480,11 @@ parse_etc_hosts(const char *name)
         /*
          * ignore characters after # 
          */
+#ifdef HAVE_STRTOK_R
         cp = (char *) strtok_r(line, "#", &buf);
+#else
+	    cp = (char *) strtok(line, "#");
+#endif
 
         if (!cp)
             continue;
@@ -2433,7 +2495,11 @@ parse_etc_hosts(const char *name)
         /*
          * read the ip address 
          */
+#ifdef HAVE_STRTOK_R
         cp = (char *) strtok_r(fileentry, white, &buf);
+#else
+        cp = (char *) strtok(fileentry, white);
+#endif
         if (!cp)
             continue;
 
@@ -2443,13 +2509,17 @@ parse_etc_hosts(const char *name)
         /*
          * read the full domain name 
          */
+#ifdef HAVE_STRTOK_R
         cp = (char *) strtok_r(NULL, white, &buf);
+#else
+        cp = (char *) strtok(NULL, white);
+#endif
         if (!cp)
             continue;
 
         domain_name = cp;
 
-        if (strcasecmp(cp, name) == 0) {
+        if (strncasecmp(name, cp, strlen(cp)) == 0) {
             matchfound = 1;
         }
 
@@ -2458,9 +2528,13 @@ parse_etc_hosts(const char *name)
          */
         memset(alias_list, 0, MAX_ALIAS_COUNT);
         alias_index = 0;
+#ifdef HAVE_STRTOK_R
         while ((cp = (char *) strtok_r(NULL, white, &buf)) != NULL) {
+#else
+        while ((cp = (char *) strtok(NULL, white)) != NULL) {
+#endif
             alias_list[alias_index++] = cp;
-            if ((!matchfound) && (strcasecmp(cp, name) == 0)) {
+            if ((!matchfound) && (strncasecmp(name, cp, strlen(cp)) == 0)) {
                 matchfound = 1;
             }
         }
@@ -2475,7 +2549,7 @@ parse_etc_hosts(const char *name)
         if (hentry == NULL)
             break;              /* return results so far */
 
-        bzero(hentry, sizeof(struct hosts));
+        memset(hentry, 0, sizeof(struct hosts));
         hentry->address = (char *) strdup(addr_buf);
         hentry->canonical_hostname = (char *) strdup(domain_name);
         hentry->aliases =
@@ -2532,7 +2606,6 @@ val_add_valpolicy(val_context_t *context,
     struct val_query_chain *q;
     policy_entry_t *pol_entry;
     val_context_t *ctx = NULL;
-    int retval;
 
     libval_policy_definition_t *libval_pol;
 
@@ -2546,15 +2619,8 @@ val_add_valpolicy(val_context_t *context,
         libval_pol->value == NULL)
         return VAL_BAD_ARGUMENT;
 
-    if (context == NULL) {
-        /* Set the policy for the default context */
-        if (VAL_NO_ERROR != (retval = val_create_context(NULL, &ctx)))
-            return retval;
-    } else
-        ctx = (val_context_t *) context;
-   
     *pol = NULL;
-    
+
     /* find the policy according to the keyword */
     for (index = 0; index < MAX_POL_TOKEN; index++) {
         if (!strcmp(libval_pol->keyword, conf_elem_array[index].keyword)) {
@@ -2573,7 +2639,7 @@ val_add_valpolicy(val_context_t *context,
         gettimeofday(&tv, NULL);
         ttl_x = libval_pol->ttl + tv.tv_sec;
     } else
-        ttl_x = -1;
+        ttl_x = 0;
         
     buf_ptr = libval_pol->value;
     end_ptr = libval_pol->value+strlen(libval_pol->value);
@@ -2582,7 +2648,6 @@ val_add_valpolicy(val_context_t *context,
     if (pol_entry == NULL) {
         return VAL_OUT_OF_MEMORY;
     }
-
     memcpy(pol_entry->zone_n, zone_n, wire_name_length(zone_n));
     pol_entry->exp_ttl = ttl_x;
     pol_entry->next = NULL;
@@ -2596,36 +2661,42 @@ val_add_valpolicy(val_context_t *context,
         return VAL_BAD_ARGUMENT;
     }
 
-    /* Lock appropriately */
-    CTX_LOCK_VALPOL_EX(ctx);
-    CTX_LOCK_ACACHE(ctx);
+    *pol = (val_policy_handle_t *) MALLOC (sizeof(val_policy_handle_t));
+    if (*pol == NULL) {
+        FREE(pol_entry);
+        return VAL_OUT_OF_MEMORY;
+    }
+
+    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
+    if (ctx == NULL) { 
+        FREE(pol_entry);
+        FREE(*pol);
+        *pol = NULL;
+        return VAL_OUT_OF_MEMORY;
+    }
+   
+    /* Lock exclusively */
+    CTX_UNLOCK_POL(ctx);
+    CTX_LOCK_POL_EX(ctx);
 
     /* Flush queries that match this name */
     for(q=ctx->q_list; q; q=q->qc_next) {
         /* Should never fail when holding above locks */
         if (NULL != namename(q->qc_name_n, zone_n)) {
-            clear_query_chain_structure(q);
+            q->qc_flags |= VAL_QUERY_REFRESH_QCACHE;
             if (pol_entry->exp_ttl > 0)
                 q->qc_ttl_x = pol_entry->exp_ttl;
         }
     }
-    *pol = (val_policy_handle_t *) MALLOC (sizeof(val_policy_handle_t));
-    if (*pol == NULL) {
-        retval = VAL_OUT_OF_MEMORY;
-        goto err;
-    }
     (*pol)->pe = pol_entry;
     (*pol)->index = index;
 
     /* Merge this policy into the context */
     STORE_POLICY_ENTRY_IN_LIST(pol_entry, ctx->e_pol[index]);
-    retval = VAL_NO_ERROR;
     
-err:
-    CTX_UNLOCK_ACACHE(ctx);
-    CTX_UNLOCK_VALPOL(ctx);
-    
-    return retval;
+    CTX_UNLOCK_POL(ctx);
+
+    return VAL_NO_ERROR;
 }  
 
 int 
@@ -2639,16 +2710,13 @@ val_remove_valpolicy(val_context_t *context, val_policy_handle_t *pol)
     if (pol == NULL || pol->pe == NULL|| pol->index >= MAX_POL_TOKEN)
        return VAL_BAD_ARGUMENT; 
 
-    if (context == NULL) {
-        /* Get the policy for the default context */
-        if (VAL_NO_ERROR != (retval = val_create_context(NULL, &ctx)))
-            return retval;
-    } else
-        ctx = (val_context_t *) context;
+    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
+    if (ctx == NULL)
+        return VAL_INTERNAL_ERROR;
     
-    /* Lock appropriately */
-    CTX_LOCK_VALPOL_EX(ctx);
-    CTX_LOCK_ACACHE(ctx);
+    /* Lock exclusively */
+    CTX_UNLOCK_POL(ctx);
+    CTX_LOCK_POL_EX(ctx);
 
     /* find this policy in the context */
     prev = NULL;
@@ -2675,7 +2743,7 @@ val_remove_valpolicy(val_context_t *context, val_policy_handle_t *pol)
     for(q=ctx->q_list; q; q=q->qc_next) {
         /* Should never fail when holding above locks */
         if (NULL != namename(q->qc_name_n, p->zone_n)) {
-            clear_query_chain_structure(q);
+            q->qc_flags |= VAL_QUERY_REFRESH_QCACHE;
         }
     }
     
@@ -2687,8 +2755,7 @@ val_remove_valpolicy(val_context_t *context, val_policy_handle_t *pol)
     retval = VAL_NO_ERROR;
 
 err:
-    CTX_UNLOCK_ACACHE(ctx);
-    CTX_UNLOCK_VALPOL(ctx);
+    CTX_UNLOCK_POL(ctx);
     
     return retval;
 }
@@ -2697,24 +2764,20 @@ int
 val_is_local_trusted(val_context_t *context, int *trusted)
 {
     val_context_t *ctx = NULL;
-    int retval;
 
     if (trusted == NULL)
         return VAL_BAD_ARGUMENT;
 
-    if (context == NULL) {
-        /* Get the policy for the default context */
-        if (VAL_NO_ERROR != (retval = val_create_context(NULL, &ctx)))
-            return retval;
-    } else
-        ctx = (val_context_t *) context;
+    ctx = val_create_or_refresh_context(context); /* does CTX_LOCK_POL_SH */
+    if (ctx == NULL)
+        return VAL_INTERNAL_ERROR;
 
-    CTX_LOCK_VALPOL_SH(ctx);
     if (ctx && ctx->g_opt && ctx->g_opt->local_is_trusted)
         *trusted = 1;
     else
         *trusted = 0;
-    CTX_UNLOCK_VALPOL(ctx);
+
+    CTX_UNLOCK_POL(ctx);
 
     return VAL_NO_ERROR;    
 }
