@@ -20,24 +20,8 @@
  * See the COPYING file distributed with this software for details.
  */
 #include "validator-config.h"
+#include "validator-internal.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/time.h>
-#include <sys/types.h>
-
-#include <arpa/nameser.h>
-#ifdef HAVE_ARPA_NAMESER_COMPAT_H
-#include <arpa/nameser_compat.h>
-#else
-#include "arpa/header.h"
-#endif
-
-#include <validator/resolver.h>
-#include <validator/validator.h>
-#include <validator/validator-internal.h>
 #include "val_support.h"
 
 int
@@ -103,8 +87,8 @@ labelcmp(const u_char * name1, const u_char * name2, size_t label_cnt)
     size_t offset2 = 0;
     size_t i;
     
-    length1 = (int) name1 ? name1[0] : 0;
-    length2 = (int) name2 ? name2[0] : 0;
+    length1 = (int) (name1 ? name1[0] : 0);
+    length2 = (int) (name2 ? name2[0] : 0);
 
     if (length1 == 0 || length2 == 0)
         return length1 - length2;
@@ -372,29 +356,6 @@ wire_name_labels(const u_char * field)
         return l;
 }
 
-size_t
-wire_name_length(const u_char * field)
-{
-    /*
-     * Calculates the number of bytes in a DNS wire format name 
-     */
-    size_t j;
-    if (field == NULL)
-        return 0;
-
-    for (j = 0; field[j] && !(0xc0 & field[j]) && j < NS_MAXCDNAME;
-         j += field[j] + 1);
-    if (field[j])
-        j++;
-    j++;
-
-    if (j > NS_MAXCDNAME)
-        return 0;
-    else
-        return j;
-}
-
-
 void
 res_sq_free_rr_recs(struct val_rr_rec **rr)
 {
@@ -529,7 +490,7 @@ is_tail(u_char * full, u_char * tail)
         size_t        index = 0;
 
         while (index < (f_len - t_len)) {
-            index += (full[index]) + (u_int8_t) 1;
+            index += (full[index]) + (u_char) 1;
             if (index == f_len - t_len)
                 return TRUE;
         }
@@ -637,7 +598,8 @@ init_rr_set(struct rrset_rec *new_set, u_char * name_n,
             u_int16_t type_h, u_int16_t set_type_h,
             u_int16_t class_h, u_int32_t ttl_h,
             u_char * hptr, int from_section,
-            int authoritive_answer, struct name_server *respondent_server)
+            int authoritive_answer, int iterative_answer,
+            struct name_server *respondent_server)
 {
     size_t name_len = wire_name_length(name_n);
     struct timeval  tv;
@@ -688,8 +650,10 @@ init_rr_set(struct rrset_rec *new_set, u_char * name_n,
         memcpy(new_set->rrs_server,
                respondent_server->ns_address[0],
                sizeof(struct sockaddr_storage));
+        new_set->rrs_ns_options = respondent_server->ns_options;
     } else {
         new_set->rrs_server = NULL;
+        new_set->rrs_ns_options = 0;
     }
 
     new_set->rrs_next = NULL;
@@ -699,13 +663,16 @@ init_rr_set(struct rrset_rec *new_set, u_char * name_n,
      */
     if (from_section == VAL_FROM_ANSWER)
         new_set->rrs_cred = authoritive_answer ?
-            SR_CRED_AUTH_ANS : SR_CRED_NONAUTH_ANS;
+            SR_CRED_AUTH_ANS : 
+                (iterative_answer ? SR_CRED_ITER_ANS : SR_CRED_NONAUTH_ANS);
     else if (from_section == VAL_FROM_AUTHORITY)
         new_set->rrs_cred = authoritive_answer ?
-            SR_CRED_AUTH_AUTH : SR_CRED_NONAUTH_AUTH;
+            SR_CRED_AUTH_AUTH : 
+                (iterative_answer ? SR_CRED_ITER_AUTH : SR_CRED_NONAUTH_AUTH);
     else if (from_section == VAL_FROM_ADDITIONAL)
         new_set->rrs_cred = authoritive_answer ?
-            SR_CRED_AUTH_ADD : SR_CRED_NONAUTH_ADD;
+            SR_CRED_AUTH_ADD : 
+                (iterative_answer ? SR_CRED_ITER_ADD : SR_CRED_NONAUTH_ADD);
     else
         new_set->rrs_cred = SR_CRED_UNSET;
 
@@ -768,12 +735,13 @@ find_rr_set(struct name_server *respondent_server,
             u_char * rdata_n,
             int from_section, 
             int authoritive_answer, 
+            int iterative_answer, 
             u_char * zonecut_n)
 {
     struct rrset_rec *tryit;
     struct rrset_rec *last;
     struct rrset_rec *new_one;
-    size_t             name_len = wire_name_length(name_n);
+    size_t             name_len;
 
     if ((the_list == NULL) || (name_n == NULL))
         return NULL;
@@ -783,6 +751,7 @@ find_rr_set(struct name_server *respondent_server,
      */
     tryit = *the_list;
     last = NULL;
+    name_len = wire_name_length(name_n);
 
     while (tryit) {
         /*
@@ -834,7 +803,8 @@ find_rr_set(struct name_server *respondent_server,
 
         if ((init_rr_set(new_one, name_n, type_h, set_type_h,
                          class_h, ttl_h, hptr, from_section,
-                         authoritive_answer, respondent_server))
+                         authoritive_answer, iterative_answer, 
+                         respondent_server))
             != VAL_NO_ERROR) {
             res_sq_free_rrset_recs(the_list);
             return NULL;
@@ -1424,8 +1394,6 @@ copy_rr_rec_list(u_int16_t type_h, struct val_rr_rec *o_rr, int dolower)
     return n_head;
 }
 
-#define INSERTED    1
-#define DUPLICATE   -1
 int
 link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
 {
@@ -1442,7 +1410,7 @@ link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
 
     if (*cs == NULL) {
         *cs = cr;
-        return INSERTED;
+        return 1;
     } else {
         length = (*cs)->rr_rdata_length < cr->rr_rdata_length ?
             (*cs)->rr_rdata_length : cr->rr_rdata_length;
@@ -1458,19 +1426,19 @@ link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
                 cr->rr_next = NULL;
             }
             res_sq_free_rr_recs(&cr);
-            return DUPLICATE;
+            return -1;
         } else if (ret_val > 0
                    || (ret_val == 0 && length == cr->rr_rdata_length)) {
             cr->rr_next = *cs;
             *cs = cr;
-            return INSERTED;
+            return 1;
         } else {
             temp_rr = *cs;
 
             if (temp_rr->rr_next == NULL) {
                 temp_rr->rr_next = cr;
                 cr->rr_next = NULL;
-                return INSERTED;
+                return 1;
             }
             while (temp_rr->rr_next) {
                 length = temp_rr->rr_next->rr_rdata_length <
@@ -1490,7 +1458,7 @@ link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
                         cr->rr_next = NULL;
                     }
                     res_sq_free_rr_recs(&cr);
-                    return DUPLICATE;
+                    return -1;
                 } else if (ret_val > 0
                            || (ret_val == 0
                                && length == cr->rr_rdata_length)) {
@@ -1499,7 +1467,7 @@ link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
                      */
                     cr->rr_next = temp_rr->rr_next;
                     temp_rr->rr_next = cr;
-                    return INSERTED;
+                    return 1;
                 }
                 temp_rr = temp_rr->rr_next;
             }
@@ -1510,7 +1478,7 @@ link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr)
 
             temp_rr->rr_next = cr;
             cr->rr_next = NULL;
-            return INSERTED;
+            return 1;
         }
     }
 }
@@ -1625,8 +1593,10 @@ copy_rrset_rec(struct rrset_rec *rr_set)
         }
         memcpy(copy_set->rrs_server, rr_set->rrs_server,
                sizeof(struct sockaddr_storage));
+        copy_set->rrs_ns_options = rr_set->rrs_ns_options;
     } else {
         copy_set->rrs_server = NULL;
+        copy_set->rrs_ns_options = 0;
     }
 
     return copy_set;
@@ -1659,6 +1629,41 @@ copy_rrset_rec_list(struct rrset_rec *rr_set)
     return copy_set;
 }
 
+struct rrset_rec *
+copy_rrset_rec_list_in_zonecut(struct rrset_rec *rr_set, u_char *qname_n) 
+{
+    struct rrset_rec *copy_set, *cur_set, *prev_set, *new_set;
+
+    copy_set = cur_set = prev_set = new_set = NULL;
+
+    if (qname_n == NULL)
+        return NULL;
+
+    for (cur_set = rr_set; cur_set; cur_set=cur_set->rrs_next) {
+
+        /* 
+         * if the zonecut exists, check if it is within the query name 
+         * it is okay for the zonecut to be NULL 
+         */
+        if(cur_set->rrs_zonecut_n && !namename(qname_n, cur_set->rrs_zonecut_n)) {
+            continue;
+        }
+
+        new_set = copy_rrset_rec(cur_set);
+        if (!new_set) {
+            res_sq_free_rrset_recs(&copy_set);
+            return NULL;
+        }    
+        if (prev_set) {
+            prev_set->rrs_next = new_set;
+        } else {
+            copy_set = new_set;
+        }
+        prev_set = new_set;
+    }
+    return copy_set;
+}
+
 /*
  *
  * returns
